import React from 'react';
import Candidates from './Candidates.jsx';
import WineEditor from './WineEditor.jsx';
import ClickableWord from './ClickableWord.jsx';
import ExternalResources from './ExternalResources.jsx';
import * as Constants from '../constants';
import {CleanUpWord, WineDescription} from "../helpers/helper";
import ProducerEditor from "./ProducerEditor";

class Addition extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
            allData: [],
            data: {
                mongoId: "",
                wineVinId: "",
                regionCode: "",
                vintage: "",
                variety: "",
                label: "",
                producerName: "",
                producerShow: "",
                country: "",
                region: "",
                location: "",
                locale: "",
                site: "",
                sweetness: "",
                color: "",
                wineType: "",
                acceptedMatch: {
                    id: "",
                    name: "",
                    color: ""
                }
            },
            currentIndex: -1,
            narrowKeywords: [],
            resetFilterID: false,
            selectedMatch: {
                id: "",
                name: "",
                color: ""
            },
            isLoadingMatch: false,
            isCheckingMatched: false,
            isEditingWine: false,
            isUpdatingMongoMap: false,
            wineDbId: "", // WineDbId to be edited
            wine: {}, // Wine to be edited
            isEditingProducer: false,
            producerId: "" // ProducerId to be edited
		};
        this.wineIdFilterClearedTimer = null;

        this.beforeUnload = this.beforeUnload.bind(this);
        this.handleInputChange = this.handleInputChange.bind(this);

        this.uploadFile = this.uploadFile.bind(this);

        this.selectMatch = this.selectMatch.bind(this);
        this.acceptSavedMatch = this.acceptSavedMatch.bind(this);
        this.resetMatch = this.resetMatch.bind(this);
        this.acceptMatch = this.acceptMatch.bind(this);
        this.undoMatch = this.undoMatch.bind(this);
        this.checkAdditionMatched = this.checkAdditionMatched.bind(this);
        this.updateMongoMap = this.updateMongoMap.bind(this);
        this.undoMongoMap = this.undoMongoMap.bind(this);

        this.editWineDb = this.editWineDb.bind(this);
        this.createWineNewProducer = this.createWineNewProducer.bind(this);
        this.createWine = this.createWine.bind(this);
        this.cancelEditWine = this.cancelEditWine.bind(this);

        this.editProducer = this.editProducer.bind(this);
        this.cancelEditProducer = this.cancelEditProducer.bind(this);

        this.nextRecord = this.nextRecord.bind(this);
        this.prevRecord = this.prevRecord.bind(this);

        this.addNarrowWord = this.addNarrowWord.bind(this);
        this.removeNarrowWord = this.removeNarrowWord.bind(this);
        this.refreshCandidates = this.refreshCandidates.bind(this);

        this.onWineIdFilterCleared = this.onWineIdFilterCleared.bind(this);
	}

    componentWillMount() {
    }

	componentDidMount() {
        window.addEventListener("beforeunload", this.beforeUnload);
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillUnmount() {
        window.removeEventListener("beforeunload", this.beforeUnload);
    }

    beforeUnload(e) {
        if (this.state.allData && this.state.allData.length > 0 && this.state.allData.length > this.state.currentIndex + 1) {
            e.preventDefault();
            e.returnValue = true;
        }
    }

    handleInputChange(event) {
        let _this = this;
        const target = event.target;
        const name = target.name;
        let value = target.type === 'checkbox' ? target.checked : target.value;
        _this.setState({
            [name]: value
        });
    }

    nextRecord() {
        let _this = this;
        let next = _this.state.currentIndex + 1;
        if (_this.state.allData.length >= next + 1) {
            _this.setState({
                data: _this.state.allData[next],
                narrowKeywords: [],
                resetFilterID: true,
                currentIndex: next,
                selectedMatch: {
                    id: "",
                    name: "",
                    color: ""
                }
            }, function () {
                _this.checkAdditionMatched();
            });
        } else {
            alert("This is the last record.");
        }
    }

    prevRecord() {
        let _this = this;
        let prev = _this.state.currentIndex - 1;
        if (prev >= 0) {
            _this.setState({
                data: _this.state.allData[prev],
                narrowKeywords: [],
                resetFilterID: true,
                currentIndex: prev,
                selectedMatch: {
                    id: "",
                    name: "",
                    color: ""
                }
            }, function () {
                _this.checkAdditionMatched();
            });
        } else {
            alert("This is the first record.");
        }
    }

    addNarrowWord(word) {
        if (word) {
            let n = this.state.narrowKeywords;
            if (n.indexOf(word) === -1) {
                n.push(word);
                this.setState({
                    narrowKeywords: n,
                    selectedMatch: {
                        id: "",
                        name: "",
                        color: ""
                    }
                });
            }
        }
    }

    removeNarrowWord(word) {
        word = CleanUpWord(word);
        if (word) {
            let n = this.state.narrowKeywords;
            let index = n.indexOf(word);
            if (index > -1) {
                n.splice(index, 1);
                this.setState({
                    narrowKeywords: n,
                    resetFilterID: n && n.length < 1,
                    selectedMatch: {
                        id: "",
                        name: "",
                        color: ""
                    }
                });
            }
        }
    }

    refreshCandidates() {
        let _this = this;
        let words = _this.state.narrowKeywords;
        _this.setState({
            wineDbId: "",
            isEditingWine: false,
            producerId: "",
            isEditingProducer: false,
            narrowKeywords: [],
            resetFilterID: true
        }, function () {
            _this.setState({narrowKeywords: words});
        });
    }

    selectMatch(selectedMatch) {
        let selectedClone = Object.assign({}, selectedMatch);
        this.setState({selectedMatch: selectedClone});
    }

    resetMatch() {
        this.setState({
            selectedMatch: {
                id: "",
                name: "",
                color: ""
            }
        });
    }

    acceptSavedMatch() {
        this.setState({wine: {}, isEditingWine: false}, this.nextRecord);
    }

    acceptMatch() {
        let _this = this;
        _this.setState({isUpdatingMongoMap: true}, function () {
            let formData = new FormData();
            formData.append('wineDbId', _this.state.selectedMatch.id);
            formData.append('mongoId', _this.state.data.mongoId);
            formData.append('wineVinId', _this.state.data.wineVinId);
            formData.append('vintage', _this.state.data.vintage);
            formData.append('source', Constants.SOURCE.WINE_ADDITION);
            _this.updateMongoMap(formData);
        });
    }

    undoMatch() {
        let _this = this;
        _this.setState({isUpdatingMongoMap: true}, function () {
            let formData = new FormData();
            formData.append('id', _this.state.data.acceptedMatch.id);
            formData.append('mongoId', _this.state.data.mongoId);
            _this.undoMongoMap(formData);
        });
    }

    createWineNewProducer() {
        this.createWine();
    }

    createWine(producerId) {
        let wine = Object.assign({}, this.state.data);
	    if (producerId) wine.producerId = producerId;
        this.setState({wine: wine, isEditingWine: true});
    }

    cancelEditWine() {
        this.setState({wine: {}, wineDbId: "", isEditingWine: false});
    }

    editWineDb(wineDbId) {
        if (!wineDbId) return;
        this.setState({wineDbId: wineDbId, isEditingWine: true});
    }

    editProducer(producerId) {
        if (!producerId) return;
        this.setState({producerId: producerId, isEditingProducer: true});
    }

    cancelEditProducer() {
        this.setState({producerId: "", isEditingProducer: false});
    }

    onWineIdFilterCleared() {
        clearTimeout(this.wineIdFilterClearedTimer);
        this.wineIdFilterClearedTimer = setTimeout(function() {
            this.setState((state) => {
                return {resetFilterID: false};
            });
        }.bind(this), 2);
    }

    async uploadFile(event) {
        event.preventDefault();
        let _this = this;
        _this.setState({currentIndex: -1, isLoadingMatch: true});
        let input = document.querySelector('input[type="file"]');

        let data = new FormData();
        data.append('file', input.files[0]);

        fetch(Constants.UPLOAD_WINES_ADDITION_URL, {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                if (json.data.length > 0) {
                    let data = json.data;
                    let i;
                    for (i in data) {
                        data[i].acceptedMatch = {
                            id: "",
                            name: "",
                            color: ""
                        }
                    }
                    _this.setState({allData: json.data, isLoadingMatch: false}, function () {
                        _this.nextRecord();
                    });
                } else {
                    throw new Error("No record found.");
                }
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isLoadingMatch: false});
        });
    }

    async checkAdditionMatched() {
        let _this = this;
        _this.setState({isCheckingMatched: true});
        fetch(Constants.CHECK_ADDITION_MATCHED + "?mongoId=" + _this.state.data.mongoId, {
            method: 'GET',
            headers: {
                'Accept': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                let data = _this.state.data;
                data.acceptedMatch = json.data;
                _this.setState({data: data, isCheckingMatched: false});
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isCheckingMatched: false});
        });
    }

    async updateMongoMap(formData) {
        let _this = this;
        fetch(Constants.UPDATE_MONGO_MAP, {
            method: 'POST',
            headers: {
                'Accept': 'application/json'
            },
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                let data = _this.state.data;
                data.acceptedMatch = json.data;
                _this.setState({data: data, isUpdatingMongoMap: false}, function () {
                    _this.nextRecord();
                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isUpdatingMongoMap: false});
        });
    }

    async undoMongoMap(formData) {
        let _this = this;
        fetch(Constants.UNDO_MONGO_MAP_URL, {
            method: 'POST',
            headers: {
                'Accept': 'application/json'
            },
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                let data = _this.state.data;
                data.acceptedMatch = {
                    id: "",
                    name: "",
                    color: ""
                };
                _this.setState({data: data, isUpdatingMongoMap: false}, function () {

                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isUpdatingMongoMap: false});
        });
    }

    render() {
		return (
			<div className="row">
				<div className="col-xs-12" hidden={this.state.isEditingWine || this.state.isEditingProducer}>
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <h2 className="title">Wine Addition - External Input</h2>
                        </div>
                    </div>
                    <div className="row wine-match-row retailer-form">
                        <div className="col-xs-10">
                            <form className="form-inline" onSubmit={this.uploadFile}>
                                <div className="row row-form">
                                    <div className="col-md-1">
                                        <label className="control-label">File</label>
                                    </div>
                                    <div className="col-md-4 text-right">
                                        <input type="file" name="file" className="form-control" onChange={this.handleInputChange}/>
                                    </div>
                                    <div className="col-md-7 text-left">
                                        <button type="submit" className="btn btn-default" disabled={this.state.isLoadingMatch}>{this.state.isLoadingMatch ? 'Loading...' : 'Load Data'}</button>
                                    </div>
                                </div>
                            </form>
                        </div>
                        <div className="col-xs-2 text-right">
                            <span><strong>{this.state.allData.length > 0 ? this.state.currentIndex + 1 : 0} of {this.state.allData.length}</strong></span>
                        </div>
                    </div>
                    <div className="row wine-match-row">
                        <table className="table table-bordered">
                            <tbody>
                            <tr>
                                <td style={{width: '15%'}}><b>Mongo ID</b></td>
                                <td colSpan={3}>{this.state.data.mongoId}</td>
                            </tr>
                            <tr>
                                <td style={{width: '15%'}}><b>Label Name</b></td>
                                <td colSpan={3}>
                                    <ClickableWord phrase={this.state.data.label} onClick={this.addNarrowWord} />
                                </td>
                            </tr>
                            <tr>
                                <td style={{width: '15%'}}><b>Producer Name</b></td>
                                <td><ClickableWord phrase={this.state.data.producerName} onClick={this.addNarrowWord} /></td>
                                <td style={{width: '15%'}}><b>Producer Show</b></td>
                                <td><ClickableWord phrase={this.state.data.producerShow} onClick={this.addNarrowWord} /></td>
                            </tr>
                            <tr>
                                <td colSpan={4}>
                                    <div className="row">
                                        <div className="col-md-2"><b>Country</b><br /><ClickableWord phrase={this.state.data.country} onClick={this.addNarrowWord} /></div>
                                        <div className="col-md-2"><b>Region Code</b><br />{this.state.data.regionCode}</div>
                                        <div className="col-md-2"><b>Region</b><br /><ClickableWord phrase={this.state.data.region} onClick={this.addNarrowWord} /></div>
                                        <div className="col-md-2"><b>Sub-Region</b><br /><ClickableWord phrase={this.state.data.location} onClick={this.addNarrowWord} /></div>
                                        <div className="col-md-2"><b>Appellation</b><br /><ClickableWord phrase={this.state.data.locale} onClick={this.addNarrowWord} /></div>
                                        <div className="col-md-2"><b>Sub-Appellation</b><br /><ClickableWord phrase={this.state.data.site} onClick={this.addNarrowWord} /></div>
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td colSpan={4}>
                                    <div className="row">
                                        <div className="col-md-2"><b>Wine Vin ID</b><br />{this.state.data.wineVinId}</div>
                                        <div className="col-md-2"><b>Vintage</b><br />{this.state.data.vintage}</div>
                                        <div className="col-md-2"><b>Variety</b><br />{this.state.data.variety}</div>
                                        <div className="col-md-2"><b>Dryness</b><br />{this.state.data.sweetness}</div>
                                        <div className="col-md-2"><b>Color Class</b><br />{this.state.data.color}</div>
                                        <div className="col-md-2"><b>Type</b><br />{this.state.data.wineType}</div>
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td style={{width: '15%'}}><b>Matched Wine</b></td>
                                <td colSpan={3}>{this.state.data.acceptedMatch ? this.state.data.acceptedMatch.id + " - " + WineDescription(this.state.data.acceptedMatch.name, this.state.data.acceptedMatch.color) : ""}</td>
                            </tr>
                            </tbody>
                        </table>
                    </div>
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <ul className="list-inline btn-list">
                                <li><button className="btn btn-md btn-default" onClick={this.createWineNewProducer} disabled={this.state.currentIndex < 0}>Add Wine New Producer</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.currentIndex < 1 || this.state.isCheckingMatched}
                                            onClick={this.prevRecord}>Back</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || this.state.allData.length === this.state.currentIndex + 1 || this.state.isCheckingMatched}
                                            onClick={this.nextRecord}>Next (without match)</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || !this.state.selectedMatch.id || this.state.data.acceptedMatch.id}
                                            onClick={this.acceptMatch}>Accept Match</button></li>
                                <li><button className="btn btn-md btn-default" disabled={!this.state.selectedMatch.id}
                                            onClick={this.resetMatch}>Reset Match</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.data.acceptedMatch && this.state.data.acceptedMatch.id.length < 1}
                                            onClick={this.undoMatch}>Undo Match</button></li>
                            </ul>
                        </div>
                    </div>
                    <Candidates words={this.state.narrowKeywords} selectedMatch={this.state.selectedMatch} selectMatch={this.selectMatch}
                                addWord={this.addNarrowWord} removeWord={this.removeNarrowWord}
                                createWine={this.createWine} editWineDb={this.editWineDb} editProducer={this.editProducer}
                                resetFilterID={this.state.resetFilterID} onWineIdFilterCleared={this.onWineIdFilterCleared}
                    />
                    <ExternalResources wine={this.state.data} wineBaseUrl={Constants.RP_WSF_BASE_URL} hideRetailerLink={true} />
				</div>
                <div className="col-xs-12">
                    <WineEditor shouldShow={this.state.isEditingWine} wineDbId={this.state.wineDbId}
                                producerId={this.state.wine.producerId}
                                wine={this.state.wine} source={Constants.SOURCE.WINE_ADDITION}
                                onUpdated={this.refreshCandidates} onSaved={this.acceptSavedMatch} onCanceled={this.cancelEditWine}
                                defaultNoiseMode={true}
                    />
                </div>
                <div className="col-xs-12">
                    <ProducerEditor shouldShow={this.state.isEditingProducer} producerId={this.state.producerId} source={Constants.SOURCE.NARROW}
                                    onUpdated={this.refreshCandidates} onCanceled={this.cancelEditProducer} />
                </div>
			</div>
		);
	}
}

export default Addition;