import React, { Component } from 'react';
import Merge from './Merge.jsx';
import * as Constants from '../constants';
import ClickableWord from "./ClickableWord";
import {WineDescription} from "../helpers/helper";

class Candidates extends Component {
	constructor() {
		super();
		this.state = {
            fileSource: Constants.DATA_SOURCES.INTERNAL,
            filterID: "",
            words: [],
            producers: [],
            wineCandidates: [],
            selectedProducers: [],
            selectedWines: [],
            producerRequests: 0,
            candidateRequests: 0,
            producerController: new AbortController(),
            candidateController: new AbortController(),
            searchPartialProducer: true,
            searchPartial: true,
		};

        this.getProducers = this.getProducers.bind(this);
        this.getWineCandidates = this.getWineCandidates.bind(this);
        this.refreshCandidates = this.refreshCandidates.bind(this);

        this.handleInputChange = this.handleInputChange.bind(this);
        this.handleKeyPress = this.handleKeyPress.bind(this);
        this.selectWine = this.selectWine.bind(this);

        this.editWineDb = this.editWineDb.bind(this);
        this.createWine = this.createWine.bind(this);

        this.editProducer = this.editProducer.bind(this);

        this.removeWord = this.removeWord.bind(this);

        this.listNarrowKeywords = this.listNarrowKeywords.bind(this);
        this.listWines = this.listWines.bind(this);
        this.selectMatch = this.selectMatch.bind(this);

        this.listProducers = this.listProducers.bind(this);
        this.selectProducer = this.selectProducer.bind(this);
        this.listSelectedProducers = this.listSelectedProducers.bind(this);
        this.removeSelectedProducer = this.removeSelectedProducer.bind(this);
	}

    componentDidMount() {
    }

    componentDidUpdate(prevProps, prevState) {
        if (this.props.hasOwnProperty('isDisabled') && this.props.isDisabled) return;
        let stateMap = {};
        if (this.props.words && this.props.words.length !== this.state.words.length) {
            stateMap.words = JSON.parse(JSON.stringify(this.props.words));
            stateMap.selectedProducers = this.props.words.length === 0 ? [] : this.state.selectedProducers;
        }
        let shouldResetFilterID = this.props.resetFilterID && this.props.resetFilterID !== prevProps.resetFilterID;
        if (shouldResetFilterID) stateMap.filterID = '';

        if (Object.keys(stateMap).length > 0) {
            let _this = this;
            _this.setState(stateMap, function () {
                if (shouldResetFilterID) {
                    _this.inputFilterID.value = '';
                    _this.props.onWineIdFilterCleared();
                }
                _this.refreshCandidates();
            });
        }
    }

    handleInputChange(event) {
        const target = event.target;
        const name = target.name;

        let value = target.type === 'checkbox' ? target.checked : target.value;

        if (name === 'date') {
            let date = target.value;
            value = date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
        }

        let _this = this;
        _this.setState({
            [name]: value
        }, function () {
            if (name === 'searchPartial' || name === 'searchPartialProducer' || name === 'filterID') {
                _this.refreshCandidates();
            }
        });
    }

    handleKeyPress(event) {
        if (event.keyCode===13) {
            this.props.addWord(event.target.value);
            event.target.value = "";
        }
    }

    refreshCandidates() {
	    let _this = this;

        if (_this.state.producerRequests > 0){
            _this.state.producerController.abort();
            _this.state.producerController = new AbortController();
            _this.state.producerRequests--;
        }

	    if (_this.state.candidateRequests > 0){
	        _this.state.candidateController.abort();
	        _this.state.candidateController = new AbortController();
	        _this.state.candidateRequests--;
        }

        _this.getProducers();
        _this.getWineCandidates();
    }

    async getProducers() {
        let _this = this;
        let producers = _this.state.selectedProducers;

        let words = _this.state.words;
        if (words.length < 1) {
            _this.setState({producers: []});
            return;
        }
        _this.setState({producerRequests: _this.state.producerRequests++});

        let filterID = _this.state.filterID && _this.state.filterID.length > 5 ? _this.state.filterID.substr(0, 6) : _this.state.filterID;
        fetch(Constants.PRODUCERS_URL, {
            signal: _this.state.producerController.signal,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                filterID: filterID,
                words: words,
                producers: producers,
                searchPartial: _this.state.searchPartialProducer
            }),
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({
                producerRequests: _this.state.producerRequests--,
                producers: json.data
            });
        }).catch(function(ex) {
            _this.setState({producers: []});
            if (ex.name === 'AbortError'){
                console.log("Producers fetch aborted");
                return;
            }
            console.log('Parsing data failed!', ex);
        });
    }

    async getWineCandidates() {
        let _this = this;

        let words = _this.state.words;
        if (words.length < 1) {
            _this.setState({wineCandidates: [], selectedWines: []});
            return;
        }
        _this.setState({candidateRequests: _this.state.candidateRequests++});

        let producers = _this.state.selectedProducers;
        fetch(Constants.WINE_DBS_URL, {
            signal: _this.state.candidateController.signal,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                filterID: _this.state.filterID,
                words: words,
                producers: producers,
                searchPartial: _this.state.searchPartial
            }),
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({
                candidateRequests: _this.state.candidateRequests--,
                wineCandidates: json.data,
                selectedWines: []});
        }).catch(function(ex) {
            _this.setState({wineCandidates: [], selectedWines: []});
            if (ex.name === 'AbortError'){
                console.log("Candidates fetch aborted");
                return;
            }
            console.log('Parsing data failed!', ex);
        });
    }

    listNarrowKeywords() {
        if (!this.props.words || this.props.words.length < 1) return "";

        let _this = this;
        return this.props.words.map(function(row, i) {
            return (
                <div className="row" key={i}>
                    <div className="col-xs-12">
                        <span className="remove" onClick={_this.removeWord} data-id={row}><i className="glyphicon glyphicon-remove" />{row}</span>
                    </div>
                </div>
            );
        });
    }

    removeWord(e) {
        if (!e.currentTarget) return;

        let word = e.currentTarget.getAttribute('data-id');
        if (word) {
            this.props.removeWord(word);
        }
    }

    removeSelectedProducer(e) {
        if (!e.currentTarget) return;

        let producerId = e.currentTarget.getAttribute('data-id');
        if (producerId) {
            let sp = this.state.selectedProducers;
            let index = sp.indexOf(producerId);
            if (index > -1) {
                let _this = this;
                sp.splice(index, 1);
                _this.setState({selectedProducers: sp}, function () {
                    _this.refreshCandidates();
                });
            }
        }
    }

    selectMatch(e) {
        if (!e.currentTarget) return;

        let wid = e.currentTarget.textContent;
        if (wid) {
            wid = wid.trim();
            let i;
            for (i in this.state.wineCandidates) {
                let w = this.state.wineCandidates[i];
                if (w.id === wid) {
                    if (typeof this.props.selectMatch === 'function') {
                        this.props.selectMatch({
                            id: w.id,
                            name: w.name,
                            color: w.color,
                            pattern: w.pattern,
                            variety: w.variety,
                            waNote: w.waNote,
                            country: w.country,
                            region: w.region,
                            location: w.location,
                            locale: w.locale,
                            site: w.site
                        });
                    }
                    break;
                }
            }
        }
    }

    selectWine(e) {
        if (!e.currentTarget) return;

        let wid = e.currentTarget.value;
        if (wid) {
            let selectedWines = this.state.selectedWines;
            let index = selectedWines.indexOf(wid);
            if (index > -1) {
                selectedWines.splice(index, 1);
                this.setState({
                    selectedWines: selectedWines
                });
            } else {
                if (selectedWines.length < 3) {
                    selectedWines.push(wid);
                    this.setState({
                        selectedWines: selectedWines
                    });
                }
            }
        }
    }

    editWineDb(e) {
        if (!e.currentTarget) return;

        let wineDbId = e.currentTarget.getAttribute('data-id');
        this.props.editWineDb(wineDbId);
    }

    createWine(e) {
        if (!e.currentTarget) return;

        let producerId = e.currentTarget.getAttribute('data-id');
        this.props.createWine(producerId);
    }

    editProducer(e) {
        if (!e.currentTarget) return;

        let producerId = e.currentTarget.getAttribute('data-id');
        this.props.editProducer(producerId);
    }

    selectProducer(e) {
        if (!e.currentTarget) return;

        let _this = this;
        let word = e.currentTarget.textContent.trim();
        if (word) {
            let sp = _this.state.selectedProducers;
            let index = sp.indexOf(word);
            if (index === -1) {
                sp.push(word);
                _this.setState({selectedProducers: sp}, function () {
                    _this.refreshCandidates();
                });
            }
        }
    }

    listProducers() {
        if (!this.state.producers || this.state.producers.length < 1) return "";

        let _this = this;
        return _this.state.producers.map(function(row, i) {
            return (
                <div className="row row-striped" key={i}>
                    <div className="col-xs-3"><i onClick={_this.editProducer} data-id={row.id} className="glyphicon glyphicon-pencil" />
                        <span onClick={_this.selectProducer} className="clickable-word">{row.id}</span>
                    </div>
                    <div className="col-xs-9">
                        <ClickableWord phrase={row.name} onClick={_this.props.addWord} />
                        <span className="pull-right">
                            <button className="btn btn-xs btn-primary" onClick={_this.createWine} data-id={row.id}><i className="glyphicon glyphicon-plus" /> Wine</button>
                        </span>
                    </div>
                </div>
            );
        });
    }

    listWines() {
        if (!this.state.wineCandidates || this.state.wineCandidates.length < 1) return "";

        let _this = this;
        return this.state.wineCandidates.map(function(row, i) {
            let description = WineDescription(row.name, row.color);
            return (
                <div className="row row-striped" key={i}>
                    <div className="col-xs-2">
                        <span><i onClick={_this.editWineDb} data-id={row.id} className="glyphicon glyphicon-pencil" /></span>&nbsp;
                        <span onClick={_this.selectMatch} className={"clickable-word" + (_this.props.selectedMatch && _this.props.selectedMatch.id === row.id ? " label label-primary selected-wine" : "")}>{row.id}</span>
                    </div>
                    <div className="col-xs-9"><ClickableWord phrase={description} onClick={_this.props.addWord} /></div>
                    <div className="col-xs-1">
                        <input type="checkbox" name="mergeId" value={row.id} onClick={_this.selectWine}
                               checked={_this.state.selectedWines && _this.state.selectedWines.indexOf(row.id) > -1} readOnly={true} />
                    </div>
                </div>
            );
        });
    }

    listSelectedProducers() {
        if (!this.state.selectedProducers || this.state.selectedProducers.length < 1) return "-";

        let _this = this;
        return this.state.selectedProducers.map(function(row, i) {
            return (
                <button onClick={_this.removeSelectedProducer} className="btn btn-primary btn-xs selected-producer" key={i} data-id={row}>{row}</button>
            );
        });
    }

	render() {
		return (
            <div className="row wine-match-row">
                <div className="col-xs-5">
                    <div className="row">
                        <div className="col-md-12 word-list">
                            <div className="row data-list-header">
                                <div className="col-md-6">
                                    <div className="row">
                                        <div className="col-md-4"><strong>Keywords</strong></div>
                                        <div className="col-md-8 text-right new-narrow-keyword">
                                            <input type="text" name="newNarrowKeyword" onKeyDown={this.handleKeyPress} />
                                        </div>
                                    </div>
                                </div>
                                <div className="col-md-6">
                                    <div className="row">
                                        <div className="col-md-4"><strong>ID</strong></div>
                                        <div className="col-md-8 text-right new-narrow-keyword">
                                            <input type="text" name="filterID" onChange={this.handleInputChange} ref={el => this.inputFilterID = el}/>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12 narrow-keywords">
                                    {this.listNarrowKeywords()}
                                </div>
                            </div>
                        </div>
                    </div>
                    <div className="row">
                        <div className="col-md-12 producer-list">
                            <div className="row data-list-header">
                                <div className="col-md-4"><strong>Producers</strong></div>
                                <div className="col-md-8 text-right">
                                    <div className="form-group">
                                        <div className="pull-right">
                                            <label className="control-label">Search partial word</label>
                                            <input type="checkbox" name="searchPartialProducer"
                                                   onChange={this.handleInputChange}
                                                   checked={this.state.searchPartialProducer}/>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div className="row data-list-header">
                                <div className="col-md-8">
                                    Locked: {this.listSelectedProducers()}
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12 found-producers">
                                    {this.listProducers()}
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
                <div className="col-xs-7 match-list">
                    <div className="row data-list-header">
                        <div className="col-md-12">
                            <div className="row">
                                <div className="col-xs-6">
                                    <strong>Matches</strong>
                                </div>
                                <div className="col-xs-6">
                                    <div className="form-group">
                                        <div className="pull-right">
                                            <Merge wines={this.state.selectedWines} callback={this.refreshCandidates} />
                                        </div>
                                        <div className="pull-right">
                                            <label className="control-label">Search partial word</label>
                                            <input type="checkbox" name="searchPartial" onChange={this.handleInputChange} checked={this.state.searchPartial} />
                                            &nbsp;&nbsp;
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div className="row">
                        <div className="col-md-12 found-candidates">
                            {this.listWines()}
                        </div>
                    </div>
                </div>
            </div>
		);
	}
}

export default Candidates;