import React from 'react';
import * as Constants from "../constants";

class ExternalResources extends React.Component {
    constructor() {
        super();
        this.state = {
            note: "",
            isEditingNote: false,
            isSavingNote: false
        };

        this.getRPNoteUrl = this.getRPNoteUrl.bind(this);

        this.handleInputChange = this.handleInputChange.bind(this);

        this.editNote = this.editNote.bind(this);
        this.saveNote = this.saveNote.bind(this);
        this.updateNote = this.updateNote.bind(this);
        this.cancelNote = this.cancelNote.bind(this);
    }

    handleInputChange(event) {
        const target = event.target;
        const name = target.name;

        let value = target.type === 'checkbox' ? target.checked : target.value;

        if (name === 'date') {
            let date = target.value;
            value = date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
        }

        let stateData = this.state;
        stateData[name] = value;
        this.setState(stateData);
    }

    componentWillReceiveProps(nextProps) {
        if (this.state.isEditingNote) return;
        let wine = JSON.parse(JSON.stringify(nextProps.wine));
        if (wine) {
            if (wine.note !== this.state.note) {
                this.setState({note: wine.note ? wine.note : ""});
            }
        }
    }

    editNote() {
        this.setState({isEditingNote: !this.state.isEditingNote});
    }

    saveNote() {
        let _this = this;
        if (!_this.state.note) return;

        let data = new FormData();
        data.append('wineDbId', this.state.data.wineDbId);
        data.append('note', _this.state.note.trim());

        _this.updateNote(data);
    }

    async updateNote(formData) {
        let _this = this;
        _this.setState({isSavingNote: true});
        fetch(Constants.UPDATE_WA_NOTE, {
            method: 'POST',
            headers: {
                'Accept': 'application/json'
            },
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                _this.setState({isSavingNote: false}, function () {
                    _this.props.onSavedNote(_this.state.note);
                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isSavingNote: false});
        });
    }

    cancelNote() {
        let _this = this;
        let wine = JSON.parse(JSON.stringify(_this.props.wine));
        if (wine) {
            _this.setState({note: wine.note, isEditingNote: false, isSavingNote: false});
        }
    }

    getAppellation(data) {
        let appellation = "";
        if (data.country && data.country !== "") {
            appellation = data.country;
        }
        if (data.region && data.region !== "") {
            if (appellation && appellation !== "") appellation += ", ";
            appellation += data.region;
        }
        if (data.location && data.location !== "") {
            if (appellation && appellation !== "") appellation += ", ";
            appellation += data.location;
        }
        if (data.locale && data.locale !== "") {
            if (appellation && appellation !== "") appellation += ", ";
            appellation += data.locale;
        }
        if (data.site && data.site !== "") {
            if (appellation && appellation !== "") appellation += ", ";
            appellation +=  data.site;
        }
        return appellation;
    }

    getRPNoteUrl() {
        if (!this.props.wine || !this.props.wine.mongoId) return "";
        let base = this.props.wineBaseUrl ? this.props.wineBaseUrl : Constants.RP_PROD_BASE_URL;
        return base + this.props.wine.mongoId;
    }

    getRetailerUrl(data) {
        if (!data) return "";
        return data.url ? data.url : data.retailerUrl
    }

    render() {
        return (
            <div className="row wine-match-row">
                <table className="table table-bordered">
                    <thead className="thead-default">
                    <tr>
                        <th className="default" colSpan="2">External Resources</th>
                    </tr>
                    </thead>
                    <tbody>
                    <tr hidden={typeof this.props.hideRetailerLink !== 'undefined' || this.props.hideRetailerLink}>
                        <td width="15%">Retailer URL</td>
                        <td>
                            <a href={this.getRetailerUrl(this.props.wine)} target="_blank">
                                {this.getRetailerUrl(this.props.wine)}
                            </a>
                        </td>
                    </tr>
                    <tr hidden={typeof this.props.hideNoteLink !== 'undefined' || this.props.hideNoteLink}>
                        <td width="15%">RP Note</td>
                        <td>
                            <a href={this.getRPNoteUrl()} target="_blank">
                                {this.getRPNoteUrl()}
                            </a>
                        </td>
                    </tr>
                    <tr hidden={typeof this.props.hideAppellation !== 'undefined' || this.props.hideAppellation}>
                        <td width="15%">Appellation Lookup</td>
                        <td>{this.getAppellation(this.props.wine)}</td>
                    </tr>
                    <tr hidden={typeof this.props.hideVariety !== 'undefined' || this.props.hideVariety}>
                        <td width="15%">Variety Lookup</td>
                        <td>{this.props.wine && this.props.wine.variety ? this.props.wine.variety : ''}</td>
                    </tr>
                    <tr hidden={!this.props.showNote}>
                        <td width="15%">WineAlert Note</td>
                        <td>
                            <div className="row" style={{display: this.state.isEditingNote ? 'none' : 'block'}}>
                                <div className="col-xs-12">
                                    {this.state.note}
                                </div>
                                <div className="col-xs-12" style={{display: this.props.wine && this.props.wine.wineDbId ? 'block': 'none'}}>
                                    <ul className="list-inline btn-list">
                                        <li><a href="javascript:void(0)" onClick={this.editNote}>Edit WineAlert Note</a></li>
                                    </ul>
                                </div>
                            </div>
                            <div className="row" style={{display: this.state.isEditingNote ? 'block' : 'none'}}>
                                <div className="col-xs-12">
                                    <textarea className="form-control" value={this.state.note} name="note" onChange={this.handleInputChange}/>
                                </div>
                                <div className="col-xs-12">
                                    <ul className="list-inline btn-list">
                                        <li><button className="btn btn-md btn-default" onClick={this.cancelNote}>Cancel</button></li>
                                        <li><button className="btn btn-md btn-primary" onClick={this.saveNote}>Save</button></li>
                                    </ul>
                                </div>
                            </div>
                        </td>
                    </tr>
                    </tbody>
                </table>
            </div>
        );
    }
}

export default ExternalResources;