/**
 * Created by marjan on 4/20/17.
 */
import React from 'react';
import * as Constants from "../constants";
import {ContextMenu, ContextMenuTrigger, MenuItem} from "react-contextmenu";
import {CheckForInvalidKeyword, CleanUpSpaces, EscapeHtml, EscapeRegExp} from "../helpers/helper";

const RE_NOT_REMOVER = new RegExp(/(![^&|!()]+(_)?)|(&![^&|!()]+(_)?)|(\|![^&|!()]+(_)?)|(&!\(+(.*?)\)+)|(\|!\(+(.*?)\)+)/gm);
const RE_LOGICAL_SPLITTER = new RegExp('[&|](?![^(]*\\))', 'g');
const RE_LOGICAL_CLEANER = new RegExp(/^\(?|\)?$/g);

class Keyword extends React.Component {
    constructor() {
        super();
        this.state = {
            separateNoises: true,
            autovetedString: "",
            isMatched: false,
            selectedPhrase: "",
            noises: {},
            producerNoises: {},
            globalNoises: [],
            newNoises: {},
            newProducerNoises: {},
            newGlobalNoises: []
        };

        this.handleClick = this.handleClick.bind(this);
        this.handleSelection = this.handleSelection.bind(this);

        this.getGlobalNoises = this.getGlobalNoises.bind(this);

        this.autovetString = this.autovetString.bind(this);
        this.getProducerNoises = this.getProducerNoises.bind(this);
        this.getWineNoises = this.getWineNoises.bind(this);
        this.processNoises = this.processNoises.bind(this);

        this.searchNoises = this.searchNoises.bind(this);
        this.updateNoises = this.updateNoises.bind(this);
    };

    componentWillMount() {
        this.getGlobalNoises();
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.recordNo !== this.props.recordNo || prevProps.wineDbId !== this.props.wineDbId ||
            prevProps.keywordInput !== this.props.keywordInput) {
            //console.log(this.props.recordNo);
            let _this = this;
            _this.getProducerNoises(function () {
                _this.getWineNoises(_this.autovetString);
            });
        }
    }

    handleClick(e, data) {
        e.preventDefault();
        let _this = this;
        let globalNoises = _this.state.globalNoises;
        let newGlobalNoises = _this.state.newGlobalNoises;
        let noises = _this.state.noises;
        let newNoises = _this.state.newNoises;
        let producerNoises = _this.state.producerNoises;
        let newProducerNoises = _this.state.newProducerNoises;

        let selectedPhrase = _this.state.selectedPhrase;
        selectedPhrase = CleanUpSpaces(selectedPhrase);
        let producerId = _this.props.wineDbId.substring(0,6);
        let wineDbId = _this.props.wineDbId;

        if (selectedPhrase.length > 0) {
            if (globalNoises.indexOf(selectedPhrase) > -1 || newGlobalNoises.indexOf(selectedPhrase) > -1) {
                if (data.action === 'DelGlobalNoise') {
                    let nn = newGlobalNoises.indexOf(selectedPhrase);
                    if (nn > -1) {
                        newGlobalNoises.splice(nn, 1);

                        //push noise to new WineIdNoises
                        let widNoises = noises[wineDbId] ? noises[wineDbId] : [];
                        let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];

                        if (selectedPhrase.includes(" ")) {
                            let selectedPhraseArray = selectedPhrase.split(" ");
                            for (let i=0; i < selectedPhraseArray.length; i++){
                                if (widNoises.indexOf(selectedPhraseArray[i]) === -1 && newWidNoises.indexOf(selectedPhraseArray[i]) === -1) {
                                    newWidNoises.push(selectedPhraseArray[i])
                                }
                            }
                        } else if (widNoises.indexOf(selectedPhrase) === -1 && newWidNoises.indexOf(selectedPhrase) === -1) {
                            newWidNoises.push(selectedPhrase);
                        }
                        newNoises[wineDbId] = newWidNoises;
                    }

                    let en = globalNoises.indexOf(selectedPhrase);
                    if (en > -1) {
                        globalNoises.splice(en, 1);
                    }
                }
            } else {
                if (data.action === 'AddWIDNoise') {
                    let widNoises = noises[wineDbId] ? noises[wineDbId] : [];
                    let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];

                    if (widNoises.indexOf(selectedPhrase) === -1 && newWidNoises.indexOf(selectedPhrase) === -1) {
                        newWidNoises.push(selectedPhrase);
                    }
                } else if (data.action === 'AddGlobalNoise') {
                    if (newGlobalNoises.indexOf(selectedPhrase) === -1 && globalNoises.indexOf(selectedPhrase) === -1) {
                        //delete noises from new noises
                        if (selectedPhrase.includes(" ")) {
                            // let selectedPhraseArray = selectedPhrase.split(" ");
                            let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];

                            newNoises[wineDbId] = newWidNoises.filter(function (el) {
                                return selectedPhrase.indexOf(el) < 0;
                            });
                        } else {
                            let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];
                            let nn = newWidNoises.indexOf(selectedPhrase);

                            if (nn > -1){
                                newWidNoises.splice(nn,1);
                                newNoises[wineDbId] = newWidNoises;
                            }
                        }

                        newGlobalNoises.push(selectedPhrase);
                    }
                } else if (data.action === 'AddProducerNoise') {
                    //delete noises from new noises
                    if (selectedPhrase.includes(" ")) {
                        // let selectedPhraseArray = selectedPhrase.split(" ");
                        let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];

                        newNoises[wineDbId] = newWidNoises.filter(function (el) {
                            return selectedPhrase.indexOf(el) < 0;
                        });
                    } else {
                        let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];
                        let nn = newWidNoises.indexOf(selectedPhrase);

                        if (nn > -1){
                            newWidNoises.splice(nn,1);
                            newNoises[wineDbId] = newWidNoises;
                        }
                    }

                    let producerIdNoises = producerNoises[producerId] ? producerNoises[producerId] : [];
                    let newProducerIdNoises = newProducerNoises[producerId] ? newProducerNoises[producerId] : [];

                    if (producerIdNoises.indexOf(selectedPhrase) === -1 && newProducerIdNoises.indexOf(selectedPhrase) === -1) {
                        newProducerIdNoises.push(selectedPhrase);
                        newProducerNoises[producerId] = newProducerIdNoises;
                    }
                } else if (data.action === 'DelWIDNoise') {
                    let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];
                    let nn = newWidNoises.indexOf(selectedPhrase);
                    if (nn > -1) {
                        newWidNoises.splice(nn, 1);
                    }

                    let widNoises = noises[wineDbId] ? noises[wineDbId] : [];
                    let en = widNoises.indexOf(selectedPhrase);
                    if (en > -1) {
                        widNoises.splice(en, 1);
                    }
                } else if (data.action === 'DelProducerNoise') {
                    //delete noise from newProducerNoises by producerId
                    let newProducerIdNoises = newProducerNoises[producerId] ? newProducerNoises[producerId] : [];
                    let nn = newProducerIdNoises.indexOf(selectedPhrase);
                    if (nn > -1) {
                        newProducerIdNoises.splice(nn, 1);
                        newProducerNoises[producerId] = newProducerIdNoises;

                        //push noise to new WineIdNoises
                        let widNoises = noises[wineDbId] ? noises[wineDbId] : [];
                        let newWidNoises = newNoises[wineDbId] ? newNoises[wineDbId] : [];

                        if (selectedPhrase.includes(" ")) {
                            let selectedPhraseArray = selectedPhrase.split(" ");
                            for (let i=0; i < selectedPhraseArray.length; i++){
                                if (widNoises.indexOf(selectedPhraseArray[i]) === -1 && newWidNoises.indexOf(selectedPhraseArray[i]) === -1) {
                                    newWidNoises.push(selectedPhraseArray[i])
                                }
                            }
                        } else if (widNoises.indexOf(selectedPhrase) === -1 && newWidNoises.indexOf(selectedPhrase) === -1) {
                            newWidNoises.push(selectedPhrase);
                        }
                        newNoises[wineDbId] = newWidNoises;
                    }

                    //delete noise from producerNoises by producerId
                    let producerIdNoise = producerNoises[producerId] ? producerNoises[producerId] : [];
                    let en = producerIdNoise.indexOf(selectedPhrase);
                    if (en > -1) {
                        producerIdNoise.splice(en, 1);
                        producerNoises[producerId] = producerIdNoise;
                    }

                }
            }

            _this.setState({
                noises: noises,
                newNoises: newNoises,
                globalNoises: globalNoises,
                newGlobalNoises: newGlobalNoises,
                producerNoises: producerNoises,
                newProducerNoises: newProducerNoises
            });
        }
    }

    handleSelection() {
        let selection = window.getSelection();
        let s = selection.toString();
        this.setState({selectedPhrase: s});
    }

    logicalSplit(str) {
        let words = [];
        str = str ? str.replace(RE_LOGICAL_CLEANER, "") : str;
        if (CheckForInvalidKeyword(str)) return words;

        let strs = str.split(RE_LOGICAL_SPLITTER);
        let i;
        for (i in strs) {
            let word = strs[i];
            if (word) {
                if (word.indexOf("(") > -1 || word.indexOf(")") > -1) {
                    let deeps = this.logicalSplit(word, true);
                    if (deeps && deeps.length > 0) {
                        words = words.concat(deeps);
                    }
                } else {
                    /*if (word.indexOf("!")===0) {
                        word = word.replace("!", ""); // Complex logic
                    }*/
                    if (word.indexOf("!")===-1) words.push(word);
                }
            }
        }
        return words;
    }

    autovetString() {
        if (!this.props.pattern) {
            if (this.state.autovetedString !== "" || this.state.isMatched || this.state.selectedPhrase !== "") {
                let stateData = {};
                if (this.state.autovetedString !== "") stateData.autovetedString = "";
                if (this.state.isMatched) stateData.isMatched = false;
                if (this.state.selectedPhrase !== "") stateData.selectedPhrase = "";
                this.setState(stateData);
            }
            return;
        }

        // clean up not expression
        let pattern = this.props.pattern;
        pattern = pattern.replace(RE_NOT_REMOVER, "");
        let a = pattern.split(RE_LOGICAL_SPLITTER);
        let u = a.filter(function(item, pos) { // get unique
            return a.indexOf(item) === pos;
        });

        let i;
        let lk = [];
        let ak = "";
        for (i in u) {
            if (u[i].indexOf("|") > -1) {
                lk.push(u[i].trim());
                let j;
                let stringInAutovet = "";
                let orR = new RegExp('[()|]', 'g');
                let or = u[i].split(orR);
                for (j in or) {
                    let k = or[j].trim();
                    if (k && or[j].indexOf("&") < 0 && stringInAutovet === "") {
                        stringInAutovet = or[j];
                    }
                }
                ak += "<span title='" + EscapeHtml(u[i]) + "' data-toggle='tooltip' data-placement='bottom'>" + EscapeHtml(u[i]) + " </span>";
            } else {
                let _newKeyword = u[i].replace(new RegExp('[()]', 'g'), '').trim();
                lk.push(_newKeyword);
                ak += "<span title='" + EscapeHtml(u[i]) + "' data-toggle='tooltip' data-placement='bottom'>" + EscapeHtml(_newKeyword) + " </span>";
            }
        }
        let autovetedString = <div dangerouslySetInnerHTML={{ __html: ak }} />;

        let stateData = {};
        stateData.autovetedString = autovetedString;
        if (this.state.selectedPhrase !== "") stateData.selectedPhrase = "";
        this.setState(stateData, this.searchNoises);
    }

    searchNoises() {
        if (!this.props.pattern) {
            if (typeof this.props.onMatched === "function") this.props.onMatched([], false);
            return;
        }

        let _this = this;
        let wineDbId = _this.props.wineDbId || '';
        let pattern = _this.props.pattern;
        let desc = _this.props.keywordInput || '';
        let producerId = _this.props.wineDbId.substring(0,6);

        let originalDesc = desc;

        // Remove words that exists in positive keywords
        // Populate true-positive-words
        pattern = pattern.replace(RE_NOT_REMOVER, "");
        let result = _this.logicalSplit(pattern);
        result = result.filter(function(item, pos) { // unique
            return result.indexOf(item) === pos;
        });
        result.sort(function(a, b) { // sort by word length
            return b.length - a.length;
        });
        //console.log("True positive words:", result);

        let i;
        for (i in result) {
            let search = result[i];
            let rs = search.indexOf("_") > -1 ?
                new RegExp('\\b' + EscapeRegExp(search.replace("_", "")) + '\\w*\\b', 'gi') :
                new RegExp('\\b' + EscapeRegExp(search) + '\\b', 'gi');
            desc = desc.replace(rs, ' ');
        }
        //console.log("Description:", desc);

        // Remove words which are already in existing noises
        let allNoises = [];
        if (_this.state.globalNoises) allNoises = allNoises.concat(_this.state.globalNoises);
        if (_this.state.newGlobalNoises) allNoises = allNoises.concat(_this.state.newGlobalNoises);
        if (_this.state.noises && _this.state.noises[wineDbId]) allNoises = allNoises.concat(_this.state.noises[wineDbId]);
        if (_this.state.producerNoises && _this.state.producerNoises[producerId]) allNoises = allNoises.concat(_this.state.producerNoises[producerId]);
        if (_this.state.newProducerNoises && _this.state.newProducerNoises[producerId]) allNoises = allNoises.concat(_this.state.newProducerNoises[producerId]);
        let j;
        for (j in allNoises) {
            let search = allNoises[j];
            let rs = new RegExp('\\b' + EscapeRegExp(search) + '\\b', 'gi');
            desc = desc.replace(rs, ' ');
        }

        let noises = _this.convertToNoises(desc, originalDesc);
        if (noises.length > 0) {
            noises = noises.filter(function(item, pos) { // unique
                return noises.indexOf(item) === pos;
            });
            noises.sort(function(a, b) { // sort by word length
                return b.length - a.length;
            });
            //console.info(wineDbId + "; Found new noises: " + noises.join(","));

            let newNoises = this.state.newNoises;
            newNoises[wineDbId] = noises;

            let stateData = {};
            stateData.newNoises = newNoises;
            if (_this.state.isMatched) stateData.isMatched = false;
            _this.setState(stateData, function () {
                if (_this.props.autoUpdateNoises) {
                    _this.updateNoises(typeof _this.props.onMatched === "function" ? _this.props.onMatched : null);
                } else {
                    if (typeof _this.props.onMatched === "function") _this.props.onMatched(noises);
                }
            });
        } else {
            //console.info(wineDbId + "; No new noise found.");
            if (!_this.state.isMatched) {
                _this.setState({isMatched: true}, typeof _this.props.onMatched === "function" ? _this.props.onMatched : null);
            } else {
                if (typeof _this.props.onMatched === "function") _this.props.onMatched();
            }
        }
    }

    convertToNoises(desc, originalDesc) {
        desc = CleanUpSpaces(desc);
        if (!desc) return [];

        let noises = [];
        let ar = desc.split(new RegExp('\\s', 'g'));
        let i = 0;
        let len = ar.length;
        for (; i < len; ) {
            if (this.state.separateNoises) {
                noises.push(ar[i]);
            } else {
                let sibling = this.searchSibling(ar[i], i, ar, originalDesc);
                if (sibling.length > 0) {
                    noises.push(ar[i] + ' ' + sibling.join(" "));
                    i = i + sibling.length;
                } else {
                    noises.push(ar[i]);
                }
            }
            i = i + 1;
        }
        return noises;
    }

    searchSibling(search, currentPos, allArray, originalDesc) {
        let sibling = [];
        let next = currentPos + 1;
        if (allArray.length > next) {
            let newSearch = search + " " + allArray[next];
            let rs = new RegExp('\\b' + EscapeRegExp(newSearch) + '\\b', 'g');
            if (originalDesc.search(rs) !== -1) {
                sibling.push(allArray[next]);
                let newSibling = this.searchSibling(newSearch, next, allArray, originalDesc);
                if (newSibling) {
                    sibling = sibling.concat(newSibling);
                }
            }
        }
        return sibling;
    }

    processNoises() {
        if (this.props.keywordInput && this.props.pattern) {
            let arr = [this.props.keywordInput];
            let elKey = 0;

            // Not Noise
            // Populate true-positive-words
            let pattern = this.props.pattern.toLowerCase();
            pattern = pattern.replace(RE_NOT_REMOVER, "");
            let result = this.logicalSplit(pattern);
            result = result.filter(function(item, pos) { // unique
                return result.indexOf(item) === pos;
            });
            result.sort(function(a, b) { // sort by word length
                return b.length - a.length;
            });
            for (let i in result) {
                let notNoise = result[i];
                arr = arr.flatMap(function (kw) {
                    if (kw === '') {
                        return [];
                    } else if (typeof kw === 'string') {
                        let rs = notNoise.indexOf("_") > -1 ?
                            new RegExp('(\\b' + EscapeRegExp(notNoise.replace("_", "")) + '\\w*\\b)', 'gi') :
                            new RegExp('(\\b' + EscapeRegExp(notNoise) + '\\b)', 'gi');
                        return kw.split(rs).map(function(w) {
                            elKey++;
                            if (notNoise.indexOf("_") > -1) {
                                return w.trim().indexOf(notNoise.replace("_", "")) > -1 ? <span title={w} key={elKey}> {w} </span> : w;
                            } else {
                                return w.trim() === notNoise ? <span title={w} key={elKey}> {w} </span> : w;
                            }
                        });
                    } else {
                        return kw;
                    }
                });
            }

            // Global Noises
            if (this.state.newGlobalNoises) {
                for (let i in this.state.newGlobalNoises) {
                    let noise = this.state.newGlobalNoises[i];
                    noise = noise.trim().toLowerCase();
                    arr = arr.flatMap(function (kw) {
                        if (kw === '') {
                            return [];
                        } else if (typeof kw === 'string') {
                            return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)', 'gi')).map(function(w) {
                                elKey++;
                                return w.trim() === noise ? <span className="new-global-noise" title={w} key={elKey}>{w}</span> : w;
                            });
                        } else {
                            return kw;
                        }
                    });
                }
            }
            if (this.state.globalNoises) {
                for (let i in this.state.globalNoises) {
                    let noise = this.state.globalNoises[i];
                    noise = noise.trim().toLowerCase();
                    arr = arr.flatMap(function (kw) {
                        if (kw === '') {
                            return [];
                        } else if (typeof kw === 'string') {
                            return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)', 'gi')).map(function(w) {
                                elKey++;
                                return w.trim() === noise ? <span className="global-noise" title={w} key={elKey}>{w}</span> : w;
                            });
                        } else {
                            return kw;
                        }
                    });
                }
            }

            if (this.props.wineDbId) {

                // Producer noises
                let producerId = this.props.wineDbId.substring(0,6);
                let newProducerIdNoises = this.state.newProducerNoises[producerId];
                if (newProducerIdNoises) {
                    for (let i in newProducerIdNoises) {
                        let noise = newProducerIdNoises[i];
                        noise = noise.trim().toLowerCase();
                        arr = arr.flatMap(function (kw) {
                            if (kw === '') {
                                return [];
                            } else if (typeof kw === 'string') {
                                return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)')).map(function(w) {
                                    elKey++;
                                    return w.trim() === noise ? <span className="new-producer-noise" title={w} key={elKey}>{w}</span> : w;
                                });
                            } else {
                                return kw;
                            }
                        });
                    }
                }
                let producerIdNoises = this.state.producerNoises[producerId];
                if (producerIdNoises) {
                    for (let i in producerIdNoises) {
                        let noise = producerIdNoises[i];
                        noise = noise.trim().toLowerCase();
                        arr = arr.flatMap(function (kw) {
                            if (kw === '') {
                                return [];
                            } else if (typeof kw === 'string') {
                                return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)')).map(function(w) {
                                    elKey++;
                                    return w.trim() === noise ? <span className="producer-noise" title={w} key={elKey}>{w}</span> : w;
                                });
                            } else {
                                return kw;
                            }
                        });
                    }
                }

                // Wine Noises
                let newWidNoises = this.state.newNoises[this.props.wineDbId];
                if (newWidNoises) {
                    for (let i in newWidNoises) {
                        let noise = newWidNoises[i];
                        noise = noise.trim().toLowerCase();
                        arr = arr.flatMap(function (kw) {
                            if (kw === '') {
                                return [];
                            } else if (typeof kw === 'string') {
                                return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)')).map(function(w) {
                                    elKey++;
                                    return w.trim() === noise ? <span className="new-wid-noise" title={w} key={elKey}>{w}</span> : w;
                                });
                            } else {
                                return kw;
                            }
                        });
                    }
                }
                let widNoises = this.state.noises[this.props.wineDbId];
                if (widNoises) {
                    for (let i in widNoises) {
                        let noise = widNoises[i];
                        noise = noise.trim().toLowerCase();
                        arr = arr.flatMap(function (kw) {
                            if (kw === '') {
                                return [];
                            } else if (typeof kw === 'string') {
                                return kw.split(new RegExp('(\\b' + EscapeRegExp(noise) + '\\b)', 'gi')).map(function(w) {
                                    elKey++;
                                    return w.trim() === noise ? <span className="wid-noise" title={w} key={elKey}>{w}</span> : w;
                                });
                            } else {
                                return kw;
                            }
                        });
                    }
                }
            }
            return (
                <div>{arr}</div>
            );
        }
        return <div>&nbsp;</div>;
    };

    async getWineNoises(callback) {
        let wineDbId = this.props.wineDbId;

        if (!wineDbId) return;
        /*if (this.state.noises[wineDbId] && this.state.noises[wineDbId].length > 0) {
            if (typeof callback === 'function') callback();
            return;
        }*/

        let _this = this;
        new Promise(function(resolve) {
            fetch(Constants.WINE_NOISE_URL + "?wineDbId=" + wineDbId, {
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                resolve(json);
            }).catch(function(ex) {
                resolve();
                alert(ex);
            });
        }).then(function (res) {
            if (res && res.data && res.data.length > 0) {
                let sorted = res.data.sort(function(a, b) { // sort by word length
                    return b.length - a.length;
                });
                let noises = _this.state.noises;
                noises[wineDbId] = sorted;
                _this.setState({noises: noises}, typeof callback === 'function' ? callback : null);
            } else {
                if (typeof callback === 'function') callback();
            }
        });
    }

    async getGlobalNoises() {
        const res = await fetch(Constants.GLOBAL_NOISE_URL, {
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            return json;
        }).catch(function(ex) {
            alert(ex);
        });

        if (res.data && res.data.length > 0) {
            let sorted = res.data.sort(function(a, b) { // sort by word length
                return b.length - a.length;
            });
            this.setState({globalNoises: sorted});
        }
    }

    async getProducerNoises(callback) {
        let _this = this;
        let producerId = this.props.wineDbId.substring(0,6);

        if (!producerId) return;
        if (this.state.producerNoises[producerId] && this.state.producerNoises[producerId].length > 0) {
            if (typeof callback === "function") callback();
            return;
        }

        new Promise(function(resolve) {
            fetch(Constants.PRODUCER_NOISE_URL+"?producerId=" + producerId, {
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                resolve(json);
            }).catch(function(ex) {
                resolve();
                alert(ex);
            });
        }).then(function (res) {
            if (res && res.data && res.data.length > 0) {
                let sorted = res.data.sort(function(a, b) { // sort by word length
                    return b.length - a.length;
                });
                let producerNoises = _this.state.producerNoises;
                producerNoises[producerId] = sorted;
                _this.setState({producerNoises: producerNoises}, function () {
                    if (typeof callback === "function") callback();
                });
            } else {
                if (typeof callback === "function") callback();
            }
        });
    }

    async updateNoises(callback) {
        let _this = this;

        // New wine noises
        let wineDbId = _this.props.wineDbId;
        let newNoises = _this.state.newNoises;
        let newWineNoises = newNoises && newNoises[wineDbId] ? JSON.parse(JSON.stringify(newNoises[wineDbId])) : [];

        // New producer noises
        let producerId = _this.props.wineDbId.substring(0,6);
        let newProducerNoises = _this.state.newProducerNoises;
        let newProducerIdNoises = newProducerNoises && newProducerNoises[producerId] ? JSON.parse(JSON.stringify(newProducerNoises[producerId])) : [];

        // New global noises
        let newGlobalNoises = JSON.parse(JSON.stringify(_this.state.newGlobalNoises));

        if (newWineNoises.length === 0 && newGlobalNoises.length === 0 && newProducerIdNoises.length === 0) {
            if (typeof callback === "function") callback();
            return;
        }

        let p1 = new Promise(function(resolve) {
            if (!newWineNoises || newWineNoises.length === 0) {
                resolve(true);
                return
            }

            fetch(Constants.UPDATE_WID_NOISES_URL, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    wineDbId: wineDbId,
                    noises: newWineNoises
                }),
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                if (json.success) {
                    resolve(true);
                } else {
                    resolve();
                    throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
                }
            }).catch(function(ex) {
                alert(ex);
                resolve();
            });
        });

        let p2 = new Promise(function(resolve) {
            if (!newProducerIdNoises || newProducerIdNoises.length === 0) {
                resolve(true);
                return
            }

            fetch(Constants.UPDATE_PRODUCER_NOISES_URL, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    producerId: producerId,
                    noises: newProducerIdNoises
                }),
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                if (json.success) {
                    resolve(true);
                } else {
                    resolve();
                    throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
                }
            }).catch(function(ex) {
                alert(ex);
                resolve();
            });
        });

        let p3 = new Promise(function(resolve) {
            if (!newGlobalNoises || newGlobalNoises.length === 0) {
                resolve(true);
                return
            }

            fetch(Constants.UPDATE_GLOBAL_NOISES_URL, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    noises: newGlobalNoises
                }),
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                if (json.success) {
                    resolve(true);
                } else {
                    resolve();
                    throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
                }
            }).catch(function(ex) {
                alert(ex);
                resolve();
            });
        });

        Promise.all([p1, p2, p3]).then(function(res) {
            let noises = _this.state.noises;
            /*if (res[0]) {
                // Merge new wine noises
                let widNoises = [];
                if (_this.state.noises[wineDbId]) widNoises = widNoises.concat(_this.state.noises[wineDbId]);
                if (newWineNoises) widNoises = widNoises.concat(newWineNoises);
                widNoises = widNoises.filter(function(item, pos) { // unique
                    return widNoises.indexOf(item) === pos;
                }).sort(function(a, b) { // sort by word length
                    return b.length - a.length;
                });
                noises[wineDbId] = widNoises;
            }*/

            let producerNoises = _this.state.producerNoises;
            if (res[1]) {
                // Merge new producer noises
                let prodNoises = [];
                if (_this.state.producerNoises[producerId]) prodNoises = prodNoises.concat(_this.state.producerNoises[producerId]);
                if (newProducerIdNoises) prodNoises = prodNoises.concat(newProducerIdNoises);
                prodNoises = prodNoises.filter(function (item, pos) { // unique
                    return prodNoises.indexOf(item) === pos;
                }).sort(function (a, b) { // sort by word length
                    return b.length - a.length;
                });
                producerNoises[producerId] = prodNoises;
            }

            let globalNoises = _this.state.globalNoises;
            if (globalNoises === undefined) globalNoises = [];
            if (res[2]) {
                // Merge new global noises
                if (newGlobalNoises) globalNoises = globalNoises.concat(newGlobalNoises);
                globalNoises = globalNoises.filter(function(item, pos) { // unique
                    return globalNoises.indexOf(item) === pos;
                }).sort(function(a, b) { // sort by word length
                    return b.length - a.length;
                });
            }

            // Empty new wine noise for this wine
            try {
                delete newNoises[wineDbId];
            } catch(e) {
                newNoises[wineDbId] = undefined;
            }

            // Empty new producer noise for this producer
            try {
                delete newProducerNoises[producerId];
            } catch(e) {
                newProducerNoises[producerId] = undefined;
            }

            _this.setState({
                noises: noises,
                newNoises: newNoises,
                producerNoises: producerNoises,
                newProducerNoises: newProducerNoises,
                globalNoises: globalNoises,
                newGlobalNoises: []
            }, typeof callback === "function" ? callback : null);
        });
    }

    render() {
        return (
            <table className="table table-bordered table-match-validation">
                <thead className="thead-default">
                <tr>
                    <th className="default" style={{width: '45%'}}>Autovet String</th>
                    <th className="default text-center" style={{width: '10%'}}>#</th>
                    <th className="default" style={{width: '45%'}}>Retailer Keyword String</th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>
                        <div className="wine-match-fixed-height-lg">
                            {this.state.autovetedString}
                        </div>
                    </td>
                    <td className="text-center">
                        <span className={this.state.isMatched ? 'validation-success' : 'validation-failed'}>{this.state.isMatched ? '√' : 'X'}</span>
                    </td>
                    <td>
                        <ContextMenuTrigger id={Constants.MENU_TYPE} holdToDisplay={1000}>
                            <div className="wine-keyword" onMouseUp={this.handleSelection}>
                                {this.processNoises()}
                            </div>
                        </ContextMenuTrigger>
                        <ContextMenu id={Constants.MENU_TYPE}>
                            <MenuItem onClick={this.handleClick} data={{action: 'AddWIDNoise'}}><span className="new-wid-noise">Add to WID Noise</span></MenuItem>
                            <MenuItem onClick={this.handleClick} data={{action: 'AddProducerNoise'}}><span className="new-producer-noise">Add to Producer Noise</span></MenuItem>
                            <MenuItem onClick={this.handleClick} data={{action: 'AddGlobalNoise'}}><span className="new-global-noise">Add to Global Noise</span></MenuItem>
                            <MenuItem divider />
                            <MenuItem onClick={this.handleClick} data={{action: 'DelWIDNoise' }}>Remove from WID Noises</MenuItem>
                            <MenuItem onClick={this.handleClick} data={{action: 'DelProducerNoise' }}>Remove from Producer Noise</MenuItem>
                            <MenuItem onClick={this.handleClick} data={{action: 'DelGlobalNoise' }}>Remove from Global Noise</MenuItem>
                        </ContextMenu>
                    </td>
                </tr>
                </tbody>
            </table>
        );
    };
}

export default Keyword;