import React from 'react';
import FixedHeaderStory from 'react-data-table-component';
import * as Constants from '../constants';
import {ConcatenateKeyword, DownloadCSV, GenerateWineKeyword} from "../helpers/helper";
import SlicedKeyword from "./SlicedKeyword";

const columns = [
    {
        name: 'ID',
        selector: row => row.wineDbId,
        grow: 1,
        sortable: true,
    },
    {
        name: 'Name',
        grow: 3,
        selector: row => row.wineDbName,
        sortable: true,
    },
    {
        name: 'Keyword',
        selector: (row) => {
            return <SlicedKeyword keyword={row.winePattern}/>;
        },
        grow: 10,
    },
];

class KeywordGenerator extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            spreadTables: {},
            reverseAliases: [],
            wineEliminations: [],
            massChanges: [],
            customChanges: [],
            colorSubstitutions: [],
            colorNotSubstitutions: [],
            reserveWords: [],
            redOpposites: [],
            roseOpposites: [],
            whiteOpposites: [],
            data: [],
            currentIndex: -1,
            isLoadingData: false,
            isGenerating: false,
            isGenerated: false,
            isSaving: false,
            isSaved: false,
        };
        this.timer = null;

        this.beforeUnload = this.beforeUnload.bind(this);
        this.uploadFile = this.uploadFile.bind(this);

        this.getSpreadTables = this.getSpreadTables.bind(this);
        this.getReverseAliases = this.getReverseAliases.bind(this);
        this.getWineEliminations = this.getWineEliminations.bind(this);
        this.getMassChanges = this.getMassChanges.bind(this);
        this.getCustomChanges = this.getCustomChanges.bind(this);
        this.getColorSubstitutions = this.getColorSubstitutions.bind(this);
        this.getRedOpposites = this.getRedOpposites.bind(this);
        this.getRoseOpposites = this.getRoseOpposites.bind(this);
        this.getWhiteOpposites = this.getWhiteOpposites.bind(this);
        this.getColorNotSubstitutions = this.getColorNotSubstitutions.bind(this);
        this.getReserveWords = this.getReserveWords.bind(this);

        this.start = this.start.bind(this);
        this.export = this.export.bind(this);
        this.save = this.save.bind(this);
        this.saveKeyword = this.saveKeyword.bind(this);
    }

    componentWillMount() {
        this.getReverseAliases();
        this.getWineEliminations();
        this.getMassChanges();
        this.getCustomChanges();
        this.getSpreadTables();
        this.getColorSubstitutions();
        this.getColorNotSubstitutions();
        this.getRedOpposites();
        this.getReserveWords();
        this.getRoseOpposites();
        this.getWhiteOpposites();
    }

    componentDidMount() {
        window.addEventListener("beforeunload", this.beforeUnload);
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillUnmount() {
        window.removeEventListener("beforeunload", this.beforeUnload);
    }

    beforeUnload(e) {
        if (this.state.data && this.state.data.length > 0 && this.state.data.length > this.state.currentIndex + 1) {
            e.preventDefault();
            e.returnValue = true;
        }
    }

    start() {
        if (!this.state.data || !this.state.data.length) {
            alert("This is no record.");
            return;
        }
        let _this = this;
        _this.setState({isGenerated: false, isGenerating: true, isSaving: false, isSaved: false}, function () {
            _this.state.data.forEach(function (wine, i) {
                clearTimeout(_this.timer);
                _this.timer = setTimeout(function () {
                    _this.setState((state) => {
                        return {
                            currentIndex: i
                        };
                    });
                }.bind(_this), 100);

                let colorOpposites = {
                    red: _this.state.redOpposites,
                    rose: _this.state.roseOpposites,
                    white: _this.state.whiteOpposites
                };
                wine.regionCode = wine.wineDbId.substring(0, 2);
                wine = GenerateWineKeyword(
                    wine,
                    _this.state.spreadTables,
                    _this.state.reverseAliases,
                    _this.state.wineEliminations,
                    _this.state.massChanges,
                    _this.state.customChanges,
                    _this.state.colorSubstitutions,
                    colorOpposites,
                    _this.state.colorNotSubstitutions,
                    _this.state.reserveWords
                );
                wine = ConcatenateKeyword(wine)
                wine.status = '-';
                _this.state.data[i] = wine;
            });
            _this.setState({isGenerated: true, isGenerating: false});
        });
    }

    export() {
        if (!this.state.data) return;
        let data = this.state.data.map(function (item) {
            return {
                "ID": item.wineDbId,
                "Name": item.wineDbName,
                "Color": item.color,
                "Producer": item.producerShow,
                "Producer Keyword": item.producerKeyword,
                "Noise": item.noises ? item.noises.join('/') : '',
                "Base Keyword": item.baseKeyword,
                "Appellation Keyword": item.appellationKeyword,
                "Color Keyword": item.colorKeyword,
                "Common Modifiers Keyword": item.commonModifiersKeyword,
                "Modifiers Keyword": item.modifiersKeyword,
                "Global Not Keyword": item.globalNotKeyword,
                "Keyword": item.winePattern,
                "Status": item.status,
            };
        });
        DownloadCSV(data, 'generated_kw');
    }

    save() {
        if (!this.state.data || this.state.isSaving) return;
        let _this = this;
        _this.setState({isSaving: true, isSaved: false});
        let data = this.state.data.map(function (item) {
            return {
                wineDbId: item.wineDbId,
                producerKeyword: item.producerKeyword,
                baseKeyword: item.baseKeyword,
                appellationKeyword: item.appellationKeyword,
                colorKeyword: item.colorKeyword,
                commonModifiersKeyword: item.commonModifiersKeyword,
                modifiersKeyword: item.modifiersKeyword,
                globalNotKeyword: item.globalNotKeyword
            };
        });
        const chunkSize = 10;
        for (let i = 0; i < data.length; i += chunkSize) {
            const chunk = data.slice(i, i + chunkSize);
            _this.saveKeyword(chunk);
        }
        _this.setState({isSaving: false, isSaved: true});
    }

    async saveKeyword(chunk) {
        const response = await fetch(Constants.SAVE_WID_KEYWORD, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({keywords: chunk}),
            credentials: 'same-origin'
        });
        return await response.json();
    }

    async uploadFile(event) {
        event.preventDefault();
        let _this = this;
        _this.setState({narrowKeywords: [], currentIndex: -1, isLoadingData: true});
        let input = document.querySelector('input[type="file"]');

        let data = new FormData();
        data.append('file', input.files[0]);

        fetch(Constants.UPLOAD_FOR_KEYWORD_GENERATOR, {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function (response) {
            return response.json();
        }).then(function (json) {
            if (json.success) {
                if (json.data.length > 0) {
                    _this.setState({data: json.data, isLoadingData: false}, _this.start);
                } else {
                    throw new Error("No record found.");
                }
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function (ex) {
            _this.setState({isLoadingData: false});
            alert(ex);
        });
    }

    async getSpreadTables() {
        const rawData = await fetch(Constants.SPREAD_TABLES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({spreadTables: res.data});
    }

    async getReverseAliases() {
        const rawData = await fetch(Constants.REVERSE_ALIASES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function (a, b) { // sort by key length, descx
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({reverseAliases: sorted});
    }

    async getWineEliminations() {
        const rawData = await fetch(Constants.ELIMINATIONS_URL + "?type=" + Constants.ELIMINATION_TYPE.WINE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function (a, b) { // sort by word length, desc
            return b.length - a.length;
        });
        this.setState({wineEliminations: sorted});
    }

    async getMassChanges() {
        const rawData = await fetch(Constants.WORD_CHANGES_URL + "?type=" + Constants.ALIAS_TYPE.MASS_CHANGES, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function (a, b) { // sort by key length, desc
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({massChanges: sorted});
    }

    async getCustomChanges() {
        const rawData = await fetch(Constants.WORD_CHANGES_URL + "?type=" + Constants.ALIAS_TYPE.CUSTOM_CHANGES, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function (a, b) { // sort by key length, desc
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({customChanges: sorted});
    }

    async getColorSubstitutions() {
        const rawData = await fetch(Constants.SUBSTITUTIONS_URL + "?type=" + Constants.SUBSTITUTION_TYPE.COLOR, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({colorSubstitutions: res.data});
    }

    async getRedOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.RED_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({redOpposites: res.data});
    }

    async getRoseOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.ROSE_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({roseOpposites: res.data});
    }

    async getWhiteOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.WHITE_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({whiteOpposites: res.data});
    }

    async getColorNotSubstitutions() {
        const rawData = await fetch(Constants.SUBSTITUTIONS_URL + "?type=" + Constants.SUBSTITUTION_TYPE.COLOR_NOT, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({colorNotSubstitutions: res.data});
    }

    async getReserveWords() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.RESERVE_WORD, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({reserveWords: res.data});
    }

    actions() {
        return <span>
            <button className="btn btn-sm" onClick={this.export}>Export</button>
            &nbsp;&nbsp;
            <button className="btn btn-sm btn-primary"
                    onClick={this.save}
                    disabled={this.state.isSaving}>{this.state.isSaving ? 'Saving...' : (this.state.isSaved ? 'Saved' : 'Save to db')}</button>
        </span>;
    }

    indicator() {
        if (this.state.isLoadingData) return 'Loading...';
        if (this.state.isGenerating || this.state.isGenerated) {
            return (this.state.currentIndex + 1) + ' of ' + this.state.data.length;
        }
    }

    render() {
        return (
            <div className="row">
                <div className="col-xs-12">
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <h2 className="title">Keyword Generator</h2>
                        </div>
                    </div>
                    <div className="row wine-match-row retailer-form">
                        <div className="col-xs-10">
                            <form className="form-inline" onSubmit={this.uploadFile}>
                                <div className="row row-form">
                                    <div className="col-md-1">
                                        <label className="control-label">File</label>
                                    </div>
                                    <div className="col-md-4 text-right">
                                        <input type="file" name="file" className="form-control"
                                               onChange={this.handleInputChange}/>
                                    </div>
                                    <div className="col-md-7 text-left">
                                        <button type="submit" className="btn btn-default"
                                                disabled={this.state.isLoadingData}>{this.state.isLoadingData ? 'Loading...' : 'Load Data'}</button>
                                    </div>
                                </div>
                            </form>
                        </div>
                        <div className="col-xs-2 text-right">
                            <span><strong>{this.indicator()}</strong></span>
                        </div>
                    </div>
                    {this.state.isGenerating || this.state.isGenerated ? <FixedHeaderStory
                        columns={columns}
                        data={this.state.isGenerating ? [] : this.state.data}
                        fixedHeader={true}
                        fixedHeaderScrollHeight="500px"
                        pagination={true}
                        progressPending={this.state.isGenerating}
                        paginationRowsPerPageOptions={[25, 50, 100]}
                        actions={this.actions()}
                    /> : null}
                </div>
            </div>
        );
    }
}

export default KeywordGenerator;
