import React, { Component } from 'react';
import Select from 'react-select';
import * as Constants from '../constants';
import ExternalResources from "./ExternalResources";
import RetailerWine from "./RetailerWine";
import Candidates from "./Candidates";
import WineEditor from "./WineEditor";
import ProducerEditor from "./ProducerEditor";

class Narrow extends Component {
    constructor(props) {
        super(props);
        this.state = {
            activeRetailers: [],
            fileSource: Constants.DATA_SOURCES.INTERNAL,
            matchedFilter: "all",
            retailers: [],
            allData: [],
            data: {
                retailer: "",
                description: "",
                bottleSize: "",
                price: "",
                url: "",
                wineHistory: "",
                wineOriginalKeyword: "",
                wineKeyword: "",
                id: "",
                mongoId: "",
                wineVinId: "",
                regionCode: "",
                vintage: "",
                wineDbId: "",
                wineDbName: "",
                pattern: "",
                waNote: "",
                color: "",
                variety: "",
                sweetness: "",
                wineType: "Table",
                country: "",
                region: "",
                location: "",
                locale: "",
                site: "",
                //rawData: "",
                acceptedMatch: {},
                logs: ""
            },
            currentIndex: -1,
            revisit: false,
            isLoadingMatch: false,
            isPreviousMatched: false,
            resetFilterID: false,
            narrowKeywords: [],
            selectedMatch: {},
            isEditingWine: false,
            wineDbId: "", // WineDbId to be edited
            wine: {}, // Wine to be edited
            isEditingProducer: false,
            producerId: "" // ProducerId to be edited
        };
        this.timer = null;
        this.wineIdFilterClearedTimer = null;

        this.beforeUnload = this.beforeUnload.bind(this);
        this.handleInputChange = this.handleInputChange.bind(this);

        this.getRetailers = this.getRetailers.bind(this);

        this.selectRetailers = this.selectRetailers.bind(this);

        this.getData = this.getData.bind(this);
        this.loadData = this.loadData.bind(this);
        this.uploadFile = this.uploadFile.bind(this);

        this.nextRecord = this.nextRecord.bind(this);
        this.prevRecord = this.prevRecord.bind(this);
        this.jumpToRecord = this.jumpToRecord.bind(this);
        this.goToRecord = this.goToRecord.bind(this);

        this.repeatPrevious = this.repeatPrevious.bind(this);

        this.updateHistory = this.updateHistory.bind(this);

        this.deferWine = this.deferWine.bind(this);
        this.insufficientWine = this.insufficientWine.bind(this);
        this.markWine = this.markWine.bind(this);

        this.onSavedVintage = this.onSavedVintage.bind(this);
        this.onSavedBottleSize = this.onSavedBottleSize.bind(this);
        this.onSavedNote = this.onSavedNote.bind(this);

        this.selectMatch = this.selectMatch.bind(this);
        this.resetMatch = this.resetMatch.bind(this);
        this.acceptMatch = this.acceptMatch.bind(this);
        this.updateOriginal = this.updateOriginal.bind(this);

        this.editWineDb = this.editWineDb.bind(this);
        this.createWineNewProducer = this.createWineNewProducer.bind(this);
        this.createWine = this.createWine.bind(this);
        this.cancelEditWine = this.cancelEditWine.bind(this);
        this.onWineDbSaved = this.onWineDbSaved.bind(this);

        this.editProducer = this.editProducer.bind(this);
        this.cancelEditProducer = this.cancelEditProducer.bind(this);

        this.addNarrowWord = this.addNarrowWord.bind(this);
        this.removeNarrowWord = this.removeNarrowWord.bind(this);
        this.refreshCandidates = this.refreshCandidates.bind(this);

        this.onWineIdFilterCleared = this.onWineIdFilterCleared.bind(this);
    }

    componentWillMount() {
        this.getRetailers();
    }

    componentDidMount() {
        window.addEventListener("beforeunload", this.beforeUnload);
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillUnmount() {
        window.removeEventListener("beforeunload", this.beforeUnload);
    }

    beforeUnload(e) {
        if (this.state.allData && this.state.allData.length > 0 && this.state.allData.length > this.state.currentIndex + 1) {
            e.preventDefault();
            e.returnValue = true;
        }
    }

    handleInputChange(event) {
        const target = event.target;
        const name = target.name;

        let value = target.type === 'checkbox' ? target.checked : target.value;
        this.setState({
            [name]: value
        }, function () {
            //console.log(name, value);
        });
    }

    nextRecord() {
        let next = this.state.currentIndex + 1;
        if (this.state.allData.length > next) {
            // Latest Matched Data
            let latestData = this.state.data;
            let isPreviousMatched = latestData && latestData.acceptedMatch && latestData.acceptedMatch.id;

            this.goToRecord(next, false, isPreviousMatched);

        } else {
            alert("This is the last record.");
        }
    }

    prevRecord() {
        let prev = this.state.currentIndex - 1;
        if (prev >= 0) {
            // Latest Matched Data
            let latestData = this.state.allData[prev - 1];
            let isPreviousMatched = latestData && latestData.acceptedMatch && latestData.acceptedMatch.id;

            this.goToRecord(prev, true, isPreviousMatched);

        } else {
            alert("This is the first record.");
        }
    }

    jumpToRecord(event) {
        if (event.keyCode===13) {
            let recordNo = parseInt(event.target.value);
            this.goToRecord(recordNo - 1, true);
            event.target.value = "";
        }
    }

    goToRecord(index, revisit, isPreviousMatched) {
        if (index > -1 && this.state.allData && this.state.allData.length >= index + 1) {
            clearTimeout(this.timer);
            this.timer = setTimeout(function () {
                this.setState((state) => {
                    let wineInfo = state.allData[index];
                    let data = {};
                    data.retailer = wineInfo.retailerCode;
                    data.description = wineInfo.description;
                    data.bottleSize = wineInfo.bottleSize;
                    data.price = wineInfo.price;
                    data.url = wineInfo.url;
                    data.wineHistory = wineInfo.historyString;
                    data.wineOriginalKeyword = wineInfo.originalKeywordString;
                    data.wineKeyword = wineInfo.keywordString;
                    data.id = wineInfo.id;
                    data.mongoId = wineInfo.mongoId ? wineInfo.mongoId : "";
                    data.vintage = wineInfo.vintage;
                    data.wineDbId = wineInfo.wineDbId;
                    data.wineDbName = wineInfo.wineName;
                    data.pattern = wineInfo.pattern;
                    data.waNote = wineInfo.waNote;
                    data.color = wineInfo.color;
                    data.variety = wineInfo.variety;
                    data.sweetness = wineInfo.sweetness;
                    data.wineType = wineInfo.wineType;
                    data.country = wineInfo.country;
                    data.region = wineInfo.region;
                    data.location = wineInfo.location;
                    data.locale = wineInfo.locale;
                    data.site = wineInfo.site;
                    data.acceptedMatch = Object.assign({}, wineInfo.acceptedMatch ? wineInfo.acceptedMatch : {});

                    return {
                        data: data,
                        currentIndex: index,
                        narrowKeywords: [],
                        resetFilterID: true,
                        revisit: revisit,
                        selectedMatch: {},
                        isPreviousMatched: isPreviousMatched
                    };
                });
            }.bind(this), 2);
        } else {
            alert("Incorrect record number.");
        }
    }

    repeatPrevious() {
        let _this = this;
        if (_this.state.currentIndex === 0) {
            alert("This is the first record.");
            return;
        }

        let prevIndex = _this.state.currentIndex - 1;
        let prefInfo = _this.state.allData[prevIndex];
        let selectedMatch = Object.assign({}, prefInfo.acceptedMatch);
        _this.setState({selectedMatch: selectedMatch}, function () {
            // Reserved
        });
    }

    selectRetailers(val) {
        let activeRetailers = [];
        let i;
        for (i in val) {
            let v = val[i];
            activeRetailers.push(v);
        }
        this.setState({
            activeRetailers: activeRetailers
        });
    }

    loadData(event) {
        event.preventDefault();

        let matchedFilter = this.state.matchedFilter;
        if (this.state.fileSource === Constants.DATA_SOURCES.INTERNAL) {
            let codes = this.state.activeRetailers.map(function(row){
                return row.value;
            });

            if (codes.indexOf("all") >= 0) {
                codes = this.state.retailers.map(function(row){
                    return row.code;
                });
                codes = codes.filter(function(item){
                    return item !== "all";
                });
            }

            if (codes.length < 1) {
                alert("Retailer is required");
                return;
            }
            this.getData(codes, matchedFilter);
        } else {
            this.uploadFile(matchedFilter);
        }
    }

    deferWine() {
        this.markWine(Constants.WINE_MARKER.DEFER);
    }

    insufficientWine() {
        this.markWine(Constants.WINE_MARKER.INSUFFICIENT);
    }

    markWine(markType) {
        if (!this.state.data.wineHistory || !markType) return;
        let data = new FormData();
        data.append('wineDbId', this.state.data.wineDbId);
        data.append('wineHistory', this.state.data.wineHistory);
        data.append('wineOriginalKeyword', this.state.data.wineOriginalKeyword);
        data.append('wineVintage', this.state.data.vintage);
        data.append('wineBottleSize', this.state.data.bottleSize);
        data.append('wineRetailer', this.state.data.retailer);
        data.append('historyType', markType);
        data.append('validationType', Constants.VALIDATION_TYPE.MANUAL);
        this.deferOrInsufficientWine(data, this.nextRecord);
    }

    onSavedVintage(vintage) {
        let _this = this;
        let data = _this.state.data;
        data.vintage = vintage;

        let allData = _this.state.allData;
        let currentData = allData[_this.state.currentIndex];
        currentData.vintage = vintage;

        _this.setState({allData: allData, data: data}, function () {
            //console.log(_this.state.data.vintage);
        });
    }

    onSavedBottleSize(bottleSize) {
        let _this = this;
        let data = _this.state.data;
        data.bottleSize = bottleSize;

        let allData = _this.state.allData;
        let currentData = allData[_this.state.currentIndex];
        currentData.bottleSize = bottleSize;

        _this.setState({allData: allData, data: data}, function () {
            //console.log(_this.state.data.bottleSize);
        });
    }

    onSavedNote(note) {
        let _this = this;
        let data = _this.state.data;
        data.waNote = note;

        let allData = _this.state.allData;
        let currentData = allData[_this.state.currentIndex];
        currentData.waNote = note;

        _this.setState({allData: allData, data: data}, function () {
            //console.log(_this.state.data.waNote);
        });
    }

    acceptMatch() {
        let _this = this;
        let data = _this.state.data;
        let selected = _this.state.selectedMatch;
        if (selected && selected.id) {
            let oriDescription = data.description;

            _this.updateHistory(Constants.VALIDATION_TYPE.MANUAL, function () {
                data.wineDbId = selected.id;
                data.wineDbName = selected.name;
                data.color = selected.color;
                data.pattern = selected.pattern;
                data.variety = selected.variety;
                data.waNote = selected.waNote;
                data.country = selected.country;
                data.region = selected.region;
                data.location = selected.location;
                data.locale = selected.locale;
                data.site = selected.site;
                data.acceptedMatch = Object.assign({}, selected);

                let allData = _this.state.allData;
                let currentIndex = _this.state.currentIndex;
                let _oD = allData[currentIndex];
                _this.updateOriginal(_oD, selected);

                for (let i = currentIndex + 1; i < allData.length; i++) {
                    let _d = allData[i];
                    if (_d && _d.description === oriDescription) {
                        _this.updateOriginal(_d, selected);
                    }
                }

                _this.setState({allData: allData, data: data}, _this.nextRecord);
            });
        }
    }

    updateOriginal(original, selected) {
        original.wineDbId = selected.id
        original.wineName = selected.name;
        original.pattern = selected.pattern;
        original.waNote = selected.waNote;
        original.color = selected.color;
        original.variety = selected.variety;
        original.country = selected.country;
        original.region = selected.region;
        original.location = selected.location;
        original.locale = selected.locale;
        original.site = selected.site;
        original.acceptedMatch = Object.assign({}, selected);
    }

    resetMatch() {
        this.setState({
            selectedMatch: {}
        });
    }

    addNarrowWord(word) {
        let n = this.state.narrowKeywords;
        if (n.indexOf(word) === -1) {
            n.push(word);
            this.setState({
                narrowKeywords: n,
                selectedMatch: {}
            });
        }
    }

    removeNarrowWord(word) {
        let n = this.state.narrowKeywords;
        let index = n.indexOf(word);
        if (index > -1) {
            n.splice(index, 1);
            this.setState({
                narrowKeywords: n,
                resetFilterID: n && n.length < 1,
                selectedMatch: {}
            });
        }
    }

    refreshCandidates() {
        let _this = this;
        let words = _this.state.narrowKeywords;
        _this.setState({
            wineDbId: "",
            isEditingWine: false,
            producerId: "",
            isEditingProducer: false,
            narrowKeywords: [],
            resetFilterID: true
        }, function () {
            _this.setState({narrowKeywords: words});
        });
    }

    selectMatch(selectedMatch) {
        if (!selectedMatch || !selectedMatch.id) return;
        let selectedClone = Object.assign({}, selectedMatch);
        this.setState({selectedMatch: selectedClone});
    }

    createWineNewProducer() {
        this.createWine();
    }

    createWine(producerId) {
        let data = this.state.data;
        let wine = {
            vintage: data.vintage,
            sweetness: "Dry",
            wineType: "Table",
            producerId: producerId ? producerId : "",
            retailerCode: data.retailer,
            retailerDescription: data.description,
            retailerUrl: data.url
        };
        this.setState({wine: wine, isEditingWine: true});
    }

    cancelEditWine() {
        this.setState({wine: {}, wineDbId: "", isEditingWine: false});
    }

    editWineDb(wineDbId) {
        if (!wineDbId) return;
        this.setState({wineDbId: wineDbId, isEditingWine: true});
    }

    onWineDbSaved(wineDb) {
        if (wineDb) {
            let data = this.state.data;
            data.wineDbName = wineDb.wineDbName;
            data.pattern = wineDb.winePattern;
            this.setState({data: data}, this.refreshCandidates);
        } else {
            this.refreshCandidates();
        }
    }

    editProducer(producerId) {
        if (!producerId) return;
        this.setState({producerId: producerId, isEditingProducer: true});
    }

    cancelEditProducer() {
        this.setState({producerId: "", isEditingProducer: false});
    }

    onWineIdFilterCleared() {
        clearTimeout(this.wineIdFilterClearedTimer);
        this.wineIdFilterClearedTimer = setTimeout(function() {
            this.setState((state) => {
                return {resetFilterID: false};
            });
        }.bind(this), 2);
    }

    getRetailers() {
        let _this = this;
        let p1 = new Promise(function(resolve) {
            fetch(Constants.WORK_GROUPS_URL, {
                method: 'GET',
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                if (json.data.length > 0) {
                    resolve(json.data);
                } else {
                    resolve();
                }
            }).catch(function(ex) {
                console.debug('Parsing work groups data failed!', ex);
                resolve();
            });
        });

        let p2 = new Promise(function(resolve) {
            fetch(Constants.RETAILERS_URL, {
                method: 'GET',
                credentials: 'same-origin'
            }).then(function(response) {
                return response.json();
            }).then(function(json) {
                if (json.data.length > 0) {
                    resolve(json.data);
                } else {
                    resolve();
                }
            }).catch(function(ex) {
                console.debug('Parsing retailers data failed!', ex);
                resolve();
            });
        });

        let retailers = [{code: 'all', date: ''}];
        Promise.all([p1, p2]).then(function(res) {
            if (res[0]) retailers = retailers.concat(res[0]);
            if (res[1]) retailers = retailers.concat(res[1]);

            _this.setState({retailers: retailers});
        });
    }

    async getData(codes, matchedFilter) {
        this.setState({currentIndex: -1, isLoadingMatch: true});

        const retailerResponse = await fetch(Constants.POPULATE_RETAILERS_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                codes: codes
            }),
            credentials: 'same-origin'
        });
        if (!retailerResponse.ok) {
            throw Error(retailerResponse.statusText);
        }
        const retailerJson = await retailerResponse.json();
        let retailers = retailerJson.data;

        let records = [];
        try {
            for (let i in retailers) {
                const response = await fetch(Constants.DATA_URL + "?narrow=1", {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        retailer: retailers[i],
                        matchedFilter: matchedFilter
                    }),
                    credentials: 'same-origin'
                });
                if (!response.ok) {
                    throw Error(response.statusText);
                }
                const json = await response.json();
                if (json.data.matches && json.data.matches.length > 0) records = records.concat(json.data.matches);

                if (json.data.offsets && json.data.offsets.length > 0) {
                    for (let j in json.data.offsets) {
                        const offsetResponse = await fetch(Constants.DATA_URL + "?narrow=1", {
                            method: 'POST',
                            headers: {
                                'Content-Type': 'application/json'
                            },
                            body: JSON.stringify({
                                retailer: retailers[i],
                                matchedFilter: matchedFilter,
                                offset: json.data.offsets[j]
                            }),
                            credentials: 'same-origin'
                        });
                        if (!offsetResponse.ok) {
                            throw Error(offsetResponse.statusText);
                        }
                        const offsetJson = await offsetResponse.json();
                        if (offsetJson.data.matches && offsetJson.data.matches.length > 0) records = records.concat(offsetJson.data.matches);
                    }
                }
            }
        } catch (error) {
            alert(error);
        }

        records.sort(function(a, b) {
            let idA = a.id.toUpperCase(); // ignore upper and lowercase
            let idB = b.id.toUpperCase(); // ignore upper and lowercase
            if (idA < idB) {
                return -1;
            }
            if (idA > idB) {
                return 1;
            }
            // names must be equal
            return 0;
        });

        this.setState({allData: records, isLoadingMatch: false}, this.nextRecord);
    }

    async uploadFile(matchedFilter) {
        let _this = this;
        _this.setState({currentIndex: -1, isLoadingMatch: true});
        let input = document.querySelector('input[type="file"]');

        let data = new FormData();
        data.append('file', input.files[0]);
        data.append('matchedFilter', matchedFilter);

        fetch(Constants.UPLOAD_MATCHED_URL + "?narrow=1", {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.data.length > 0) {
                _this.setState({allData: json.data, isLoadingMatch: false}, function () {
                    _this.nextRecord();
                });
            } else {
                alert("No keyword match found.");
                _this.setState({isLoadingMatch: false});
            }
        }).catch(function(ex) {
            console.log('Parsing uploaded matched data failed!', ex);
            _this.setState({isLoadingMatch: false});
        });
    }

    async updateHistory(validationType, callback) {
        let data = new FormData();
        data.append('wineDbId', this.state.selectedMatch ? this.state.selectedMatch.id : '');
        data.append('wineHistory', this.state.data.wineHistory);
        data.append('wineOriginalKeyword', this.state.data.wineOriginalKeyword);
        data.append('wineVintage', this.state.data.vintage);
        data.append('wineBottleSize', this.state.data.bottleSize);
        data.append('wineRetailer', this.state.data.retailer);
        if (validationType) data.append('validationType', validationType);

        fetch(Constants.UPDATE_HISTORY, {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                if (typeof callback === "function") {
                    callback();
                }
            } else {
                alert(json.message);
            }
        }).catch(function(ex) {
            console.log('Parsing response data failed!', ex);
        });
    }

    async deferOrInsufficientWine(data, callback) {

        fetch(Constants.UPDATE_HISTORY, {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                if (typeof callback === "function") {
                    callback();
                }
            } else {
                alert(json.message);
            }
        }).catch(function(ex) {
            console.log('Parsing response data failed!', ex);
        });
    }

    render() {
        return (
            <div className="row">
                <div className="col-xs-12" hidden={this.state.isEditingWine || this.state.isEditingProducer}>
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <h2 className="title">Narrow Application</h2>
                        </div>
                    </div>
                    <div className="row wine-match-row retailer-form" style={{display: this.state.fileSource !== Constants.DATA_SOURCES.VALIDATION ? 'block' : 'none'}}>
                        <div className="col-xs-10">
                            <form className="form-inline" onSubmit={this.loadData}>
                                <div className="row row-form">
                                    <div className="col-md-1">
                                        <label className="control-label">Source</label>
                                    </div>
                                    <div className="col-md-11">
                                        <div className="form-group">
                                            <select className="form-control" name="fileSource" onChange={this.handleInputChange}>
                                                <option value="internal">Wine Match</option>
                                                <option value="external">External file</option>
                                            </select>
                                        </div>
                                        <div className="form-group">
                                            <label className="control-label">Matched Filter</label>
                                            <select className="form-control" name="matchedFilter" onChange={this.handleInputChange}>
                                                <option value="all">All Wines</option>
                                                <option value="rp wines">RP Wines</option>
                                                <option value="non rp wines">Non RP Wines</option>
                                            </select>
                                        </div>
                                        <button type="submit" className="btn btn-default" disabled={this.state.isLoadingMatch}>{this.state.isLoadingMatch ? 'Loading...' : 'Load Data'}</button>
                                    </div>
                                </div>
                                <div className="row row-form" hidden={this.state.fileSource !== Constants.DATA_SOURCES.INTERNAL}>
                                    <div className="col-md-1">
                                        <label className="control-label">Retailers</label>
                                    </div>
                                    <div className="col-md-11">
                                        <div className="section">
                                            <Select
                                                className="retailers-selection"
                                                name="activeRetailers"
                                                placeholder="Select retailers"
                                                value={this.state.activeRetailers}
                                                options={this.state.retailers.map(
                                                    retailer => ({label: retailer.code === 'all' ? 'All retailers' : retailer.code + ' - ' + retailer.date, value: retailer.code})
                                                )}
                                                isMulti={true}
                                                onChange={this.selectRetailers}
                                            />
                                        </div>
                                    </div>
                                </div>
                                <div className="row row-form" hidden={this.state.fileSource === Constants.DATA_SOURCES.INTERNAL}>
                                    <div className="col-md-1">
                                        <label className="control-label">File</label>
                                    </div>
                                    <div className="col-md-11">
                                        <input type="file" name="file" className="form-control" onChange={this.handleInputChange}/>
                                    </div>
                                </div>
                            </form>
                        </div>
                        <div className="col-xs-2 text-right">
                            <span><strong>{this.state.allData.length > 0 ? this.state.currentIndex + 1 : 0} of {this.state.allData.length}</strong></span>
                            <div className="form-group form-inline">
                                <label className="control-label">Go to</label>
                                <input type="text" className="form-control go-to-record" name="jumpToRecord" onKeyDown={this.jumpToRecord} />
                            </div>
                        </div>
                    </div>

                    <RetailerWine wine={this.state.data} isDataExists={this.state.allData.length > 0} selectedMatch={this.state.selectedMatch}
                                  onSavedVintage={this.onSavedVintage} onSavedBottleSize={this.onSavedBottleSize} addWord={this.addNarrowWord} editWineDb={this.editWineDb} />

                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <ul className="list-inline btn-list">
                                <li><button className="btn btn-md btn-default" onClick={this.createWineNewProducer} disabled={this.state.currentIndex < 0}>Add Wine New Producer</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.currentIndex < 1}
                                            onClick={this.prevRecord}>Back</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || this.state.allData.length === this.state.currentIndex + 1}
                                            onClick={this.nextRecord}>Next (without match)</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || !this.state.selectedMatch.id || this.state.data.acceptedMatch.id}
                                            onClick={this.acceptMatch}>Accept Match</button></li>
                                <li><button className="btn btn-md btn-default" disabled={!this.state.isPreviousMatched}
                                            onClick={this.repeatPrevious}>Repeat Previous</button></li>
                                <li><button className="btn btn-md btn-default" disabled={!this.state.selectedMatch.id} onClick={this.resetMatch}>Reset Match</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || this.state.allData.length === this.state.currentIndex + 1}
                                            onClick={this.deferWine}>X-Defer</button></li>
                                <li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1 || this.state.allData.length === this.state.currentIndex + 1}
                                            onClick={this.insufficientWine}>Z-Insufficient</button></li>
                            </ul>
                        </div>
                    </div>

                    <Candidates words={this.state.narrowKeywords} selectedMatch={this.state.selectedMatch} selectMatch={this.selectMatch}
                                addWord={this.addNarrowWord} removeWord={this.removeNarrowWord}
                                createWine={this.createWine} editWineDb={this.editWineDb} editProducer={this.editProducer}
                                resetFilterID={this.state.resetFilterID} onWineIdFilterCleared={this.onWineIdFilterCleared}
                    />

                    <ExternalResources wine={this.state.data} wineBaseUrl={Constants.RP_PROD_BASE_URL} showNote={true} onSavedNote={this.onSavedNote} />
                </div>
                <div className="col-xs-12">
                    <WineEditor shouldShow={this.state.isEditingWine} wineDbId={this.state.wineDbId}
                                producerId={this.state.wine.producerId}
                                wine={this.state.wine} source={Constants.SOURCE.NARROW}
                                onUpdated={this.onWineDbSaved} onSaved={this.refreshCandidates} onCanceled={this.cancelEditWine} />
                </div>
                <div className="col-xs-12">
                    <ProducerEditor shouldShow={this.state.isEditingProducer} producerId={this.state.producerId} source={Constants.SOURCE.NARROW}
                                onUpdated={this.refreshCandidates} onCanceled={this.cancelEditProducer} />
                </div>
            </div>
        );
    }
}

export default Narrow;