import React from 'react';
import * as Constants from '../constants';
import Keyword from "./Keyword";
import SlicedKeyword from "./SlicedKeyword";

class NoiseParser extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
            allData: [],
            data: {
                wineDbId: "",
                description: "",
                pattern: "",
                wineKeyword: ""
            },
            narrowKeywords: [],
            currentIndex: -1,
            isLoadingMatch: false,
            abort: false
		};
        this.timer = null;

        this.beforeUnload = this.beforeUnload.bind(this);
        this.uploadFile = this.uploadFile.bind(this);

        this.nextRecord = this.nextRecord.bind(this);
        this.prevRecord = this.prevRecord.bind(this);
        this.abort = this.abort.bind(this);
	}

    componentWillMount() {
    }

	componentDidMount() {
        window.addEventListener("beforeunload", this.beforeUnload);
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillUnmount() {
        window.removeEventListener("beforeunload", this.beforeUnload);
    }

    beforeUnload(e) {
        if (this.state.allData && this.state.allData.length > 0 && this.state.allData.length > this.state.currentIndex + 1) {
            e.preventDefault();
            e.returnValue = true;
        }
    }

    nextRecord() {
        if (this.state.allData.length > this.state.currentIndex + 1 && !this.state.abort) {
            let _this = this;
            clearTimeout(this.timer);
            this.timer = setTimeout(function() {
                _this.setState((state) => {
                    return {
                        data: state.allData[state.currentIndex + 1],
                        currentIndex: state.currentIndex + 1,
                        narrowKeywords: []
                    };
                });
            }.bind(this), 5);
        } else {
            alert("This is the last record.");
        }
    }

    prevRecord() {
        if (this.state.currentIndex - 1 >= 0) {
            clearTimeout(this.timer);
            this.timer = setTimeout(function() {
                this.setState((state) => {
                    return {
                        data: state.allData[state.currentIndex - 1],
                        currentIndex: state.currentIndex - 1,
                        narrowKeywords: []
                    };
                });
            }.bind(this), 5);
        } else {
            alert("This is the first record.");
        }
    }

    abort() {
	    this.setState({abort: true});
    }

    async uploadFile(event) {
        event.preventDefault();
        let _this = this;
        _this.setState({narrowKeywords: [], currentIndex: -1, isLoadingMatch: true});
        let input = document.querySelector('input[type="file"]');

        let data = new FormData();
        data.append('file', input.files[0]);

        fetch(Constants.UPLOAD_WINES_FOR_NOISES_URL, {
            method: 'POST',
            body: data,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                if (json.data.length > 0) {
                    _this.setState({allData: json.data, isLoadingMatch: false}, _this.nextRecord);
                } else {
                    throw new Error("No record found.");
                }
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            _this.setState({isLoadingMatch: false});
            alert(ex);
        });
    }

	render() {
		return (
			<div className="row">
				<div className="col-xs-12">
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <h2 className="title">Noise Parser</h2>
                        </div>
                    </div>
                    <div className="row wine-match-row retailer-form">
                        <div className="col-xs-10">
                            <form className="form-inline" onSubmit={this.uploadFile}>
                                <div className="row row-form">
                                    <div className="col-md-1">
                                        <label className="control-label">File</label>
                                    </div>
                                    <div className="col-md-4 text-right">
                                        <input type="file" name="file" className="form-control" onChange={this.handleInputChange}/>
                                    </div>
                                    <div className="col-md-7 text-left">
                                        <button type="submit" className="btn btn-default" disabled={this.state.isLoadingMatch}>{this.state.isLoadingMatch ? 'Loading...' : 'Load Data'}</button>
                                    </div>
                                </div>
                            </form>
                        </div>
                        <div className="col-xs-2 text-right">
                            <span><strong>{this.state.allData.length > 0 ? this.state.currentIndex + 1 : 0} of {this.state.allData.length}</strong></span>
                        </div>
                    </div>
                    <div className="row wine-match-row">
                        <table className="table table-bordered">
                            <tbody>
                            <tr>
                                <td style={{width: '15%'}}><b>WineDb ID</b></td>
                                <td colSpan={3}>{this.state.data.wineDbId}</td>
                            </tr>
                            <tr>
                                <td style={{width: '15%'}}><b>Description</b></td>
                                <td colSpan={3}>{this.state.data.description}</td>
                            </tr>
                            <tr>
                                <td style={{width: '15%'}}><b>Keyword</b></td>
                                <td colSpan={3}>
                                    <div className="row">
                                        <div className="col-xs-12 wine-match-fixed-height break-word">
                                            <SlicedKeyword keyword={this.state.data.pattern} />
                                        </div>
                                    </div>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </div>
                    <div className="row wine-match-row">
                        <Keyword keywordInput={this.state.data.wineKeyword}
                                 wineDbId={this.state.data.wineDbId}
                                 pattern={this.state.data.pattern}
                                 recordNo={this.state.currentIndex}
                                 autoUpdateNoises={true}
                                 onMatched={this.nextRecord} />
                    </div>
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <ul className="list-inline btn-list">
                                {/*<li><button className="btn btn-md btn-default" disabled={this.state.allData.length < 1} onClick={this.abort}>Abort</button></li>*/}
                                {/*<li><button className="btn btn-md btn-default" disabled={this.state.currentIndex < 1 || this.state.isCheckingMatched}
                                            onClick={this.prevRecord}>Back</button></li>*/}
                                <li><button className="btn btn-md btn-default"
                                            disabled={this.state.allData.length < 1 || this.state.allData.length === this.state.currentIndex + 1 || this.state.isLoadingMatch}
                                            onClick={this.nextRecord}>Next (skip noises)</button></li>
                            </ul>
                        </div>
                    </div>
				</div>
			</div>
		);
	}
}

export default NoiseParser;