import React from 'react';
import * as Constants from '../constants';
import {EscapeRegExp, ReplacePunctuationsForKeyword} from "../helpers/helper";

class ProducerEditor extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            regions: [],
            reverseAliases: [],
            producerEliminations: [],
            data: {
                //regionCode: "",
                name: "",
                nameShow: "",
                keyword: "",
            },
            isSaving: false,
            lastInputFocus: "name"
        };

        //this.getRegionCodes = this.getRegionCodes.bind(this);

        this.getProducer = this.getProducer.bind(this);

        this.getReverseAliases = this.getReverseAliases.bind(this);
        this.getProducerEliminations = this.getProducerEliminations.bind(this);
        this.globalSubstitute = this.globalSubstitute.bind(this);

        this.generateProducerKeyword = this.generateProducerKeyword.bind(this);
        this.proceedProducerEliminations = this.proceedProducerEliminations.bind(this);
        this.replaceWithReverseAliases = this.replaceWithReverseAliases.bind(this);

        this.getProducerData = this.getProducerData.bind(this);
        this.updateProducer = this.updateProducer.bind(this);
        this.updateExistingProducer = this.updateExistingProducer.bind(this);

        this.handleInputChange = this.handleInputChange.bind(this);
        this.handleOnFocus = this.handleOnFocus.bind(this);
    }

    componentWillMount() {
        this.getReverseAliases();
        this.getProducerEliminations();
    }

    componentDidMount() {
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillReceiveProps(nextProps) {
        if (!nextProps.shouldShow) return;
        if (nextProps.producerId) { // Editing wineDb
            //let regionCode = nextProps.wineDbId.substring(0, 2);
            this.getProducer(nextProps.producerId);
        }
    }

    handleInputChange(event) {
        if (!event.target) return;
        const target = event.target;
        const name = target.name;
        let value = target.type === 'checkbox' ? target.checked : target.value;

        if (name === 'date') {
            let date = target.value;
            value = date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
        }

        let data = this.state.data;
        data[name] = value;

        this.setState({data: data}, function () {
            // Reserved
        });
    }

    handleOnFocus(e) {
        if (!e.target) return;
        let name = e.target.name;
        let inputs = ['name', 'nameShow'];
        if (inputs.indexOf(inputs) && name !== this.state.lastInputFocus) {
            this.setState({lastInputFocus: name})
        }
    }

    globalSubstitute(kw) {
        if (!kw) return '';
        kw = kw.toLowerCase(); // Lower case
        kw = ReplacePunctuationsForKeyword(kw, " "); // Remove punctuation
        kw = this.replaceWithReverseAliases(kw); // Correct reverse aliases using table
        return kw;
    }

    generateProducerKeyword() {
        if (!this.state.data.nameShow) return "";
        let producerKw = this.state.data.nameShow;
        producerKw = this.globalSubstitute(producerKw);
        producerKw = this.proceedProducerEliminations(producerKw); //7	remove producer eliminations using table
        producerKw = producerKw.replace(new RegExp('\\s+', 'g'), '&').trim(); //9	replace spaces with &'s

        let data = this.state.data;
        data.keyword = producerKw;
        this.setState({data: data});
    }

    proceedProducerEliminations(text) {
        let producerEliminations = this.state.producerEliminations;
        for (let i in producerEliminations) {
            let re = producerEliminations[i];
            let rs = new RegExp('\\b' + EscapeRegExp(re) + '\\b', 'gi');
            text = text.replace(rs, '');
        }
        return text;
    }

    replaceWithReverseAliases(text) {
        let array = this.state.reverseAliases;
        for (let i in array) {
            let obj = array[i];
            let key = Object.keys(obj)[0];
            let rs = new RegExp('\\b' + EscapeRegExp(key) + '\\b', 'gi');
            text = text.replace(rs, obj[key]);
        }
        return text;
    }

    async getProducer(producerId) {
        if (producerId) {
            const rawData = await fetch(Constants.PRODUCER_URL + "?id=" + producerId, {
                credentials: 'same-origin'
            });
            const res = await rawData.json();

            let _this = this;
            this.setState({
                data: {
                    name: res.data.name,
                    nameShow: res.data.nameShow,
                    keyword: res.data.keyword
                },
                lastInputFocus: "name"
            }, function () {
                if (!res.data.keyword) _this.generateProducerKeyword();
            });
        }
    }

    getProducerData() {
        let data = this.state.data;
        let formData = new FormData();
        formData.append('name', typeof data.name !== "undefined" ? data.name : '');
        formData.append('nameShow', typeof data.nameShow !== "undefined" && data.nameShow ? data.nameShow : '');
        formData.append('keyword', typeof data.keyword !== "undefined" && data.keyword ? data.keyword : '');
        formData.append('source', this.props.source ? this.props.source : '');
        return formData;
    }

    updateProducer() {
        let _this = this;
        let wineData = _this.getProducerData();
        _this.updateExistingProducer(wineData);
    }

    async updateExistingProducer(formData) {
        let _this = this;
        _this.setState({isSaving: true});
        fetch(Constants.UPDATE_PRODUCER + "?producerId=" + this.props.producerId, {
            method: 'POST',
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                _this.setState({isSaving: false}, function () {
                    _this.props.onUpdated();
                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isSaving: false});
        });
    }

    async getReverseAliases() {
        const rawData = await fetch(Constants.REVERSE_ALIASES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by key length, descx
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({reverseAliases: sorted});
    }

    async getProducerEliminations() {
        const rawData = await fetch(Constants.ELIMINATIONS_URL + "?type=" + Constants.ELIMINATION_TYPE.PRODUCER, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by word length, desc
            return b.length - a.length;
        });
        this.setState({producerEliminations: sorted});
    }

    render() {
        return (
            <div className="row" hidden={typeof this.props.shouldShow !== 'undefined' && !this.props.shouldShow}>
                <div className="col-xs-12">
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center"><p>&nbsp;</p></div>
                    </div>
                    <div className="row wine-match-row">
                        <div className="col-xs-8 col-xs-offset-2">
                            <div className="row">
                                <div className="col-xs-6">
                                    <h3 className="title">Edit Producer</h3>
                                </div>
                                <div className="col-xs-6 text-right">
                                    <h3 className="title">{this.props.producerId}</h3>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12 winedb-data">
                                    <div className="row">
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Producer Name</label>
                                                <input type="text" className="form-control" name="name" value={this.state.data.name} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.nameTxt = input;}}/>
                                            </div>
                                        </div>
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Producer Show</label>
                                                <input type="text" className="form-control" name="nameShow" value={this.state.data.nameShow} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.nameShowTxt = input;}}/>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-xs-10">
                                            <div className="form-group">
                                                <label className="control-label">Keyword</label>
                                                <input type="text" className="form-control" name="keyword" value={this.state.data.keyword} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.keywordTxt = input;}}/>
                                            </div>
                                        </div>
                                        <div className="col-xs-2">
                                            <div className="form-group pull-right">
                                                <br />
                                                <button className="btn btn-default" onClick={this.generateProducerKeyword}>Re-Generate</button>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12">
                                    <div className="form-group">
                                        <button className="btn btn-default" onClick={this.props.onCanceled} disabled={this.state.isSaving}>Cancel</button>&nbsp;&nbsp;
                                        <button className="btn btn-primary" onClick={this.updateProducer} disabled={this.state.isSaving}>{this.state.isSaving ? 'Updating...' : 'Update'}</button>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}

export default ProducerEditor;