import React from 'react';
import * as Constants from "../constants";
import ClickableWord from "./ClickableWord";
import {WineDescription} from "../helpers/helper";
import SlicedKeyword from "./SlicedKeyword";

class RetailerWine extends React.Component {
    constructor() {
        super();
        this.state = {
            vintage: "",
            bottleSize: "",
            bottleSizes: [],
            vintages: [],
            isEditingVintage: false,
            isEditingBottleSize: false
        };

        this.handleInputChange = this.handleInputChange.bind(this);

        this.getVintages = this.getVintages.bind(this);
        this.getBottleSizes = this.getBottleSizes.bind(this);

        this.addWord = this.addWord.bind(this);

        this.editVintage = this.editVintage.bind(this);
        this.saveVintage = this.saveVintage.bind(this);
        this.cancelVintage = this.cancelVintage.bind(this);

        this.editBottleSize = this.editBottleSize.bind(this);
        this.saveBottleSize = this.saveBottleSize.bind(this);
        this.cancelBottleSize = this.cancelBottleSize.bind(this);

        this.editWineDb = this.editWineDb.bind(this);
    }

    handleInputChange(event) {
        const target = event.target;
        const name = target.name;

        let value = target.type === 'checkbox' ? target.checked : target.value;

        if (name === 'date') {
            let date = target.value;
            value = date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
        }

        let stateData = this.state;
        stateData[name] = value;
        this.setState(stateData);
    }

    componentWillMount() {
        let _this = this;

        _this.getBottleSizes();
        _this.getVintages();
    }

    componentDidMount() {
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillReceiveProps(nextProps) {
        let wine = JSON.parse(JSON.stringify(nextProps.wine));
        if (wine) {
            this.setState({
                vintage: wine.vintage ? wine.vintage : "",
                bottleSize: wine.bottleSize ? wine.bottleSize : ""
            });
        }
    }

    addWord(word) {
        if (typeof this.props.addWord === "function") this.props.addWord(word);
    }

    editVintage() {
        this.setState({isEditingVintage: !this.state.isEditingVintage});
    }

    saveVintage() {
        let _this = this;
        if (!_this.state.vintage) return;

        _this.setState({isEditingVintage: false}, function () {
            _this.props.onSavedVintage(_this.state.vintage);
        });
    }

    cancelVintage() {
        let _this = this;
        let wine = JSON.parse(JSON.stringify(_this.props.wine));
        if (wine) {
            _this.setState({vintage: wine.vintage, isEditingVintage: false});
        }
    }

    editBottleSize() {
        this.setState({isEditingBottleSize: !this.state.isEditingBottleSize});
    }

    saveBottleSize() {
        let _this = this;
        if (!_this.state.bottleSize) return;

        _this.setState({isEditingBottleSize: false}, function () {
            _this.props.onSavedBottleSize(_this.state.bottleSize);
        });
    }

    cancelBottleSize() {
        let _this = this;
        let wine = JSON.parse(JSON.stringify(_this.props.wine));
        if (wine) {
            _this.setState({bottleSize: wine.bottleSize, isEditingBottleSize: false});
        }
    }

    editWineDb(e) {
        if (!e.currentTarget) return;

        if (typeof this.props.editWineDb === "function") {
            let wineDbId = e.currentTarget.getAttribute('data-id');
            this.props.editWineDb(wineDbId);
        }
    }

    async getBottleSizes() {
        const rawData = await fetch(Constants.BOTTLES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({bottleSizes: res.data});
    }

    async getVintages() {
        const rawData = await fetch(Constants.VINTAGES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({vintages: res.data});
    }

    render() {
        return (
            <div>
                <div className="row wine-match-row">
                    <table className="table table-bordered">
                        <thead className="thead-default">
                        <tr>
                            <th className="default" style={{width: '20%'}}>Retailer</th>
                            <th className="default" style={{width: '60%'}}>Actual Retailer Description</th>
                            <th className="default" style={{width: '20%'}}>Standardized Price</th>
                        </tr>
                        </thead>
                        <tbody>
                        <tr>
                            <td>{this.props.wine ? this.props.wine.retailer : ''}</td>
                            <td><ClickableWord phrase={this.props.wine ? this.props.wine.description : ''} onClick={this.addWord} /></td>
                            <td className="text-center">{this.props.wine ? this.props.wine.price : ''}</td>
                        </tr>
                        </tbody>
                    </table>
                </div>
                <div className="row wine-match-row">
                    <div className="col-xs-12 text-center">
                        <h3 className="title">Proposed Match</h3>
                    </div>
                </div>
                <div className="row wine-match-row">
                    <table className="table table-bordered">
                        <thead className="thead-default">
                        <tr>
                            <th className="default" style={{width: '10%'}}>WineAlert ID</th>
                            <th className="default" style={{width: '20%'}}>Vintage</th>
                            <th className="default" style={{width: '35%'}}>Bottle Size</th>
                            <th className="default" style={{width: '35%'}}>WineAlert Description</th>
                        </tr>
                        </thead>
                        <tbody>
                        <tr>
                            <td>
                                <div className="col-xs-12 wine-match-fixed-height">
                                    {this.props.selectedMatch && this.props.selectedMatch.id ? this.props.selectedMatch.id : (this.props.wine ? this.props.wine.wineDbId : '')}
                                </div>
                            </td>
                            <td>
                                <div className="row" hidden={this.state.isEditingVintage || !this.props.isDataExists}>
                                    <div className="col-xs-12 wine-match-fixed-height">
                                        {this.state.vintage}
                                    </div>
                                    <div className="col-xs-12">
                                        <ul className="list-inline btn-list">
                                            <li><a href="javascript:void(0)" onClick={this.editVintage}>Edit Vintage</a></li>
                                        </ul>
                                    </div>
                                </div>
                                <div className="row" hidden={!this.state.isEditingVintage}>
                                    <div className="col-xs-12">
                                        <select className="form-control" id="vintage" name="vintage"
                                                onChange={this.handleInputChange} value={this.state.vintage}
                                                ref={(select) => { this.vintagesSelectBox = select; }}>
                                            {this.state.vintages.map(function(row, i) {
                                                return (
                                                    <option key={i} value={row}>{row}</option>
                                                );
                                            })}
                                        </select>
                                    </div>
                                    <div className="col-xs-12">
                                        <ul className="list-inline btn-list">
                                            <li><button className="btn btn-md btn-default" onClick={this.cancelVintage}>Cancel</button></li>
                                            <li><button className="btn btn-md btn-primary" onClick={this.saveVintage}>Save</button></li>
                                        </ul>
                                    </div>
                                </div>
                            </td>
                            <td>
                                <div className="row" hidden={this.state.isEditingBottleSize || !this.props.isDataExists}>
                                    <div className="col-xs-12 wine-match-fixed-height">
                                        {this.state.bottleSize}
                                    </div>
                                    <div className="col-xs-12">
                                        <ul className="list-inline btn-list">
                                            <li><a href="javascript:void(0)" onClick={this.editBottleSize}>Edit Bottle Size</a></li>
                                        </ul>
                                    </div>
                                </div>
                                <div className="row" hidden={!this.state.isEditingBottleSize}>
                                    <div className="col-xs-12">
                                        <select className="form-control" id="bottleSize" name="bottleSize"
                                                onChange={this.handleInputChange} value={this.state.bottleSize}
                                                ref={(select) => { this.bottlesSelectBox = select; }}>
                                            {this.state.bottleSizes.map(function(row, i) {
                                                return (
                                                    <option key={i} value={row}>{row}</option>
                                                );
                                            })}
                                        </select>
                                    </div>
                                    <div className="col-xs-12">
                                        <ul className="list-inline btn-list">
                                            <li><button className="btn btn-md btn-default" onClick={this.cancelBottleSize}>Cancel</button></li>
                                            <li><button className="btn btn-md btn-primary" onClick={this.saveBottleSize}>Save</button></li>
                                        </ul>
                                    </div>
                                </div>
                            </td>
                            <td>
                                <div className="row">
                                    <div className="col-xs-12 wine-match-fixed-height">
                                        {this.props.selectedMatch && this.props.selectedMatch.name ? WineDescription(this.props.selectedMatch.name, this.props.selectedMatch.color) :
                                            (this.props.wine ? WineDescription(this.props.wine.wineDbName, this.props.wine.color) : "")}
                                    </div>
                                    <div className="col-xs-12" hidden={!this.props.wine || !this.props.wine.wineDbId}>
                                        <ul className="list-inline btn-list">
                                            <li><a href="javascript:void(0)" onClick={this.editWineDb} data-id={this.props.wine.wineDbId}>Edit WineAlert DB</a></li>
                                        </ul>
                                    </div>
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td colSpan="4">
                                <div className="row">
                                    <div className="col-xs-12"><strong>Matched Keyword Set</strong></div>
                                </div>
                                <div className="row" style={{display: this.state.isEditingPattern ? 'none' : 'block'}}>
                                    <div className="col-xs-12 wine-match-fixed-height break-word">
                                        <SlicedKeyword keyword={this.props.selectedMatch && this.props.selectedMatch.pattern ? this.props.selectedMatch.pattern : this.props.wine.pattern} />
                                    </div>
                                </div>
                            </td>
                        </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        );
    }
}

export default RetailerWine;