import React from 'react';
import DiffMatchPatch from 'diff-match-patch';

const DIFF_DELETE = -1;
const DIFF_INSERT = 1;
const DIFF_EQUAL = 0;

class SlicedKeyword extends React.Component {
    constructor() {
        super();
        this.dmp = new DiffMatchPatch();
    }

    render() {
        let _this = this;
        let keyword = _this.props.keyword;
        if (!keyword) keyword = '';
        if (_this.props.compareWith) {
            let diffs = _this.dmp.diff_main(keyword, _this.props.compareWith);
            _this.dmp.diff_cleanupSemantic(diffs);
            let html = [];
            for (var x = 0; x < diffs.length; x++) {
                var op = diffs[x][0];    // Operation (insert, delete, equal)
                var data = diffs[x][1];  // Text of change.
                switch (op) {
                    case DIFF_INSERT:
                        html[x] = <ins key={x} className="diff-ins">{data}</ins>;
                        break;
                    case DIFF_DELETE:
                        html[x] = <del key={x} className="diff-del">{data}</del>;
                        break;
                    case DIFF_EQUAL:
                        html[x] = <span key={x}>{data}</span>;
                        break;
                }
            }
            return html;
        }

        return keyword.split(/(?=[&|])/).map(function (row, i) {
            return <span key={i} className="autoveted-long-string">{row}</span>;
        });
    }
}

export default SlicedKeyword;