import React from 'react';
import Select from 'react-select';
import Merge from './Merge.jsx';
import * as Constants from '../constants';
import ClickableWord from "./ClickableWord";
import ExternalResources from "./ExternalResources";
import {
    CheckForInvalidKeyword,
    CleanUpAnds,
    EscapeRegExp,
    ReplacePunctuationsForKeyword,
    StripAccents
} from "../helpers/helper";
import SlicedKeyword from "./SlicedKeyword";
import {VIP_VARIETIES} from "../constants";
import Creatable from 'react-select/creatable';

class WineEditor extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            vintagesReverseMap: {},
            varieties: [],
            varietiesSelect: [],
            wineColors: [],
            wineSweetness: [],
            wineTypes: [],
            producers: [],
            producersSelect: [],
            regionCodes: [],
            countries: [],
            regions: [],
            locations: [],
            locales: [],
            sites: [],
            appellations: [],
            reverseAliases: [],
            producerEliminations: [],
            wineEliminations: [],
            massChanges: [],
            customChanges: [],
            spreadTables: {},
            colorSubstitutions: [],
            colorNotSubstitutions: [],
            redOpposites: [],
            reserveWords: [],
            roseOpposites: [],
            whiteOpposites: [],
            data: {},
            existingWines: [],
            selectedCountry: {label: '', value: ''},
            selectedRegion: {label: '', value: ''},
            selectedLocation: {label: '', value: ''},
            selectedLocale: {label: '', value: ''},
            selectedSite: {label: '', value: ''},
            selectedAppellation: {label: '', value: ''},
            isProducersLoading: false,
            isVarietiesLoading: false,
            isCountriesLoading: false,
            isRegionsLoading: false,
            isLocationsLoading: false,
            isLocalesLoading: false,
            isSitesLoading: false,
            isAppellationsLoading: false,
            isSaving: false,
            selectedWines: [],
            disableProducer: false,
            lastInputFocus: "wineDbName",
            noiseMode: false
        };
        this.defaultData = {
            id: "",
            nextId: "",
            mongoId: "",
            wineVinId: "",
            regionCode: "",
            vintage: "",
            wineDbId: "",
            wineDbName: "",
            winePattern: "",
            waNote: "",
            color: "",
            variety: "",
            sweetness: "",
            wineType: "Table",
            country: "",
            region: "",
            location: "",
            locale: "",
            site: "",
            producerId: "",
            producerName: "",
            producerShow: "",
            producerKeyword: "",
            baseKeyword: "",
            appellationKeyword: "",
            colorKeyword: "",
            commonModifiersKeyword: "",
            modifiersKeyword: "",
            customNotKeyword: "",
            globalNotKeyword: "",
            retailerCode: "",
            retailerDescription: "",
            retailerUrl: "",
            noise: [],
            producerNoises: []
        };

        this.getVintagesReverseMap = this.getVintagesReverseMap.bind(this);
        this.getVarieties = this.getVarieties.bind(this);
        this.getColors = this.getColors.bind(this);
        this.getSweetness = this.getSweetness.bind(this);
        this.getTypes = this.getTypes.bind(this);
        this.getCountries = this.getCountries.bind(this);
        this.getRegions = this.getRegions.bind(this);
        this.getLocations = this.getLocations.bind(this);
        this.getLocales = this.getLocales.bind(this);
        this.getSites = this.getSites.bind(this);
        this.getAppellations = this.getAppellations.bind(this);
        this.getReverseAliases = this.getReverseAliases.bind(this);
        this.getProducerEliminations = this.getProducerEliminations.bind(this);
        this.getMassChanges = this.getMassChanges.bind(this);
        this.getCustomChanges = this.getCustomChanges.bind(this);
        this.getSpreadTables = this.getSpreadTables.bind(this);
        this.getColorSubstitutions = this.getColorSubstitutions.bind(this);
        this.getColorNotSubstitutions = this.getColorNotSubstitutions.bind(this);
        this.getRedOpposites = this.getRedOpposites.bind(this);
        this.getReserveWords = this.getReserveWords.bind(this);
        this.getRoseOpposites = this.getRoseOpposites.bind(this);
        this.getWhiteOpposites = this.getWhiteOpposites.bind(this);

        this.getRegionCodes = this.getRegionCodes.bind(this);
        this.getProducers = this.getProducers.bind(this);

        this.getWineData = this.getWineData.bind(this);

        this.saveWine = this.saveWine.bind(this);
        this.saveNewWine = this.saveNewWine.bind(this);
        this.updateWineDb = this.updateWineDb.bind(this);
        this.updateExistingWineDb = this.updateExistingWineDb.bind(this);
        this.cancel = this.cancel.bind(this);

        this.handleInputChange = this.handleInputChange.bind(this);
        this.handleOnFocus = this.handleOnFocus.bind(this);
        this.handleBooleanCheck = this.handleBooleanCheck.bind(this);
        this.filterOption = this.filterOption.bind(this);
        this.selectProducer = this.selectProducer.bind(this);
        this.selectVariety = this.selectVariety.bind(this);
        this.getSelectedVariety = this.getSelectedVariety.bind(this);
        this.selectCountry = this.selectCountry.bind(this);
        this.selectRegion = this.selectRegion.bind(this);
        this.selectLocation = this.selectLocation.bind(this);
        this.selectLocale = this.selectLocale.bind(this);
        this.selectSite= this.selectSite.bind(this);
        this.selectAppellation= this.selectAppellation.bind(this);
        this.selectWine = this.selectWine.bind(this); // For WineMerge
        this.getNextId = this.getNextId.bind(this);

        this.wordClick = this.wordClick.bind(this);
        this.handleNoiseChange = this.handleNoiseChange.bind(this);
        this.removeNoise = this.removeNoise.bind(this);
        this.addNoise = this.addNoise.bind(this);

        this.refreshExistingWines = this.refreshExistingWines.bind(this);

        this.generateProducerKeyword = this.generateProducerKeyword.bind(this);
        this.globalSubstitute = this.globalSubstitute.bind(this);
        this.substitute = this.substitute.bind(this);
        this.substituteSpreads = this.substituteSpreads.bind(this);
        this.generateWineKeyword = this.generateWineKeyword.bind(this);
        this.reGenerateKeyword = this.reGenerateKeyword.bind(this);
        this.removeNoises = this.removeNoises.bind(this);
        this.cleanupKeywordNoises = this.cleanupKeywordNoises.bind(this);
        this.replaceWithReverseAliases = this.replaceWithReverseAliases.bind(this);
        this.proceedProducerEliminations = this.proceedProducerEliminations.bind(this);
    }

    componentWillMount() {
        let _this = this;
        let defaultData = Object.assign({}, this.defaultData);
        this.setState({data: defaultData}, function () {
            _this.getVintagesReverseMap();
            _this.getVarieties();
            _this.getColors();
            _this.getSweetness();
            _this.getTypes();
            _this.getRegionCodes();
            _this.getReverseAliases();
            _this.getProducerEliminations();
            _this.getWineEliminations();
            _this.getMassChanges();
            _this.getCustomChanges();
            _this.getSpreadTables();
            _this.getColorSubstitutions();
            _this.getColorNotSubstitutions();
            _this.getRedOpposites();
            _this.getReserveWords();
            _this.getRoseOpposites();
            _this.getWhiteOpposites();

            _this.getCountries();
            _this.getRegions();
            _this.getLocations();
            _this.getLocales();
            _this.getSites();
            _this.getAppellations();
        });
    }

    componentDidMount() {
    }

    componentDidUpdate(prevProps, prevState) {
        if (this.state.data && prevState.data) {
            if (!this.props.wineDbId) {
                if (this.state.data.producerShow !== prevState.data.producerShow) {
                    this.generateProducerKeyword(this.generateWineKeyword);
                } else if (this.state.data.regionCode !== prevState.data.regionCode || this.state.data.wineDbName !== prevState.data.wineDbName ||
                    this.state.data.color !== prevState.data.color) {
                    this.generateWineKeyword();

                    if (this.state.data.regionCode !== prevState.data.regionCode) {
                        this.getNextId();
                    }
                }
            }

            if (this.state.data.producerKeyword !== prevState.data.producerKeyword || this.state.data.baseKeyword !== prevState.data.baseKeyword ||
                this.state.data.appellationKeyword !== prevState.data.appellationKeyword || this.state.data.colorKeyword !== prevState.data.colorKeyword ||
                this.state.data.commonModifiersKeyword !== prevState.data.commonModifiersKeyword || this.state.data.modifiersKeyword !== prevState.data.modifiersKeyword ||
                this.state.data.customNotKeyword !== prevState.data.customNotKeyword || this.state.data.globalNotKeyword !== prevState.data.globalNotKeyword) {
                this.concatenateKeyword();
            }
        }
    }

    componentWillReceiveProps(nextProps) {
        if (!nextProps.shouldShow) return;
        if (nextProps.wineDbId) { // Editing wineDb
            let regionCode = nextProps.wineDbId.substring(0, 2);
            let producerId = nextProps.wineDbId.substring(0, 6);
            let wineDbId = nextProps.wineDbId;
            let defaultNoiseMode = nextProps.defaultNoiseMode || false;

            let _this = this;
            this.getWineDb(wineDbId, function (response) {

                let data = Object.assign({}, _this.defaultData);
                data.regionCode = regionCode;
                data.wineDbId = wineDbId;
                data.wineDbName = response.data.wineName;
                data.winePattern = response.data.pattern;
                data.winePatternOriginal = response.data.pattern;
                data.color = response.data.color;
                data.variety = response.data.variety;
                data.sweetness = response.data.sweetness;
                data.wineType = response.data.wineType;
                data.country = response.data.country;
                data.region = response.data.region;
                data.location = response.data.location;
                data.locale = response.data.locale;
                data.site = response.data.site;
                data.producerId = producerId;
                data.producerName = response.data.producerName;
                data.producerShow = response.data.producerShow;
                data.producerKeyword = response.data.producerKeyword ? response.data.producerKeyword : "";
                data.baseKeyword = response.data.baseKeyword ? response.data.baseKeyword : "";
                data.appellationKeyword = response.data.appellationKeyword ? response.data.appellationKeyword : "";
                data.colorKeyword = response.data.colorKeyword ? response.data.colorKeyword : "";
                data.commonModifiersKeyword = response.data.commonModifiersKeyword ? response.data.commonModifiersKeyword : "";
                data.modifiersKeyword = response.data.modifiersKeyword ? response.data.modifiersKeyword : "";
                data.customNotKeyword = response.data.customNotKeyword ? response.data.customNotKeyword : "";
                data.globalNotKeyword = response.data.globalNotKeyword ? response.data.globalNotKeyword : "";
                data.noises = response.data.noises ? response.data.noises : [];
                data.producerNoises = response.data.producerNoises ? response.data.producerNoises : [];

                let stateData = {disableProducer: true, selectedAppellation: {}, existingWines: [], lastInputFocus: "wineDbName"};
                stateData.data = data;
                stateData.selectedCountry = {label: '', value: ''};
                stateData.selectedRegion = {label: '', value: ''};
                stateData.selectedLocation = {label: '', value: ''};
                stateData.selectedLocale = {label: '', value: ''};
                stateData.selectedSite = {label: '', value: ''};
                stateData.noiseMode = defaultNoiseMode;

                if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
                if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
                if (data.location) stateData.selectedLocation = {label: data.location, value: {location: data.location, region: data.region, country: data.country}};
                if (data.locale) stateData.selectedLocale = {label: data.locale, value: {locale: data.locale, location: data.location, region: data.region, country: data.country}};
                if (data.site) stateData.selectedSite = {label: data.site, value: {site: data.site, locale: data.locale, location: data.location, region: data.region, country: data.country}};

                _this.setState(stateData, function () {
                    _this.getProducers(regionCode, producerId);
                    _this.getExistingWines(producerId);
                });
            });
        } else {
            let wine = Object.assign({}, nextProps.wine);
            if (!wine.regionCode && wine.producerId) wine.regionCode = wine.producerId.substring(0, 2);
            let defaultNoiseMode = nextProps.defaultNoiseMode || false;

            let data = Object.assign({}, this.defaultData);
            data.id = wine.id ? wine.id : "";
            data.mongoId = wine.mongoId;
            data.wineVinId = wine.wineVinId;
            data.vintage = wine.vintage;
            data.regionCode = wine.regionCode ? wine.regionCode : "";
            data.wineDbId = wine.wineDbId ? wine.wineDbId : "";
            data.wineDbName = wine.label ? wine.label : "";
            data.color = wine.color ? wine.color : "";
            data.variety = wine.variety ? wine.variety : "";
            data.sweetness = wine.sweetness ? wine.sweetness : "";
            data.wineType = wine.wineType ? wine.wineType : "Table";
            data.country = wine.country ? wine.country : "";
            data.region = wine.region ? wine.region : "";
            data.location = wine.location ? wine.location : "";
            data.locale = wine.locale ? wine.locale : "";
            data.site = wine.site ? wine.site : "";
            data.producerId = wine.producerId ? wine.producerId : "";
            data.producerName = wine.producerName ? wine.producerName : "";
            data.producerShow = wine.producerShow ? wine.producerShow : "";
            data.retailerCode = wine.retailerCode;
            data.retailerDescription = wine.retailerDescription;
            data.retailerUrl = wine.retailerUrl;

            let stateData = {selectedAppellation: {}, existingWines: [], lastInputFocus: "wineDbName"};
            stateData.data = data;
            stateData.selectedCountry = {label: '', value: ''};
            stateData.selectedRegion = {label: '', value: ''};
            stateData.selectedLocation = {label: '', value: ''};
            stateData.selectedLocale = {label: '', value: ''};
            stateData.selectedSite = {label: '', value: ''};
            stateData.noiseMode = defaultNoiseMode;

            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
            if (data.location) stateData.selectedLocation = {label: data.location, value: {location: data.location, region: data.region, country: data.country}};
            if (data.locale) stateData.selectedLocale = {label: data.locale, value: {locale: data.locale, location: data.location, region: data.region, country: data.country}};
            if (data.site) stateData.selectedSite = {label: data.site, value: {site: data.site, locale: data.locale, location: data.location, region: data.region, country: data.country}};

            let _this = this;
            _this.setState(stateData, function () {
                _this.getProducers(wine.regionCode, wine.producerId);
                if (wine.producerId && wine.producerId.length > 0) {
                    _this.getExistingWines(wine.producerId);
                    _this.getProducer(wine.producerId, function (response) {
                        let _data = Object.assign({}, _this.state.data);
                        if (response.data.keyword) _data.producerKeyword = response.data.keyword;
                        if (response.data.nameShow) _data.producerShow = response.data.nameShow;
                        _data.producerNoises = response.data.producerNoises;
                        _this.setState({data: _data});
                    });
                }
                _this.getNextId();
            });
        }
    }

    handleInputChange(event) {
        if (!event.target) return;
        const target = event.target;
        const name = target.name;
        let _this = this;
        let value = target.type === 'checkbox' ? target.checked : target.value;
        let data = Object.assign({}, this.state.data);

        if (name === 'date') {
            let date = target.value;
            value = date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
        }

        data[name] = value;

        this.setState({
            data: data
        }, function () {
            if (name === 'regionCode') _this.getNextId();
        });
    }

    handleOnFocus(e) {
        if (!e.target) return;
        let name = e.target.name;
        let inputs = ['producerName', 'producerShow', 'wineDbName'];
        if (inputs.indexOf(inputs) && name !== this.state.lastInputFocus) {
            this.setState({lastInputFocus: name})
        }
    }

    handleBooleanCheck(e) {
        if (!e.target) return;
        const name = e.target.name;
        let val = e.target.checked;
        let obj = {};
        obj[name] = val;
        this.setState(obj);
    }

    handleNoiseChange(e, m) {
        if (m.action === 'create-option') {
            let f = e.find((v) => {
                return v.__isNew__;
            });
            if (f) this.addNoise(f.value);
        } else if (m.action === 'remove-value' && m.removedValue) {
            this.removeNoise(m.removedValue.value);
        } else if (m.action === 'clear') {
            let data = Object.assign({}, this.state.data);
            data.noises = [];
            this.setState({data: data});
        }
    }

    addNoise(word) {
        word = StripAccents(word);
        word = word.toLowerCase();
        let n = this.state.data && this.state.data.noises ? this.state.data.noises : [];
        if (n.indexOf(word) >= 0) return;
        n.push(word);
        let data = Object.assign({}, this.state.data);
        data.noises = n;
        this.setState({data: data}, !this.props.wineDbId ? this.generateWineKeyword : this.cleanupKeywordNoises);
    }

    removeNoise(word) {
        if (!word) return;
        word = StripAccents(word);
        word = word.toLowerCase();
        let n = this.state.data && this.state.data.noises ? this.state.data.noises : [];
        let index = n.indexOf(word);
        if (index > -1) {
            n.splice(index, 1);
            let data = Object.assign({}, this.state.data);
            data.noises = n;
            this.setState({data: data}, !this.props.wineDbId ? this.generateWineKeyword : null);
        }
    }

    filterOption = ({ label, value, data }, search) => {
        if (search) {
            return label.toLowerCase().startsWith(search.toLowerCase()) || value.toString().includes(search);
        } else {
            return true;
        }
    };

    selectProducer(val) {
        let _this = this;
        let data = Object.assign({}, this.state.data);
        let f = this.state.producers.find((e) => {
            return e.id === val.value
        });
        data.producerId = f.id;
        data.producerName = f.name;
        _this.getProducer(f.id, function (response) {
            data.producerNoises = response.data.producerNoises;
            _this.setState({data: data});
        });
        _this.getExistingWines(f.id);
    }

    selectVariety(val) {
        let _this = this;
        let data = Object.assign({}, this.state.data);
        data.variety = val && val.name ? val.name : "";
        if (val) {
            let f = _this.state.varieties.find((e) => {
                return val.name && e.name.toLowerCase() === val.name.toLowerCase()
            });
            data.color = f && f.defaultColor ? f.defaultColor : '';
        }
        _this.setState({data: data});
    }

    getSelectedVariety() {
        let _this = this;
        if (!_this.state.data || !_this.state.data.variety || !_this.state.varietiesSelect) {
            return {label: '', value: ''}
        }
        let f = _this.state.varietiesSelect.find((e) => {
            return e.name.toLowerCase() === _this.state.data.variety.toLowerCase()
        });
        return f ? f : {label: '', value: ''};
    }

    selectCountry(val) {
        let data = Object.assign({}, this.state.data);
        if (val && val.value) {
            let stateData = {selectedCountry: val};
            data.country = val.value;

            // Validate lower appellation
            if (this.state.selectedRegion && this.state.selectedRegion.value
                && this.state.selectedRegion.value.country !== val.value) {
                data.region = "";
                data.location = "";
                data.locale = "";
                data.site = "";
                stateData.selectedRegion = {label: '', value: ''};
                stateData.selectedLocation = {label: '', value: ''};
                stateData.selectedLocale = {label: '', value: ''};
                stateData.selectedSite = {label: '', value: ''};
            }
            // End validate lower appellation

            stateData.data = data;
            this.setState(stateData);
        } else {
            data.country = "";
            data.region = "";
            data.location = "";
            data.locale = "";
            data.site = "";
            this.setState({
                data: data,
                selectedCountry: {label: '', value: ''},
                selectedRegion: {label: '', value: ''},
                selectedLocation: {label: '', value: ''},
                selectedLocale: {label: '', value: ''},
                selectedSite: {label: '', value: ''}
            });
        }
    }

    selectRegion(val) {
        let data = Object.assign({}, this.state.data);
        if (val && val.value) {
            let stateData = {selectedRegion: val};
            data.region = val.value.region;

            // Auto select
            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            // End auto select

            // Validate lower appellation
            if (this.state.selectedLocation && this.state.selectedLocation.value
                && this.state.selectedLocation.value.region !== val.value) {
                data.location = "";
                data.locale = "";
                data.site = "";
                stateData.selectedLocation = {label: '', value: ''};
                stateData.selectedLocale = {label: '', value: ''};
                stateData.selectedSite = {label: '', value: ''};
            }
            // End validate lower appellation

            stateData.data = data;
            this.setState(stateData);
        } else {
            data.region = "";
            data.location = "";
            data.locale = "";
            data.site = "";
            this.setState({
                data: data,
                selectedRegion: {label: '', value: ''},
                selectedLocation: {label: '', value: ''},
                selectedLocale: {label: '', value: ''},
                selectedSite: {label: '', value: ''}
            });
        }
    }

    selectLocation(val) {
        let data = Object.assign({}, this.state.data);
        if (val && val.value) {
            let stateData = {selectedLocation: val};
            data.location = val.value.location;

            // Auto assign
            data.country = val.value.country;
            data.region = val.value.region;
            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
            // End auto assign

            // Validate lower appellation
            if (this.state.selectedLocale && this.state.selectedLocale.value
                && this.state.selectedLocale.value.location !== val.value) {
                data.locale = "";
                data.site = "";
                stateData.selectedLocale = {label: '', value: ''};
                stateData.selectedSite = {label: '', value: ''};
            }
            // End validate lower appellation

            stateData.data = data;
            this.setState(stateData);
        } else {
            data.location = "";
            data.locale = "";
            data.site = "";
            this.setState({
                data: data,
                selectedLocation: {label: '', value: ''},
                selectedLocale: {label: '', value: ''},
                selectedSite: {label: '', value: ''}
            });
        }
    }

    selectLocale(val) {
        let data = Object.assign({}, this.state.data);
        if (val && val.value) {
            let stateData = {selectedLocale: val};
            data.locale = val.value.locale;

            // Auto assign
            data.country = val.value.country;
            data.region = val.value.region;
            data.location = val.value.location;
            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
            if (data.location) stateData.selectedLocation = {label: data.location, value: {location: data.location, region: data.region, country: data.country}};
            // End auto assign

            // Validate lower appellation
            if (this.state.selectedSite && this.state.selectedSite.value
                && this.state.selectedSite.value.locale !== val.value) {
                data.site = "";
                stateData.selectedSite = {label: '', value: ''};
            }
            // End validate lower appellation

            stateData.data = data;
            this.setState(stateData);
        } else {
            data.locale = "";
            data.site = "";
            this.setState({
                data: data,
                selectedLocale: {label: '', value: ''},
                selectedSite: {label: '', value: ''}
            });
        }
    }

    selectSite(val) {
        let data = Object.assign({}, this.state.data);
        if (val && val.value) {
            let stateData = {selectedSite: val};
            data.site = val.value.site;

            // Auto assign
            data.country = val.value.country;
            data.region = val.value.region;
            data.location = val.value.location;
            data.locale = val.value.locale;
            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
            if (data.location) stateData.selectedLocation = {label: data.location, value: {location: data.location, region: data.region, country: data.country}};
            if (data.locale) stateData.selectedLocale = {label: data.locale, value: {locale: data.locale, location: data.location, region: data.region, country: data.country}};
            // End auto assign

            stateData.data = data;
            this.setState(stateData);
        } else {
            data.site = "";
            this.setState({
                data: data,
                selectedSite: {label: '', value: ''}
            });
        }
    }

    selectAppellation(val) {
        if (val && val.value) {
            let data = Object.assign({}, this.state.data);
            let stateData = {selectedAppellation: val};
            if (val.value.regionCode && !this.state.data.wineDbId && !this.props.wineDbId && !this.state.data.producerId) { // for new wine new producer only
                data.regionCode = val.value.regionCode;
                stateData.data = data;
            }

            stateData.selectedCountry = {label: '', value: ''};
            stateData.selectedRegion = {label: '', value: ''};
            stateData.selectedLocation = {label: '', value: ''};
            stateData.selectedLocale = {label: '', value: ''};
            stateData.selectedSite = {label: '', value: ''};

            // Auto assign
            data.country = val.value.country;
            data.region = val.value.region;
            data.location = val.value.location;
            data.locale = val.value.locale;
            data.site = val.value.site;
            if (data.country) stateData.selectedCountry = {label: data.country, value: data.country};
            if (data.region) stateData.selectedRegion = {label: data.region, value: {region: data.region, country: data.country}};
            if (data.location) stateData.selectedLocation = {label: data.location, value: {location: data.location, region: data.region, country: data.country}};
            if (data.locale) stateData.selectedLocale = {label: data.locale, value: {locale: data.locale, location: data.location, region: data.region, country: data.country}};
            if (data.site) stateData.selectedSite = {label: data.site, value: {site: data.site, locale: data.locale, location: data.location, region: data.region, country: data.country}};
            // End auto assign

            stateData.data = data;
            this.setState(stateData);
        } else {
            this.setState({selectedAppellation: {label: "", value: "", country: ""}});
        }
    }

    getAppellationLabel(data) {
        if (!data) return "";
        let label = data.country;
        if (data.region) label += " / " + data.region;
        if (data.location) label += " / " + data.location;
        if (data.locale) label += " / " + data.locale;
        if (data.site) label += " / " + data.site;
        return label;
    }

    selectWine(e) {
        if (!e.currentTarget) return;

        let wid = e.currentTarget.value;
        if (wid) {
            let selectedWines = this.state.selectedWines;
            let index = selectedWines.indexOf(wid);
            if (index > -1) {
                selectedWines.splice(index, 1);
                this.setState({
                    selectedWines: selectedWines
                });
            } else {
                if (selectedWines.length < 3) {
                    selectedWines.push(wid);
                    this.setState({
                        selectedWines: selectedWines
                    });
                }
            }
        }
    }

    refreshExistingWines() {
        this.setState({selectedWines: []});
        this.getExistingWines(this.state.data.producerId);
    }

    async getNextId() {
        let url = Constants.NEXT_WINE_DB_ID + "?regionCode=" + (this.state.data.regionCode ? this.state.data.regionCode : "") +
            "&producerId=" + (this.state.data.producerId ? this.state.data.producerId : "");
        const rawData = await fetch(url, {
            credentials: 'same-origin'
        });
        const response = await rawData.json();
        if (response.data) {
            let vintageId = this.state.vintagesReverseMap[this.state.data.vintage];
            let data = Object.assign({}, this.state.data);
            data.nextId = response.data + (vintageId ? vintageId : "");
            this.setState({data: data});
        }
    }

    async getWineDb(wineDbId, callback) {
        if (wineDbId) {
            const rawData = await fetch(Constants.WINE_DB_DETAILS + "?id=" + wineDbId, {
                credentials: 'same-origin'
            });
            const res = await rawData.json();
            if (typeof callback === "function") callback(res);
        }
    }

    async getProducer(producerId, callback) {
        if (producerId) {
            const rawData = await fetch(Constants.PRODUCER_URL + "?id=" + producerId, {
                credentials: 'same-origin'
            });
            const res = await rawData.json();
            if (typeof callback === "function") callback(res);
        }
    }

    async getVintagesReverseMap() {
        const rawData = await fetch(Constants.VINTAGES_REVERSE_MAP, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({vintagesReverseMap: res.data});
    }

    async getVarieties() {
        const rawData = await fetch(Constants.VARIETIES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        let vs = [];
        if (res.data) {
            let rid = 1;
            VIP_VARIETIES.forEach(function (v) {
                let f = res.data.find((e) => {
                    return e.name.toLowerCase() === v
                });
                if (f) vs.push({id: rid++, name: f.name});
            });
            res.data.forEach(function (v) {
                if (VIP_VARIETIES.indexOf(v.name.toLowerCase()) < 0) {
                    vs.push({id: rid++, name: v.name});
                }
            });
        }
        this.setState({varieties: res.data, varietiesSelect: vs});
    }

    async getColors() {
        const rawData = await fetch(Constants.COLORS_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({wineColors: [""].concat(res.data)});
    }

    async getSweetness() {
        const rawData = await fetch(Constants.SWEETNESS_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({wineSweetness: res.data});
    }

    async getTypes() {
        const rawData = await fetch(Constants.TYPES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({wineTypes: res.data});
    }

    async getRegionCodes() {
        const rawData = await fetch(Constants.REGION_CODES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        let codes = [""];
        if (res.data && res.data.length > 0) codes = codes.concat(res.data);
        this.setState({regionCodes: codes});
    }

    async getReverseAliases() {
        const rawData = await fetch(Constants.REVERSE_ALIASES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by key length, descx
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({reverseAliases: sorted});
    }

    async getProducerEliminations() {
        const rawData = await fetch(Constants.ELIMINATIONS_URL + "?type=" + Constants.ELIMINATION_TYPE.PRODUCER, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by word length, desc
            return b.length - a.length;
        });
        this.setState({producerEliminations: sorted});
    }

    async getWineEliminations() {
        const rawData = await fetch(Constants.ELIMINATIONS_URL + "?type=" + Constants.ELIMINATION_TYPE.WINE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by word length, desc
            return b.length - a.length;
        });
        this.setState({wineEliminations: sorted});
    }

    async getMassChanges() {
        const rawData = await fetch(Constants.WORD_CHANGES_URL + "?type=" + Constants.ALIAS_TYPE.MASS_CHANGES, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by key length, desc
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({massChanges: sorted});
    }

    async getCustomChanges() {
        const rawData = await fetch(Constants.WORD_CHANGES_URL + "?type=" + Constants.ALIAS_TYPE.CUSTOM_CHANGES, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        const sorted = res.data.sort(function(a, b) { // sort by key length, desc
            return (Object.keys(b)[0]).length - (Object.keys(a)[0]).length;
        });
        this.setState({customChanges: sorted});
    }

    async getSpreadTables() {
        const rawData = await fetch(Constants.SPREAD_TABLES_URL, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({spreadTables: res.data});
    }

    async getColorSubstitutions() {
        const rawData = await fetch(Constants.SUBSTITUTIONS_URL + "?type=" + Constants.SUBSTITUTION_TYPE.COLOR, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({colorSubstitutions: res.data});
    }

    async getColorNotSubstitutions() {
        const rawData = await fetch(Constants.SUBSTITUTIONS_URL + "?type=" + Constants.SUBSTITUTION_TYPE.COLOR_NOT, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({colorNotSubstitutions: res.data});
    }

    async getRedOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.RED_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({redOpposites: res.data});
    }

    async getReserveWords() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.RESERVE_WORD, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({reserveWords: res.data});
    }

    async getRoseOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.ROSE_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({roseOpposites: res.data});
    }

    async getWhiteOpposites() {
        const rawData = await fetch(Constants.RESERVED_WORDS_URL + "?type=" + Constants.RESERVED_TYPE.WHITE_OPPOSITE, {
            credentials: 'same-origin'
        });
        const res = await rawData.json();
        this.setState({whiteOpposites: res.data});
    }

    async getExistingWines(producerId) {
        let _this = this;
        fetch(Constants.PRODUCER_WINES_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                producer: producerId
            }),
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({existingWines: json.data});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
        });
    }

    async getProducers(regionCode, producerId) {
        this.setState({isProducersLoading: true});

        let _this = this;
        await fetch(Constants.PRODUCERS_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                regionCode: regionCode,
                producerId: producerId
            }),
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            let ps = json.data.map(
                prod => ({label: prod.id + " - " + prod.name, value: prod.id})
            );
            _this.setState({producers: json.data, producersSelect: ps, isProducersLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isProducersLoading: false});
        });
    }

    async getCountries() {
        this.setState({isCountriesLoading: true});

        let _this = this;
        await fetch(Constants.COUNTRIES_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({countries: json.data, isCountriesLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isCountriesLoading: false});
        });
    }

    async getRegions() {
        this.setState({isRegionsLoading: true});

        let _this = this;
        await fetch(Constants.REGIONS_URL, {
            method: 'GET',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({regions: json.data, isRegionsLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isRegionsLoading: false});
        });
    }

    async getLocations() {
        this.setState({isLocationsLoading: true});

        let _this = this;
        await fetch(Constants.LOCATIONS_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({locations: json.data, isLocationsLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isLocationsLoading: false});
        });
    }

    async getLocales() {
        this.setState({isLocalesLoading: true});

        let _this = this;
        await fetch(Constants.LOCALES_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({locales: json.data, isLocalesLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isLocalesLoading: false});
        });
    }

    async getSites() {
        let _this = this;
        await fetch(Constants.SITES_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({sites: json.data, isSitesLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isSitesLoading: false});
        });
    }

    async getAppellations() {
        let _this = this;
        _this.setState({isAppellationsLoading: true});

        return fetch(Constants.APPELLATIONS_URL, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            credentials: 'same-origin'
        }).then(function(response) {
            if (!response.ok) {
                throw Error(response.statusText);
            }
            return response.json();
        }).then(function(json) {
            _this.setState({appellations: json.data, isAppellationsLoading: false});
        }).catch(function(ex) {
            console.log('Parsing data failed!', ex);
            _this.setState({isAppellationsLoading: false});
        });
    }

    getWineData() {
        let data = this.state.data;
        let formData = new FormData();
        let nextWineId = data.nextId;
        let nextProducerId = nextWineId ? nextWineId.substring(0, 6) : '';
        let nextWineDbId = nextWineId ? nextWineId.substring(0, 9) : '';

        formData.append('regionId', typeof data.regionCode !== "undefined" ? data.regionCode : '');

        formData.append('producerId', typeof data.producerId !== "undefined" && data.producerId ? data.producerId : '');
        formData.append('nextProducerId', nextProducerId ? nextProducerId : '');
        formData.append('producerName', typeof data.producerName !== "undefined" && data.producerName ? data.producerName : '');
        formData.append('producerShow', typeof data.producerShow !== "undefined" && data.producerShow ? data.producerShow : '');

        formData.append('wineDbId', data.wineDbId ? data.wineDbId : '');
        formData.append('nextWineDbId', nextWineDbId ? nextWineDbId : '');
        formData.append('name', typeof data.wineDbName !== "undefined" && data.wineDbName && data.wineDbName ? data.wineDbName : '');
        formData.append('color', typeof data.color !== "undefined" && data.color && data.color ? data.color : '');
        formData.append('wineType', typeof data.wineType !== "undefined" &&  data.wineType ? data.wineType : '');
        formData.append('sweetness', typeof data.sweetness !== "undefined" && data.sweetness ? data.sweetness : '');
        formData.append('variety', typeof data.variety !== "undefined" && data.variety ? data.variety : '');
        formData.append('wineCountry', typeof data.country !== "undefined" && data.country ? data.country : '');
        formData.append('wineRegion', typeof data.region !== "undefined" && data.region ? data.region : '');
        formData.append('wineLocation', typeof data.location !== "undefined" && data.location ? data.location : '');
        formData.append('wineLocale', typeof data.locale !== "undefined" && data.locale ? data.locale : '');
        formData.append('wineSite', typeof data.site !== "undefined" && data.site ? data.site : '');
        formData.append('wineNId', typeof data.wineVinId !== "undefined" && data.wineVinId ? data.wineVinId : '');

        formData.append('producerKeyword', typeof data.producerKeyword !== "undefined" && data.producerKeyword ? data.producerKeyword : '');
        formData.append('baseKeyword', typeof data.baseKeyword !== "undefined" && data.baseKeyword ? data.baseKeyword : '');
        formData.append('appellationKeyword', typeof data.appellationKeyword !== "undefined" && data.appellationKeyword ? data.appellationKeyword : '');
        formData.append('colorKeyword', typeof data.colorKeyword !== "undefined" && data.colorKeyword ? data.colorKeyword : '');
        formData.append('commonModifiersKeyword', typeof data.commonModifiersKeyword !== "undefined" && data.commonModifiersKeyword ? data.commonModifiersKeyword : '');
        formData.append('modifiersKeyword', typeof data.modifiersKeyword !== "undefined" && data.modifiersKeyword ? data.modifiersKeyword : '');
        formData.append('customNotKeyword', typeof data.customNotKeyword !== "undefined" && data.customNotKeyword ? data.customNotKeyword : '');
        formData.append('globalNotKeyword', typeof data.globalNotKeyword !== "undefined" && data.globalNotKeyword ? data.globalNotKeyword : '');

        formData.append('nextWineId', nextWineId);
        formData.append('vintage', typeof data.vintage !== "undefined" && data.vintage ? data.vintage : '');
        formData.append('mongoId', typeof data.mongoId !== "undefined" && data.mongoId ? data.mongoId : '');
        formData.append('source', this.props.source ? this.props.source : '');

        for (let i in data.noises) {
            if (data.noises[i]) formData.append('noises', data.noises[i]);
        }

        return formData;
    }

    saveWine() {
        let _this = this;
        let data = _this.state.data;
        if (!data.regionCode || data.regionCode.length === 0){
            alert('Reg Code required');
            return;
        }
        if (!data.color || data.color.length === 0){
            alert('Color is required');
            return;
        }
        if (!data.nextId || data.nextId.length === 0){
            alert('Wine ID required');
            return;
        }
        if ( !data.producerId && (!data.producerShow || data.producerShow.length ===0)){
            alert('Producer Show required');
            return;
        }
        let wineData = _this.getWineData();
        _this.saveNewWine(wineData);
    }

    updateWineDb() {
        let _this = this;
        let wineData = _this.getWineData();
        _this.updateExistingWineDb(wineData);
    }

    cancel() {
        let defaultData = Object.assign({}, this.defaultData);
        this.setState({data: defaultData}, typeof this.props.onCanceled === "function" ? this.props.onCanceled : null);
    }

    async saveNewWine(formData) {
        let _this = this;
        _this.setState({isSaving: true});
        fetch(Constants.SAVE_WINE, {
            method: 'POST',
            headers: {
                'Accept': 'application/json'
            },
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                let data = Object.assign({}, _this.state.data);
                let defaultData = Object.assign({}, _this.defaultData);
                _this.setState({data: defaultData, isSaving: false}, function () {
                    if (typeof _this.props.onSaved === "function") {
                        _this.props.onSaved(data);
                    }
                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isSaving: false});
        });
    }

    async updateExistingWineDb(formData) {
        let _this = this;
        _this.setState({isSaving: true});
        fetch(Constants.UPDATE_WINE_DB + "/" + formData.get('wineDbId'), {
            method: 'POST',
            body: formData,
            credentials: 'same-origin'
        }).then(function(response) {
            return response.json();
        }).then(function(json) {
            if (json.success) {
                let data = Object.assign({}, _this.state.data);
                let defaultData = Object.assign({}, _this.defaultData);
                _this.setState({data: defaultData, isSaving: false}, function () {
                    if (typeof _this.props.onUpdated === "function") {
                        _this.props.onUpdated(data);
                    }
                });
            } else {
                throw new Error(json.errors ? json.errors.join("\n") : (json.message ? json.message : 'Something went wrong ...'));
            }
        }).catch(function(ex) {
            alert(ex);
            _this.setState({isSaving: false});
        });
    }

    wordClick(word) {
        word = StripAccents(word);

        // UCFirst handler
        if (word !== word.toLocaleLowerCase()) {
            word = word.toLocaleLowerCase();
            word = word.charAt(0).toUpperCase() + word.substring(1);
        }

        let data = Object.assign({}, this.state.data);
        let currentInput = this.state.lastInputFocus;
        if (currentInput === "producerName") {
            let producerName = data.producerName.trim() + " " + word;
            producerName = producerName.trim();
            data.producerName = producerName;
            this.producerNameTxt.focus();
        } else if (currentInput === "producerShow") {
            let producerShow = data.producerShow.trim() + " " + word;
            producerShow = producerShow.trim();
            data.producerShow = producerShow;
            this.producerShowTxt.focus();
        } else if (currentInput === "wineDbName") {
            let wineDbName = data.wineDbName.trim() + " " + word;
            wineDbName = wineDbName.trim();
            data.wineDbName = wineDbName;
            this.wineDbNameTxt.focus();
        }

        this.setState({data: data});
    }

    listWines() {
        if (!this.state.existingWines || this.state.existingWines.length < 1) return "";

        let _this = this;
        return _this.state.existingWines.map(function(row, i) {
            return (
                <div className="row row-striped" key={i}>
                    <div className="col-xs-3">{row.id}</div>
                    <div className="col-xs-8"><ClickableWord phrase={row.description} onClick={!_this.state.noiseMode ? _this.wordClick : _this.addNoise} /></div>
                    <div className="col-xs-1">
                        <input type="checkbox" name="mergeId" value={row.id} onClick={_this.selectWine}
                               checked={_this.state.selectedWines && _this.state.selectedWines.indexOf(row.id) > -1} readOnly={true} />
                    </div>
                </div>
            );
        });
    }

    globalSubstitute(kw) {
        if (!kw) return '';
        kw = kw.toLowerCase(); // Lower case
        kw = ReplacePunctuationsForKeyword(kw, " "); // Remove punctuation
        kw = this.replaceWithReverseAliases(kw); // Correct reverse aliases using table
        return kw;
    }

    generateProducerKeyword(callback) {
        if (!this.state.data.producerShow) return "";
        let producerKw = this.state.data.producerShow;
        producerKw = this.globalSubstitute(producerKw);
        producerKw = this.proceedProducerEliminations(producerKw); //7	remove producer eliminations using table
        producerKw = producerKw.replace(new RegExp('\\s+', 'g'), '&').trim(); //9	replace spaces with &'s

        let data = Object.assign({}, this.state.data);
        data.producerKeyword = producerKw;
        this.setState({data: data}, typeof callback === "function" ? callback : null);
    }

    proceedProducerEliminations(text) {
        let producerEliminations = this.state.producerEliminations;
        for (let i in producerEliminations) {
            let re = producerEliminations[i];
            let rs = new RegExp('\\b' + EscapeRegExp(re) + '\\b', 'gi');
            text = text.replace(rs, '');
        }
        return text;
    }

    removeNoises(keyword) {
        if (!keyword) return keyword;
        keyword = '&' + keyword + '&';
        keyword = CleanUpAnds(keyword);
        if (keyword === '&') return '';

        let data = Object.assign({}, this.state.data);
        let noises = data && data.noises ? data.noises : [];
        for (let i in noises) {
            let n = noises[i];
            n = this.substitute(n);
            if (n) {
                let rs = new RegExp(EscapeRegExp(n), 'gi');
                keyword = keyword.replace(rs, '&');
                keyword = CleanUpAnds(keyword);
            }
        }
        keyword = keyword.replace(/&$/g, ''); // Remove last &
        return keyword;
    }

    /**
     * Cleans up keyword noises for existing wine
     */
    cleanupKeywordNoises() {
        let data = Object.assign({}, this.state.data);
        let kk = ['baseKeyword', 'appellationKeyword', 'colorKeyword', 'commonModifiersKeyword', 'modifiersKeyword'];
        for (let i in kk) {
            let key = kk[i];
            data[key] = this.removeNoises(data[key]);
        }
        this.setState({data: data});
    }

    substitute(kw) {
        if (!kw) return '';
        kw = this.globalSubstitute(kw);
        kw = this.proceedWineEliminations(kw); //16	remove wine eliminations using table
        kw = this.proceedMassChanges(kw); //17	do mass changes using table
        kw = kw.replace(new RegExp('\\s+', 'g'), '&').trim(); //19	replace spaces with &'s
        kw = this.proceedCustomChanges(kw); //20	do custom changes using table
        kw = '&' + kw + '&'; //21	add leading and trailing &'s
        return CleanUpAnds(kw);
    }

    substituteSpreads(wineKw, spreads) {
        let kw = '';
        for (let i in spreads) {
            let sp = spreads[i];
            if (sp && wineKw.indexOf(sp) > -1) {
                kw = kw + sp;
                wineKw = wineKw.replace(sp, '&');
            }
        }
        return [kw, wineKw];
    }

    generateWineKeyword() {
        let data = Object.assign({}, this.state.data);
        let wineKw = this.substitute(data.wineDbName);

        //22	spread label name kw string using region tables
        let regionCode = data.regionCode;
        if (regionCode) {
            let spreads = this.state.spreadTables ? this.state.spreadTables[regionCode] : {};
            if (spreads) {
                let baseKw = '';
                let baseSpreads = spreads[Constants.KEYWORD_TYPE.BASE];
                if (baseSpreads) {
                    let result = this.substituteSpreads(wineKw, baseSpreads);
                    baseKw = result[0];
                    wineKw = result[1];
                }
                data.baseKeyword = this.removeNoises(baseKw);

                let appellationKw = '';
                let appellationSpreads = spreads[Constants.KEYWORD_TYPE.APPELLATION];
                if (appellationSpreads) {
                    let result = this.substituteSpreads(wineKw, appellationSpreads);
                    appellationKw = result[0];
                    wineKw = result[1];
                }
                data.appellationKeyword = this.removeNoises(appellationKw);

                let colorKw = '';
                let colorSpreads = spreads[Constants.KEYWORD_TYPE.COLOR];
                if (colorSpreads) {
                    let result = this.substituteSpreads(wineKw, colorSpreads);
                    colorKw = result[0];
                    wineKw = result[1];
                }
                if (colorKw) { //23	do color substitutions using table
                    let colorSubstitutions = this.state.colorSubstitutions;
                    Object.keys(colorSubstitutions).sort(function (a, b) {
                        return b.length - a.length;
                    }).forEach(function(key) {
                        colorKw = colorKw.replace(key, colorSubstitutions[key]);
                    });
                }
                data.colorKeyword = this.removeNoises(colorKw);

                let commonModifKw = '';
                let commonModifSpreads = spreads[Constants.KEYWORD_TYPE.COMMON_MODIFIER];
                if (commonModifSpreads) {
                    let result = this.substituteSpreads(wineKw, commonModifSpreads);
                    commonModifKw = result[0];
                    wineKw = result[1];
                }
                data.commonModifiersKeyword = this.removeNoises(commonModifKw);

                // the rest goes to modifiersKeyword
                data.modifiersKeyword = this.removeNoises(wineKw);
            }
        }

        let wineName = data.producerShow ? data.producerShow : '';
        wineName = (wineName ? wineName + ' ' : '') + (data.wineDbName ? data.wineDbName : '');

        //26	populate wine-keywords-input using global not rules in additional nots column
        let globalNots = '';

        //If there are no color opposites (for the color of that wine) in either ProducerShow or Label Name (from winealertdb)
        // 	add color nots from color not strings table
        let color = data.color ? data.color.toLowerCase() : '';
        let oppositeFound = false;
        let colorOpposites = this.state[color + 'Opposites'];
        for (let i in colorOpposites) {
            let re = new RegExp('\\b' + EscapeRegExp(colorOpposites[i]) + '\\b', 'gi');
            if (wineName.match(re)) {
                oppositeFound = true;
                break;
            }
        }
        if (!oppositeFound && this.state.colorNotSubstitutions) {
            let notSubs = this.state.colorNotSubstitutions[color];
            if (notSubs) globalNots = globalNots + notSubs;
        }

        //If there are no reserve words in either ProducerShow or Label Name (from winealertdb)
        // 	add reserve words from reserve words table
        let reserveFound = false;
        let reserveWords = this.state.reserveWords;
        for (let i in reserveWords) {
            let re = new RegExp('\\b' + EscapeRegExp(reserveWords[i]) + '\\b', 'gi');
            if (wineName.match(re)) {
                reserveFound = true;
                break;
            }
        }
        if (!reserveFound) {
            globalNots = globalNots + '&!reserve';
        }
        globalNots = CleanUpAnds(globalNots);
        globalNots = globalNots.replace(/&$/g, ''); // Remove last &
        data.globalNotKeyword = globalNots;

        this.setState({data: data});
    }

    /**
     * Join patterns of wine keyword to produce pattern<br>
     * Keyword/pattern will be constructed with these following rules:<br>
     * producerKeyword : 1x<br>
     * baseKeyword : 1x<br>
     * appellationKeyword : 1x<br>
     * colorKeyword : 1x<br>
     * commonModifiersKeyword : 2x<br>
     * modifiersKeyword : 6x<br>
     * customNotKeyword : 1x<br>
     * globalNotKeyword : 1x
     */
    concatenateKeyword() {
        let data = Object.assign({}, this.state.data);
        let keyword = data.producerKeyword;
        if (data.baseKeyword) keyword = keyword + '&' + data.baseKeyword;
        if (data.appellationKeyword) keyword = keyword + '&' + data.appellationKeyword;
        if (data.colorKeyword) keyword = keyword + data.colorKeyword;
        if (data.commonModifiersKeyword) {
            keyword = keyword + data.commonModifiersKeyword;
            keyword = keyword + data.commonModifiersKeyword;
        }
        if (data.modifiersKeyword) {
            keyword = keyword + data.modifiersKeyword;
            keyword = keyword + data.modifiersKeyword;
            keyword = keyword + data.modifiersKeyword;
            keyword = keyword + data.modifiersKeyword;
            keyword = keyword + data.modifiersKeyword;
            keyword = keyword + data.modifiersKeyword;
        }
        if (data.customNotKeyword) keyword = keyword + data.customNotKeyword;
        if (data.globalNotKeyword) keyword = keyword + data.globalNotKeyword;
        keyword = keyword.replace(new RegExp('&+', 'g'), '&');
        keyword = keyword.replace(/^&+|&+$/g, ''); // Trim &'s

        data.winePattern = keyword;
        this.setState({data: data});
    }

    proceedWineEliminations(text) {
        let wineEliminations = this.state.wineEliminations;
        for (let i in wineEliminations) {
            let re = wineEliminations[i];
            let rs = new RegExp('\\b' + EscapeRegExp(re) + '\\b', 'gi');
            text = text.replace(rs, '');
        }
        return text;
    }

    proceedMassChanges(text) {
        let array = this.state.massChanges;
        for (let i in array) {
            let obj = array[i];
            let key = Object.keys(obj)[0];
            let rs = new RegExp('\\b' + EscapeRegExp(key) + '\\b', 'gi');
            text = text.replace(rs, obj[key]);
        }
        return text;
    }

    proceedCustomChanges(text) {
        let array = this.state.customChanges;
        for (let i in array) {
            let obj = array[i];
            let key = Object.keys(obj)[0];
            let rs = new RegExp('\\b' + EscapeRegExp(key) + '\\b', 'gi');
            text = text.replace(rs, obj[key]);
        }
        return text;
    }

    replaceWithReverseAliases(text) {
        let array = this.state.reverseAliases;
        for (let i in array) {
            let obj = array[i];
            let key = Object.keys(obj)[0];
            let rs = new RegExp('\\b' + EscapeRegExp(key) + '\\b', 'gi');
            text = text.replace(rs, obj[key]);
        }
        return text;
    }

    reGenerateKeyword() {
        let wine = this.state.data;
        if (wine.producerId && wine.producerId.length > 0) {
            let data = Object.assign({}, this.state.data);
            let _this = this;
            this.getProducer(wine.producerId, function (response) {
                data.producerKeyword = response.data.keyword;
                _this.setState({data: data}, _this.generateWineKeyword);
            });
        } else {
            this.generateProducerKeyword(this.generateWineKeyword);
        }
    }

    render() {
        return (
            <div className="row" hidden={typeof this.props.shouldShow !== 'undefined' && !this.props.shouldShow}>
                <div className="col-xs-12">
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center"><p>&nbsp;</p></div>
                    </div>
                    <div className="row wine-match-row">
                        <div className="col-xs-5">
                            <div className="row">
                                <div className="col-xs-6">
                                    <h3 className="title">{this.state.data.id || this.props.wineDbId ? 'Edit' : 'New'} {this.props.wineDbId ? "WineDb" : "Wine"}</h3>
                                </div>
                                <div className="col-xs-6 text-right">
                                    <h3 className="title" hidden={!this.props.wineDbId}>{this.props.wineDbId}</h3>
                                    <h3 className="title" hidden={!this.state.data.id}>{this.state.data.id ? this.state.data.id : ''}</h3>
                                    <h3 className="title" hidden={this.state.data.id}>{this.state.data.nextId}</h3>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12">
                                    <table className="table table-bordered">
                                        <tbody>
                                        <tr hidden={!this.state.data.retailerDescription}>
                                            <td style={{width: '35%'}}><b>Retailer Description</b></td>
                                            <td>
                                                <ClickableWord phrase={this.state.data.retailerDescription} onClick={!this.state.noiseMode ? this.wordClick : this.addNoise} />
                                            </td>
                                        </tr>
                                        <tr hidden={!this.state.data.mongoId}>
                                            <td style={{width: '35%'}}><b>Mongo ID</b></td>
                                            <td>{this.state.data.mongoId}</td>
                                        </tr>
                                        <tr hidden={!this.state.data.wineVinId}>
                                            <td style={{width: '35%'}}><b>Wine Vin ID</b></td>
                                            <td>{this.state.data.wineVinId}</td>
                                        </tr>
                                        <tr hidden={!this.state.data.vintage}>
                                            <td style={{width: '35%'}}><b>Vintage</b></td>
                                            <td>{this.state.data.vintage}</td>
                                        </tr>
                                        </tbody>
                                    </table>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12 winedb-data">
                                    <div className="row">
                                        <div className="col-md-3">
                                            <div className="form-group">
                                                <label className="control-label">Reg Code</label>
                                                <select className="form-control" id="regionCode" name="regionCode"
                                                        onChange={this.handleInputChange} value={this.state.data.regionCode}
                                                        ref={(select) => { this.regionsSelectBox = select; }} disabled={this.state.data.id || this.props.wineDbId || this.props.producerId}>
                                                    {this.state.regionCodes.map(function(row, i) {
                                                        return (
                                                            <option key={i} value={row}>{row}</option>
                                                        );
                                                    })}
                                                </select>
                                            </div>
                                        </div>
                                        <div className="col-md-9" hidden={!(this.state.data.producerId && this.state.data.producerId.length > 0)}>
                                            <div className="form-group">
                                                <label className="control-label">Producer</label>
                                                <Select className="producer-selection"
                                                    name="producerId"
                                                    value={this.state.producersSelect.find((e) => {
                                                        return this.state.data && e.value === this.state.data.producerId
                                                    })}
                                                    options={this.state.producersSelect}
                                                    onChange={this.selectProducer}
                                                    isClearable={false}
                                                    isLoading={this.state.isProducersLoading}
                                                    isDisabled={this.state.disableProducer || this.props.producerId} />
                                            </div>
                                        </div>
                                        <div className="col-md-9" hidden={this.state.data.producerId}>
                                            <div className="form-group">
                                                <label className="control-label">Wine ID</label>
                                                <input type="text" className="form-control" name="nextId" value={this.state.data.nextId} onChange={this.handleInputChange} disabled={true} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row" hidden={this.state.data.producerId}>
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Producer name</label>
                                                <input type="text" className="form-control" name="producerName" value={this.state.data.producerName} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.producerNameTxt = input;}}/>
                                            </div>
                                        </div>
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Producer Show</label>
                                                <input type="text" className="form-control" name="producerShow" value={this.state.data.producerShow} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.producerShowTxt = input;}}/>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-md-6">
                                            <label className="control-label">Label Name</label>
                                        </div>
                                        <div className="col-md-6">
                                            <div className="form-check pull-right">
                                                <label className="form-check-label">Noise Mode</label>
                                                <input type="checkbox" name="noiseMode" className="form-check-input"
                                                       checked={this.state.noiseMode} onClick={this.handleBooleanCheck} readOnly={true}/>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-group" hidden={this.state.noiseMode}>
                                        <input type="text" className="form-control" name="wineDbName" value={this.state.data.wineDbName} onChange={this.handleInputChange} onFocus={this.handleOnFocus} ref={(input) => {this.wineDbNameTxt = input;}}/>
                                    </div>
                                    <div className="form-group complete-keyword break-word" hidden={!this.state.noiseMode}>
                                        <ClickableWord phrase={this.state.data.wineDbName} onClick={this.addNoise}/>
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Variety</label>
                                        <Select className="variety-selection"
                                                name="variety"
                                                value={this.getSelectedVariety()}
                                                options={this.state.varietiesSelect}
                                                onChange={this.selectVariety}
                                                isClearable={true}
                                                isLoading={this.state.isVarietiesLoading}
                                                filterOption={this.filterOption}
                                                getOptionValue={option => option.id}
                                                getOptionLabel={option => option.name}
                                        />
                                    </div>
                                    <div className="row">
                                        <div className="col-md-4">
                                            <div className="form-group">
                                                <label className="control-label">Color</label>
                                                <select className="form-control" id="color" name="color"
                                                        onChange={this.handleInputChange} value={this.state.data.color}
                                                        ref={(select) => { this.colorsSelectBox = select; }}>
                                                    {this.state.wineColors.map(function(row, i) {
                                                        return (
                                                            <option key={i} value={row}>{row}</option>
                                                        );
                                                    })}
                                                </select>
                                            </div>
                                        </div>
                                        <div className="col-md-4">
                                            <div className="form-group">
                                                <label className="control-label">Sweetness</label>
                                                <select className="form-control" id="sweetness" name="sweetness"
                                                        onChange={this.handleInputChange} value={this.state.data.sweetness}
                                                        ref={(select) => { this.sweetnessSelectBox = select; }}>
                                                    {this.state.wineSweetness.map(function(row, i) {
                                                        return (
                                                            <option key={i} value={row}>{row}</option>
                                                        );
                                                    })}
                                                </select>
                                            </div>
                                        </div>
                                        <div className="col-md-4">
                                            <div className="form-group">
                                                <label className="control-label">Type</label>
                                                <select className="form-control" id="wineType" name="wineType"
                                                        onChange={this.handleInputChange} value={this.state.data.wineType}
                                                        ref={(select) => { this.typesSelectBox = select; }}>
                                                    {this.state.wineTypes.map(function(row, i) {
                                                        return (
                                                            <option key={i} value={row}>{row}</option>
                                                        );
                                                    })}
                                                </select>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Country</label>
                                                <Select className="country-selection"
                                                        name="country"
                                                        value={this.state.selectedCountry}
                                                        options={this.state.countries.map(
                                                            country => ({label: country, value: country})
                                                        )}
                                                        onChange={this.selectCountry}
                                                        isClearable={true}
                                                        isLoading={this.state.isCountriesLoading} />
                                            </div>
                                        </div>
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Region</label>
                                                <Select className="region-selection"
                                                        name="region"
                                                        value={this.state.selectedRegion}
                                                        options={this.state.regions.map(
                                                            region => ({label: region.region, value: region})
                                                        )}
                                                        onChange={this.selectRegion}
                                                        isClearable={true}
                                                        isLoading={this.state.isRegionsLoading} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Sub-Region</label>
                                        <Select className="location-selection"
                                                name="location"
                                                value={this.state.selectedLocation}
                                                options={this.state.locations.map(
                                                    location => ({label: location.location, value: location})
                                                )}
                                                onChange={this.selectLocation}
                                                isClearable={true}
                                                isLoading={this.state.isLocationsLoading} />
                                    </div>
                                    <div className="row">
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Appellation</label>
                                                <Select className="locale-selection"
                                                        name="locale"
                                                        value={this.state.selectedLocale}
                                                        options={this.state.locales.map(
                                                            locale => ({label: locale.locale, value: locale})
                                                        )}
                                                        onChange={this.selectLocale}
                                                        isClearable={true}
                                                        isLoading={this.state.isLocalesLoading} />
                                            </div>
                                        </div>
                                        <div className="col-md-6">
                                            <div className="form-group">
                                                <label className="control-label">Sub-Appellation</label>
                                                <Select className="site-selection"
                                                        name="site"
                                                        value={this.state.selectedSite}
                                                        options={this.state.sites.map(
                                                            site => ({label: site.site, value: site})
                                                        )}
                                                        onChange={this.selectSite}
                                                        isClearable={true}
                                                        isLoading={this.state.isSitesLoading} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">All Appellations Tree</label>
                                        <Select className="appellations-selection"
                                                name="site"
                                                value={this.state.selectedAppellation}
                                                options={this.state.appellations.map(
                                                    apl => ({label: this.getAppellationLabel(apl.appellation), value: apl.appellation })
                                                )}
                                                onChange={this.selectAppellation}
                                                isClearable={true}
                                                isLoading={this.state.isAppellationsLoading} />
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Noise</label>
                                        <Creatable
                                            className="noises-selection"
                                            placeholder=""
                                            name="noises"
                                            value={this.state.data.noises ? this.state.data.noises.map(
                                                noise => ({label: noise, value: noise})
                                            ) : []}
                                            isMulti={true}
                                            isClearable={true}
                                            onChange={this.handleNoiseChange}
                                        />
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Producer Noise</label>
                                        <Select
                                            className="producer-noises-selection"
                                            placeholder=""
                                            name="producerNoises"
                                            value={this.state.data.producerNoises ? this.state.data.producerNoises.map(
                                                noise => ({label: noise, value: noise})
                                            ) : []}
                                            isMulti={true}
                                            isDisabled={true}
                                        />
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div className="col-xs-7">
                            <div className="row">
                                <div className="col-xs-12">
                                    <div className="wine-list">
                                        <div className="row data-list-header">
                                            <div className="col-md-12">
                                                <strong>Existing Producer's Wines</strong>
                                                <div className="pull-right">
                                                    <Merge wines={this.state.selectedWines} callback={this.refreshExistingWines} />
                                                </div>
                                            </div>
                                        </div>
                                        <div className="row">
                                            <div className="col-md-12 existing-wines">
                                                {this.listWines()}
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12 keyword-form">
                                    <div className="row">
                                        <div className="col-xs-10">
                                            <div className="form-group">
                                                <label className="control-label">Producer Keyword</label>
                                                <input type="text" className="form-control" name="producerKeyword" value={this.state.data.producerKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                        <div className="col-xs-2">
                                            <div className="form-group pull-right">
                                                <br />
                                                <button className="btn btn-default" onClick={this.reGenerateKeyword}>Re-Generate</button>
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Base Keyword</label>
                                                <input type="text" className="form-control" name="baseKeyword" value={this.state.data.baseKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Appellation Keyword</label>
                                                <input type="text" className="form-control" name="appellationKeyword" value={this.state.data.appellationKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Color Keyword</label>
                                                <input type="text" className="form-control" name="colorKeyword" value={this.state.data.colorKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Common Modifiers</label>
                                                <input type="text" className="form-control" name="commonModifiersKeyword" value={this.state.data.commonModifiersKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="row">
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Modifiers</label>
                                                <input type="text" className="form-control" name="modifiersKeyword" value={this.state.data.modifiersKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                        <div className="col-xs-6">
                                            <div className="form-group">
                                                <label className="control-label">Custom Nots</label>
                                                <input type="text" className="form-control" name="customNotKeyword" value={this.state.data.customNotKeyword} onChange={this.handleInputChange} />
                                            </div>
                                        </div>
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Global Nots</label>
                                        <input type="text" className="form-control" name="globalNotKeyword" value={this.state.data.globalNotKeyword} onChange={this.handleInputChange} />
                                    </div>
                                    <div className="form-group">
                                        <label className="control-label">Keyword</label>
                                        <div className={CheckForInvalidKeyword(this.state.data.winePattern) ? 'complete-keyword break-word danger' : 'complete-keyword break-word'}>
                                            <SlicedKeyword keyword={this.state.data.winePattern} />
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div className="row">
                                <div className="col-md-12">
                                    <div className="form-group">
                                        <button className="btn btn-default" onClick={this.cancel} disabled={this.state.isSaving}>Cancel</button>&nbsp;&nbsp;
                                        <span hidden={this.state.data.id || this.props.wineDbId}>
                                            <button className="btn btn-primary" onClick={this.saveWine} disabled={this.state.isSaving}>{this.state.isSaving ? 'Saving...' : 'Save'}</button>
                                        </span>
                                        <span hidden={!this.props.wineDbId}>
                                            <button className="btn btn-primary" onClick={this.updateWineDb} disabled={this.state.isSaving}>{this.state.isSaving ? 'Updating...' : 'Update'}</button>
                                        </span>
                                    </div>
                                    <div className="form-group" hidden={!this.state.data.winePatternOriginal || this.state.data.winePattern === this.state.data.winePatternOriginal}>
                                        <label className="control-label">Keyword Diff</label>
                                        <div className={CheckForInvalidKeyword(this.state.data.winePatternOriginal) ? 'complete-keyword break-word danger' : 'complete-keyword break-word'}>
                                            <SlicedKeyword keyword={this.state.data.winePatternOriginal}
                                                           compareWith={this.state.data.winePattern}
                                            />
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div className="row external-resource" hidden={!this.state.data.retailerUrl}>
                        <div className="col-md-12">
                            <ExternalResources wine={this.state.data} hideNoteLink={true} hideAppellation={true} hideVariety={true} />
                        </div>
                    </div>
                </div>
            </div>
        );
    }
}

export default WineEditor;