import React from 'react';
import Candidates from './Candidates.jsx';
import WineEditor from './WineEditor.jsx';
import * as Constants from '../constants';
import {CleanUpWord} from "../helpers/helper";
import ProducerEditor from "./ProducerEditor";

class WineSearcher extends React.Component {
	constructor(props) {
		super(props);
		this.state = {
            narrowKeywords: [],
            resetFilterID: false,
            isEditingWine: false,
            wineDbId: "", // WineDbId to be edited
            wine: {}, // Wine to be edited
            isEditingProducer: false,
            producerId: "" // ProducerId to be edited
		};
        this.wineIdFilterClearedTimer = null;

        this.beforeUnload = this.beforeUnload.bind(this);

        this.editWineDb = this.editWineDb.bind(this);
        this.createWineNewProducer = this.createWineNewProducer.bind(this);
        this.createWine = this.createWine.bind(this);
        this.cancelEditWine = this.cancelEditWine.bind(this);
        this.onWineDBSaved = this.onWineDBSaved.bind(this);

        this.editProducer = this.editProducer.bind(this);
        this.cancelEditProducer = this.cancelEditProducer.bind(this);

        this.addNarrowWord = this.addNarrowWord.bind(this);
        this.removeNarrowWord = this.removeNarrowWord.bind(this);
        this.refreshCandidates = this.refreshCandidates.bind(this);

        this.onWineIdFilterCleared = this.onWineIdFilterCleared.bind(this);
	}

    componentWillMount() {
    }

	componentDidMount() {
        window.addEventListener("beforeunload", this.beforeUnload);
    }

    componentDidUpdate(prevProps, prevState) {
    }

    componentWillUnmount() {
        window.removeEventListener("beforeunload", this.beforeUnload);
    }

    beforeUnload(e) {
        if (this.state.isEditingWine) {
            e.preventDefault();
            e.returnValue = true;
        }
    }

    addNarrowWord(word) {
        if (word) {
            let n = this.state.narrowKeywords;
            if (n.indexOf(word) === -1) {
                n.push(word);
                this.setState({
                    narrowKeywords: n
                });
            }
        }
    }

    removeNarrowWord(word) {
        word = CleanUpWord(word);
        if (word) {
            let n = this.state.narrowKeywords;
            let index = n.indexOf(word);
            if (index > -1) {
                n.splice(index, 1);
                this.setState({
                    narrowKeywords: n,
                    resetFilterID: n && n.length < 1
                });
            }
        }
    }

    refreshCandidates() {
        let _this = this;
        let words = _this.state.narrowKeywords;
        _this.setState({
            wineDbId: "",
            isEditingWine: false,
            producerId: "",
            isEditingProducer: false,
            narrowKeywords: [],
            resetFilterID: true
        }, function () {
            _this.setState({narrowKeywords: words});
        });
    }

    createWineNewProducer() {
        this.createWine();
    }

    createWine(producerId) {
	    let wine = {};
	    if (producerId) wine.producerId = producerId;
        this.setState({wine: wine, isEditingWine: true});
    }

    cancelEditWine() {
        this.setState({wine: {}, wineDbId: "", isEditingWine: false});
    }

    editWineDb(wineDbId) {
        if (!wineDbId) return;
        this.setState({wineDbId: wineDbId, isEditingWine: true});
    }

    onWineDBSaved() {
        this.setState({wine: {}}, this.refreshCandidates);
    }

    editProducer(producerId) {
        if (!producerId) return;
        this.setState({producerId: producerId, isEditingProducer: true});
    }

    cancelEditProducer() {
        this.setState({producerId: "", isEditingProducer: false});
    }

    onWineIdFilterCleared() {
        clearTimeout(this.wineIdFilterClearedTimer);
        this.wineIdFilterClearedTimer = setTimeout(function() {
            this.setState((state) => {
                return {resetFilterID: false};
            });
        }.bind(this), 2);
    }

    render() {
		return (
			<div className="row">
				<div className="col-xs-12" hidden={this.state.isEditingWine || this.state.isEditingProducer}>
                    <div className="row wine-match-row">
                        <div className="col-xs-12 text-center">
                            <h2 className="title">Wine Addition - Direct Input</h2>
                        </div>
                    </div>
                    <div className="row wine-match-row">
                        <ul className="list-inline btn-list">
                            <li><button className="btn btn-md btn-default" onClick={this.createWineNewProducer}>Add Wine New Producer</button></li>
                        </ul>
                    </div>
                    <Candidates words={this.state.narrowKeywords} addWord={this.addNarrowWord} removeWord={this.removeNarrowWord}
                                createWine={this.createWine} editWineDb={this.editWineDb} editProducer={this.editProducer}
                                resetFilterID={this.state.resetFilterID} onWineIdFilterCleared={this.onWineIdFilterCleared}
                    />
				</div>
                <div className="col-xs-12">
                    <WineEditor shouldShow={this.state.isEditingWine} wineDbId={this.state.wineDbId}
                                producerId={this.state.wine.producerId}
                                wine={this.state.wine} source={Constants.SOURCE.WINE_ADDITION}
                                onUpdated={this.onWineDBSaved} onSaved={this.onWineDBSaved} onCanceled={this.cancelEditWine} />
                </div>
                <div className="col-xs-12">
                    <ProducerEditor shouldShow={this.state.isEditingProducer} producerId={this.state.producerId} source={Constants.SOURCE.NARROW}
                                    onUpdated={this.refreshCandidates} onCanceled={this.cancelEditProducer} />
                </div>
			</div>
		);
	}
}

export default WineSearcher;