package wine.alert

import grails.plugin.springsecurity.annotation.Secured
import grails.transaction.Transactional
import wine.alert.enums.DumpType
import wine.db.DumpJob

@Secured(value=["hasRole('ROLE_ADMIN')"])
@Transactional(readOnly = true)
class DownloadController {

    def downloadService

    /**
     * Gets latest dump files (WineDb, MongoMap, WineKeyword, ProducerKeyword)
     */
    def index() {
        [
                wineDbFileName: downloadService.getLatestDumpFileName(DumpType.WINE_DB),
                mongoMapFileName: downloadService.getLatestDumpFileName(DumpType.MONGO_MAP),
                wineKeywordFileName: downloadService.getLatestDumpFileName(DumpType.WINE_KEYWORD),
                producerKeywordFileName: downloadService.getLatestDumpFileName(DumpType.PRODUCER_KEYWORD),
                globalNoiseFileName: downloadService.getLatestDumpFileName(DumpType.GLOBAL_NOISE),
                producerNoiseFileName: downloadService.getLatestDumpFileName(DumpType.PRODUCER_NOISE),
                wineNoiseFileName: downloadService.getLatestDumpFileName(DumpType.WINE_NOISE),
                runningDump: downloadService.runningDump
        ]
    }

    /**
     * Creates new dump file based on the given type ({@link DumpType})
     */
    def dumpData() {
        try {
            String dumpType = params?.dumpType
            DumpJob.triggerNow([dumpType: dumpType])
        } catch (Exception e) {
            log.error(e?.message, e)
        }
        redirect(action: "index")
    }

    /**
     * Downloads dump file based on the given type ({@link DumpType}).
     * @return the dump file if exists, else redirected to {@link DownloadController#index()}
     */
    def downloadData() {
        try {
            DumpType dumpType = params?.dumpType as DumpType
            File file = downloadService.getLatestDumpFile(dumpType)
            if (file && file.exists()) {
                header("Content-disposition", "attachment; filename=${file.name}")
                render(file: file, contentType: 'text/plain', fileName: file.name)
            } else {
                flash.message = "File not found."
                redirect action: "index", method: "GET"
            }
        } catch (Exception e) {
            log.error(e?.message, e)
            flash.message = e?.message
            redirect action: "index", method: "GET"
        }
    }
}
