package wine.alert

import grails.plugin.springsecurity.annotation.Secured
import grails.transaction.Transactional

import java.text.SimpleDateFormat

@Secured(value=["hasRole('ROLE_ADMIN')"])
@Transactional(readOnly = true)
class JobsTriggerController {

    /**
     * To run {@link CrawlAllJob} and seprate job with {@link SeparateCrawlJob}
     * @return
     */
    def webCrawler() {
        CrawlAllJob.triggerNow()
        SeparateCrawlJob.triggerNow()
        render "Crawl job triggered."
    }

    /**
     * To run {@link HistoryMatchJob}
     * @return
     */
    def historyMatch() {
        HistoryMatchJob.triggerNow()
        render "History match job triggered."
    }

    /**
     * To run {@link KeywordMatchJob}
     * @return
     */
	def keywordMatch() {
		KeywordMatchJob.triggerNow()
		render "Keyword match job triggered."
	}

    /**
     * To run {@link ClearLogsJob}
     * @return
     */
    def clearLogs() {
        ClearLogsJob.triggerNow()
        render "Clear logs job triggered."
    }

    /**
     * To run {@link ConfigFilesWatcherJob}
     * @return
     */
    def watchConfigs() {
        ConfigFilesWatcherJob.triggerNow()
        render "Configs files watcher job triggered."
    }

    /**
     * To run {@link CurrencyRateJob}
     * @return
     */
    def currencyRates() {
        CurrencyRateJob.triggerNow()
        render "CurrencyRate job triggered."
    }

    /**
     * To run {@link RemapKeywordsJob}
     * @return
     */
    def remapKeywords() {
        RemapKeywordsJob.triggerNow()
        render "RemapKeywords job triggered."
    }

    /**
     * run {@link ArchiveJob} base on {@param params.date}
     * @return
     */
    def archive() {
        def sdf = new SimpleDateFormat("yyyy-MM-dd")
        Date date = params?.get("date") ? sdf.parse((String) params?.get("date")) : null

        ArchiveJob.triggerNow([
                date: date ?: null
        ])
        render "Archive job triggered."
    }

    /**
     * To run {@link NotesUploaderJob}
     * @return
     */
    def uploadNotes() {
        NotesUploaderJob.triggerNow([test: true])
        render "NotesUploader job triggered."
    }
}
