package wine.alert

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Secured(value=["hasRole('ROLE_ADMIN')"])
@Transactional(readOnly = true)
class PostProcessingController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link PostProcessing}
     * @param max the maximum offset
     * @return postProcessingList List of {@link PostProcessing}
     * from minimum offset to maximum offset
     * @return model postProcessingCount length of postProcessingList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        respond PostProcessing.list(params), model:[postProcessingCount: PostProcessing.count()]
    }

    /**
     * To get detail {@link PostProcessing}
     * @param postProcessing object of current {@link PostProcessing}
     * @return postProcessing current {@link PostProcessing}
     */
    def show(PostProcessing postProcessing) {
        respond postProcessing
    }

    /**
     * To render create page of {@link PostProcessing}
     */
    def create() {
        respond new PostProcessing(params)
    }

    /**
     * To save new {@link PostProcessing}
     * @param postProcessing object of {@link PostProcessing}
     */
    @Transactional
    def save(PostProcessing postProcessing) {
        if (postProcessing == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (postProcessing.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond postProcessing.errors, view:'create'
            return
        }

        postProcessing.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'postProcessing.label', default: 'PostProcessing'), postProcessing.id])
                redirect postProcessing
            }
            '*' { respond postProcessing, [status: CREATED] }
        }
    }

    /**
     * To render edit page of {@link PostProcessing}
     * @param postProcessing The current {@link PostProcessing}
     * object that you want to edit
     */
    def edit(PostProcessing postProcessing) {
        respond postProcessing
    }

    /**
     * To update current post processing
     * @param postProcessing The current {@link PostProcessing}
     * object that you want to edit
     */
    @Transactional
    def update(PostProcessing postProcessing) {
        if (postProcessing == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (postProcessing.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond postProcessing.errors, view:'edit'
            return
        }

        postProcessing.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'postProcessing.label', default: 'PostProcessing'), postProcessing.id])
                redirect postProcessing
            }
            '*'{ respond postProcessing, [status: OK] }
        }
    }

    /**
     * To Delete the current {@link PostProcessing}
     * that you want to delete
     * @param postProcessing The current {@link PostProcessing}
     * to delete
     */
    @Transactional
    def delete(PostProcessing postProcessing) {

        if (postProcessing == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        postProcessing.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'postProcessing.label', default: 'PostProcessing'), postProcessing.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'postProcessing.label', default: 'PostProcessing'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
