package wine.alert

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Secured(value = ["hasRole('ROLE_ADMIN')"])
@Transactional(readOnly = true)
class RetailerGroupController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    def groupCrawlerService

    def index(Integer max) {
        params.max = Math.min(max ?: 10, 100)
        def list = RetailerGroup.list(params)
        groupCrawlerService.assignState(list)
        respond list, model: [retailerGroupCount: RetailerGroup.count()]
    }

    def show(RetailerGroup retailerGroup) {
        respond retailerGroup
    }

    def create() {
        respond new RetailerGroup(params)
    }

    @Transactional
    def save(RetailerGroup retailerGroup) {
        if (retailerGroup == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (retailerGroup.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond retailerGroup.errors, view: 'create'
            return
        }

        retailerGroup.save flush: true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'retailerGroup.label', default: 'RetailerGroup'), retailerGroup.id])
                redirect retailerGroup
            }
            '*' { respond retailerGroup, [status: CREATED] }
        }
    }

    def edit(RetailerGroup retailerGroup) {
        respond retailerGroup
    }

    @Transactional
    def update(RetailerGroup retailerGroup) {
        if (retailerGroup == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (retailerGroup.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond retailerGroup.errors, view: 'edit'
            return
        }

        retailerGroup.save flush: true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'retailerGroup.label', default: 'RetailerGroup'), retailerGroup.id])
                redirect retailerGroup
            }
            '*' { respond retailerGroup, [status: OK] }
        }
    }

    @Transactional
    def delete(RetailerGroup retailerGroup) {

        if (retailerGroup == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        retailerGroup.delete flush: true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'retailerGroup.label', default: 'RetailerGroup'), retailerGroup.id])
                redirect action: "index", method: "GET"
            }
            '*' { render status: NO_CONTENT }
        }
    }

    def run(RetailerGroup retailerGroup) {
        groupCrawlerService.run(retailerGroup)
        redirect action: "index", method: "GET"
    }

    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'retailerGroup.label', default: 'RetailerGroup'), params.id])
                redirect action: "index", method: "GET"
            }
            '*' { render status: NOT_FOUND }
        }
    }
}
