package wine.alert

import enums.RollupType
import grails.converters.JSON
import grails.plugin.springsecurity.annotation.Secured

@Secured(value=["hasRole('ROLE_ADMIN')"])
class RollupController {

    def outputService

    /**
     * To get list of {@link RollupLog}
     * @param max the maximum offset
     * @return RollupLog List of {@link RollupLog}
     * from minimum offset to maximum offset
     * @return model crawlPropertyCount length of RollupLog
     * and rollupType
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 30, 100)
        RollupType rollupType = params.rollupType == RollupType.WA.name() ? RollupType.WA : RollupType.RP
        //RollupType rollupType = RollupType.RP

        respond RollupLog.createCriteria().list(params) {
            eq("type", rollupType)
            if (!params?.sort) order("lastUpdated", "desc")
        }, model: [
                rollupLogCount: RollupLog.countByType(rollupType),
                rollupType: rollupType
        ]
    }

    /**
     * Run the rollup job based on the given rollup type ({@link RollupType}).<br>
     *     This method will produce output as JSON
     * @return the trigger status as JSON
     */
    def proceed() {
        RollupType rollupType = params.rollupType == RollupType.WA.name() ? RollupType.WA : RollupType.RP
        //RollupType rollupType = RollupType.RP
        RollupJob.triggerNow([rollupType: rollupType])
        def result = [success: true]
        render result as JSON
    }

    /**
     * Downloads rollup file based on the given id ({@link RollupLog#id}) and rollup type ({@link RollupType}).<br>
     *     This method will produce output as JSON
     * @return the rollup file if exists, else redirected to {@link RollupController#index(Integer)}
     */
    def download() {
        RollupType rollupType = params.rollupType == RollupType.WA.name() ? RollupType.WA : RollupType.RP
        //RollupType rollupType = RollupType.RP
        try {
            RollupLog log = RollupLog.findByIdAndType(params.long('id'), rollupType)
            File file = outputService.getRollupResultFile(log.filename)
            if (file && file.exists()) {
                header("Content-disposition", "attachment; filename=${file.name}")
                render(file: file, contentType: 'text/plain', fileName: file.name)
            } else {
                flash.message = "File not found."
                redirect action: "index", method: "GET", params: [rollupType: rollupType?.name()]
            }
        } catch (Exception e) {
            log.error(e?.message, e)
            flash.message = e?.message
            redirect action: "index", method: "GET", params: [rollupType: rollupType?.name()]
        }
    }

    /**
     * Gets rollup status based on the given id ({@link RollupLog#id}) and rollup type ({@link RollupType}).<br>
     *     This method will produce output as JSON
     * @return the status message as JSON
     */
    def getRollUpLog() {
        RollupType rollupType = params.rollupType == RollupType.WA.name() ? RollupType.WA : RollupType.RP
        //RollupType rollupType = RollupType.RP
        RollupLog rollupLog = RollupLog.findByIdAndType(params.long('id'), rollupType)

        def result = [
                message: rollupLog?.message,
                status: rollupLog?.status?.value()
        ]
        render result as JSON
    }
}
