package wine.alert

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class WordAliasController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link WordAlias}
     * @param max the maximum offset
     * @return wordAliasList List of {@link WordAlias}
     * from minimum offset to maximum offset
     * @return model wordAliasCount length of wordAliasList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        params.sort = params?.sort ?: "aliasType.id"
        params.order = params?.order ?: "asc"
        respond WordAlias.list(params), model:[wordAliasCount: WordAlias.count()]
    }

    /**
     * To get detail {@link WordAlias}
     * @param wordAlias object of current {@link WordAlias}
     * @return wordAlias current {@link WordAlias}
     */
    def show(WordAlias wordAlias) {
        respond wordAlias
    }

    /**
     * To render create page of {@link WordAlias}
     */
    def create() {
        respond new WordAlias(params)
    }

    /**
     * To save new {@link WordAlias}
     * @param wordAlias object of {@link WordAlias}
     */
    @Transactional
    def save(WordAlias wordAlias) {
        if (wordAlias == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wordAlias.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wordAlias.errors, view:'create'
            return
        }

        wordAlias.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'wordAlias.label', default: 'WordAlias'), wordAlias.id])
                redirect wordAlias
            }
            '*' { respond wordAlias, [status: CREATED] }
        }
    }

    /**
     * To render edit page of {@link WordAlias}
     * @param wordAlias The current {@link WordAlias}
     * object that you want to edit
     */
    def edit(WordAlias wordAlias) {
        respond wordAlias
    }

    /**
     * To update current word alias
     * @param wordAlias The current {@link WordAlias}
     * object that you want to edit
     */
    @Transactional
    def update(WordAlias wordAlias) {
        if (wordAlias == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wordAlias.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wordAlias.errors, view:'edit'
            return
        }

        wordAlias.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'wordAlias.label', default: 'WordAlias'), wordAlias.id])
                redirect wordAlias
            }
            '*'{ respond wordAlias, [status: OK] }
        }
    }

    /**
     * To Delete the current {@link WordAlias}
     * that you want to delete
     * @param wordAlias The current {@link WordAlias}
     * to delete
     */
    @Transactional
    def delete(WordAlias wordAlias) {

        if (wordAlias == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        wordAlias.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'wordAlias.label', default: 'WordAlias'), wordAlias.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'wordAlias.label', default: 'WordAlias'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
