package wine.alert

import enums.Status
import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class WorkGroupWebCrawlerController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link WorkGroupWebCrawler}
     * @param max the maximum offset
     * @return WorkGroupWebCrawlerList List of {@link WorkGroupWebCrawler}
     * from minimum offset to maximum offset
     * @return model workGroupWebCrawlerCount length of WorkGroupWebCrawlerList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 10, 100)
        respond WorkGroupWebCrawler.list(params), model:[workGroupWebCrawlerCount: WorkGroupWebCrawler.count()]
    }

    /**
     * To get detail {@link WorkGroupWebCrawler}
     * @param workGroupWebCrawler object of current {@link WorkGroupWebCrawler}
     * @return workGroupWebCrawler current {@link WorkGroupWebCrawler}
     */
    def show(WorkGroupWebCrawler workGroupWebCrawler) {
        respond workGroupWebCrawler
    }

    /**
     * To render create page of {@link WorkGroupWebCrawler}
     */
    def create() {
        def webCrawlers = WebCrawler.createCriteria().list() {
            ne("status", Status.DELETED)
            order("code", "asc")
        }
        respond new WorkGroupWebCrawler(params), model: [webCrawlers: webCrawlers]
    }

    /**
     * To save new {@link WorkGroupWebCrawler}
     * @param workGroupWebCrawler object of {@link WorkGroupWebCrawler}
     */
    @Transactional
    def save(WorkGroupWebCrawler workGroupWebCrawler) {
        if (workGroupWebCrawler == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (workGroupWebCrawler.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond workGroupWebCrawler.errors, view:'create'
            return
        }

        workGroupWebCrawler.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'workGroupWebCrawler.label', default: 'Work Group Retailer'), workGroupWebCrawler.id])
                redirect workGroupWebCrawler.workGroup
            }
            '*' { respond workGroupWebCrawler, [status: CREATED] }
        }
    }

    /**
     * To render edit page of {@link WorkGroupWebCrawler}
     * @param workGroupWebCrawler The current {@link WorkGroupWebCrawler}
     * object that you want to edit
     */
    def edit(WorkGroupWebCrawler workGroupWebCrawler) {
        respond workGroupWebCrawler
    }

    /**
     * To update current {@link WorkGroupWebCrawler}
     * @param workGroupWebCrawler The current {@link WorkGroupWebCrawler}
     * object that you want to edit
     */
    @Transactional
    def update(WorkGroupWebCrawler workGroupWebCrawler) {
        if (workGroupWebCrawler == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (workGroupWebCrawler.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond workGroupWebCrawler.errors, view:'edit'
            return
        }

        workGroupWebCrawler.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'workGroupWebCrawler.label', default: 'WorkGroupWebCrawler'), workGroupWebCrawler.id])
                redirect workGroupWebCrawler
            }
            '*'{ respond workGroupWebCrawler, [status: OK] }
        }
    }

    /**
     * To Delete the current {@link WorkGroupWebCrawler}
     * that you want to delete
     * @param workGroupWebCrawler The current {@link WorkGroupWebCrawler}
     * to delete
     */
    @Transactional
    def delete(WorkGroupWebCrawler workGroupWebCrawler) {

        if (workGroupWebCrawler == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        Long workGroupId = workGroupWebCrawler.workGroup?.id
        workGroupWebCrawler.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'workGroupWebCrawler.label', default: 'Work Group Retailer'), workGroupWebCrawler.id])
                redirect controller: "workGroup", action: "show", id: workGroupId, method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'workGroupWebCrawler.label', default: 'WorkGroupWebCrawler'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
