package wine.alert.api

import grails.converters.JSON
import grails.plugin.springsecurity.annotation.Secured
import grails.transaction.Transactional
import org.springframework.http.HttpStatus
import wine.db.Wine
import wine.db.WineLabel

@Secured(value = ['permitAll()'])
class ApiWineController {

    static allowedMethods = [show: 'GET', submit: 'POST']

    private static final String STATIC_TOKEN = "7FLSDLraXBZUGpaKAVXc845oWnCWXsX8XmL1QjpFraUkGjXI9v"
    private static final String HEADER_TOKEN_NAME = "X-API-Token"

    def show(){
        String wineId = params?.id
        String token = request.getHeader(HEADER_TOKEN_NAME)

        if(!token.equals(STATIC_TOKEN)) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value())
            render getAPIResult(HttpStatus.UNAUTHORIZED,[:], null) as JSON

            return
        }


        def result = [:]
        if(wineId) {
            Wine wine = Wine.wineDatabase.findById(wineId)

            if (wine) {
                String message = "wine found with id : " + wineId

                def wineDetail = [:]

                wineDetail.put("wineId", wine.id)
                wineDetail.put("name", wine.getFullName())
                wineDetail.put("vintage", wine.vintage)
                wineDetail.put("rating", wine.getRating())


                if(wine.wineDb?.producer) {
                        wineDetail.put("producer", wine.wineDb.producer.nameShow ?: wine.wineDb.producer.name)
                }

                result = getAPIResult(HttpStatus.OK, wineDetail, null)
            } else {
                result = getAPIResult(HttpStatus.NOT_FOUND, [:], null)
            }

        } else {
            result = getAPIResult(HttpStatus.BAD_REQUEST, [:], null)
        }

        response.setStatus((int) result.get('status'))
        render result as JSON
    }

    @Transactional
    def submit(){
        String imgId = params?.imageId
        String imgExt = params?.imageExt
        String wineName = params?.wineName
        String vintage = params?.vintage
        String token = request.getHeader(HEADER_TOKEN_NAME)

        if(!token.equals(STATIC_TOKEN)) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value())
            render getAPIResult(HttpStatus.UNAUTHORIZED,[:], null) as JSON

            return
        }

        WineLabel wineLabel = new WineLabel(
                imageId: imgId,
                imageExt: imgExt,
                name: wineName,
                vintage: vintage,
                submittedDate: new Date()
        )

        wineLabel.validate()

        def dataReturn = [:]
        dataReturn.put("imageId", wineLabel.imageId)
        dataReturn.put("imageExt", wineLabel.imageExt)
        dataReturn.put("name", wineLabel.name)
        dataReturn.put("vintage", wineLabel.vintage)
        dataReturn.put("status", wineLabel.status.value())

        def result = [:]
        if(wineLabel.hasErrors()){
            result = getAPIResult(HttpStatus.BAD_REQUEST, dataReturn, null)
        } else {
            wineLabel.save flush: true

            result = getAPIResult(HttpStatus.CREATED, dataReturn, null)
        }

        response.setStatus((int) result.get('status'))
        render result as JSON
    }

    private getAPIResult(HttpStatus status, Object data, String message) {
        def result = [:]
        String apiPath = createLink(controller: controllerName, action: actionName, id: params?.id)
        result.put("path", apiPath)
        result.put("timestamp", Calendar.getInstance().timeInMillis)
        result.put("status", status.value())
        result.put("message", message ?: status.reasonPhrase)
        result.put("data",data)

        return result
    }
}
