package wine.db

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class BottleSizeController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link BottleSize}
     * @param max the maximum offset
     * @return bottleSizeList List of {@link BottleSize}
     * from minimum offset to maximum offset
     * @return model bottleSizeCount length of bottleSizeList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        params.sort = params?.sort ?: "id"
        params.order = params?.order ?: "desc"
        respond BottleSize.wineDatabase.list(params), model:[bottleSizeCount: BottleSize.wineDatabase.count()]
    }

    /**
     * To get detail {@link BottleSize}
     * @param bottleSize object of current {@link BottleSize}
     * @return bottleSize current {@link BottleSize}
     */
    def show(BottleSize bottleSize) {
        respond bottleSize
    }

    /**
     * To render create page of {@link BottleSize}
     */
    def create() {
        respond new BottleSize(params)
    }

    /**
     * To save new {@link BottleSize}
     * @param bottleSize object of {@link BottleSize}
     */
    @Transactional
    def save(BottleSize bottleSize) {
        if (bottleSize == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (bottleSize.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond bottleSize.errors, view:'create'
            return
        }

        bottleSize.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'bottleSize.label', default: 'BottleSize'), bottleSize.id])
                redirect bottleSize
            }
            '*' { respond bottleSize, [status: CREATED] }
        }
    }

    /**
     * To render edit page of {@link BottleSize}
     * @param bottleSize The current {@link BottleSize}
     * object that you want to edit
     */
    def edit(BottleSize bottleSize) {
        respond bottleSize
    }

    /**
     * To update current bottle size
     * @param bottleSize The current {@link BottleSize}
     * object that you want to edit
     */
    @Transactional
    def update(BottleSize bottleSize) {
        if (bottleSize == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (bottleSize.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond bottleSize.errors, view:'edit'
            return
        }

        bottleSize.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'bottleSize.label', default: 'BottleSize'), bottleSize.id])
                redirect bottleSize
            }
            '*'{ respond bottleSize, [status: OK] }
        }
    }

    /*@Transactional
    def delete(BottleSize bottleSize) {

        if (bottleSize == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        bottleSize.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'bottleSize.label', default: 'BottleSize'), bottleSize.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }*/

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'bottleSize.label', default: 'BottleSize'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
