package wine.db

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class ProducerController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link Producer}
     * @param max the maximum offset
     * @return producerNoiseList List of {@link Producer}
     * from minimum offset to maximum offset
     * @return model producerCount length of producerList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        params.sort = params?.sort ?: "id"
        params.order = params?.order ?: "desc"
        respond Producer.list(params), model:[producerCount: Producer.count()]
    }

    /**
     * To get detail {@link Producer}
     * @param producer object of current {@link Producer}
     * @return producer current {@link Producer}
     */
    def show(Producer producer) {
        respond producer
    }

    /*def create() {
        respond new Producer(params)
    }

    @Transactional
    def save(Producer producer) {
        if (producer == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (producer.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond producer.errors, view:'create'
            return
        }

        producer.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'producer.label', default: 'Producer'), producer.id])
                redirect producer
            }
            '*' { respond producer, [status: CREATED] }
        }
    }

    def edit(Producer producer) {
        respond producer
    }

    @Transactional
    def update(Producer producer) {
        if (producer == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (producer.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond producer.errors, view:'edit'
            return
        }

        producer.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'producer.label', default: 'Producer'), producer.id])
                redirect producer
            }
            '*'{ respond producer, [status: OK] }
        }
    }

    @Transactional
    def delete(Producer producer) {

        if (producer == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        producer.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'producer.label', default: 'Producer'), producer.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }*/

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'producer.label', default: 'Producer'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
