package wine.db

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class WineController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link Wine}
     * @param max the maximum offset
     * @return wineList List of {@link Wine}
     * from minimum offset to maximum offset
     * @return model wineCount length of wineList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        params.sort = params?.sort ?: "id"
        params.order = params?.order ?: "asc"
        respond Wine.wineDatabase.list(params), model:[wineCount: Wine.wineDatabase.count()]
    }

    /**
     * To get detail {@link Wine}
     * @param wine object of current {@link Wine}
     * @return wine current {@link Wine}
     */
    def show(Wine wine) {
        respond wine
    }

    /*def create() {
        respond new Wine(params)
    }

    @Transactional
    def save(Wine wine) {
        if (wine == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wine.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wine.errors, view:'create'
            return
        }

        wine.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'wine.label', default: 'Wine'), wine.id])
                redirect wine
            }
            '*' { respond wine, [status: CREATED] }
        }
    }

    def edit(Wine wine) {
        respond wine
    }

    @Transactional
    def update(Wine wine) {
        if (wine == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wine.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wine.errors, view:'edit'
            return
        }

        wine.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'wine.label', default: 'Wine'), wine.id])
                redirect wine
            }
            '*'{ respond wine, [status: OK] }
        }
    }

    @Transactional
    def delete(Wine wine) {

        if (wine == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        wine.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'wine.label', default: 'Wine'), wine.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }*/

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'wine.label', default: 'Wine'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
