package wine.db

import grails.plugin.springsecurity.annotation.Secured

import static org.springframework.http.HttpStatus.*
import grails.transaction.Transactional

@Transactional(readOnly = true)
@Secured(value=["hasRole('ROLE_ADMIN')"])
class WineDbController {

    static allowedMethods = [save: "POST", update: "PUT", delete: "DELETE"]

    /**
     * To get list of {@link WineDb}
     * @param max the maximum offset
     * @return wineDbList List of {@link WineDb}
     * from minimum offset to maximum offset
     * @return model wineDbCount length of wineDbList
     */
    def index(Integer max) {
        params.max = Math.min(max ?: 20, 100)
        respond WineDb.list(params), model:[wineDbCount: WineDb.count()]
    }

    /**
     * To get detail {@link WineDb}
     * @param wineDb object of current {@link WineDb}
     * @return wineDb current {@link WineDb}
     */
    def show(WineDb wineDb) {
        respond wineDb
    }

    /*def create() {
        respond new WineDb(params)
    }

    @Transactional
    def save(WineDb wineDb) {
        if (wineDb == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wineDb.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wineDb.errors, view:'create'
            return
        }

        wineDb.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.created.message', args: [message(code: 'wineDb.label', default: 'WineDb'), wineDb.id])
                redirect wineDb
            }
            '*' { respond wineDb, [status: CREATED] }
        }
    }

    def edit(WineDb wineDb) {
        respond wineDb
    }

    @Transactional
    def update(WineDb wineDb) {
        if (wineDb == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        if (wineDb.hasErrors()) {
            transactionStatus.setRollbackOnly()
            respond wineDb.errors, view:'edit'
            return
        }

        wineDb.save flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.updated.message', args: [message(code: 'wineDb.label', default: 'WineDb'), wineDb.id])
                redirect wineDb
            }
            '*'{ respond wineDb, [status: OK] }
        }
    }

    @Transactional
    def delete(WineDb wineDb) {

        if (wineDb == null) {
            transactionStatus.setRollbackOnly()
            notFound()
            return
        }

        wineDb.delete flush:true

        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.deleted.message', args: [message(code: 'wineDb.label', default: 'WineDb'), wineDb.id])
                redirect action:"index", method:"GET"
            }
            '*'{ render status: NO_CONTENT }
        }
    }*/

    /**
     * To redirect not found page
     */
    protected void notFound() {
        request.withFormat {
            form multipartForm {
                flash.message = message(code: 'default.not.found.message', args: [message(code: 'wineDb.label', default: 'WineDb'), params.id])
                redirect action: "index", method: "GET"
            }
            '*'{ render status: NOT_FOUND }
        }
    }
}
