package wine.alert

import enums.DataPlacement
import enums.DataType
import enums.IgnoreIf
import enums.WineProperty
import org.apache.commons.lang.StringUtils

class CrawlProperty {

    static belongsTo = [webCrawler: WebCrawler]
    static hasMany = [postProcessings: PostProcessing]

    WineProperty wineProperty
    String customLabel
    DataType dataType
    Boolean parseAsHtml = false

    // html
    String selectors
    String noises
    String textToRemove
    Boolean useDetails
    String detailsSelectors
    Boolean dataOutput

    // data placement
    DataPlacement dataPlacement
    String propertyName

    // separated value / post processing
    Integer propertyIndex

    // ignore the data if this property value equals ignoreIf
    IgnoreIf ignoreIf
    String ignoreValues

    Date dateCreated
    Date lastUpdated

    static constraints = {
        dataType nullable: true
        parseAsHtml nullable: true

        customLabel nullable: true, blank: true
        selectors nullable: true, blank: true
        noises nullable: true, blank: true
        textToRemove nullable: true, blank: true
        useDetails nullable: true
        detailsSelectors nullable: true, blank: true
        dataOutput nullable: true

        dataPlacement nullable: true
        propertyName nullable: true

        propertyIndex nullable: true
        ignoreIf nullable: true
        ignoreValues nullable: true, blank: true
    }

    static mapping = {
        dataPlacement defaultValue: DataPlacement.TAG
        ignoreValues sqlType: "text"
    }

    def getIndexName() {
        String indexName = wineProperty.name()
        if (StringUtils.isNotBlank(customLabel)) {
            indexName = customLabel
        }
        return indexName
    }
}
