package wine.alert

import enums.Status
import groovy.time.TimeCategory
import groovy.time.TimeDuration
import org.grails.databinding.BindingFormat

class JobLog {

    static transients = ['webCrawler', 'executionTime', 'validation', 'logStatus', 'logMessage']

    String code
    String name
    String url
    Status status
    Integer totalRecords = 0
    BigDecimal totalPrices = 0
    Date dateStart
    Date dateEnd
    String message

    // Crawl Validation
    Integer totalValidation = 0
    Integer validationPassed = 0
    Integer validationWarn = 0
    Integer validationFailed = 0
    Status validationStatus
    String validationMessage

    Date dateCreated
    Date lastUpdated

    @BindingFormat('yyyy-MM-dd')
    Date jobDate

    static constraints = {
        name nullable: true, blank: true
        url nullable: true, blank: true
        message nullable: true, blank: true
        dateEnd nullable: true

        validationStatus nullable: true
        validationMessage nullable: true, blank: true

        jobDate nullable: true
    }

    static mapping = {
        code index: 'code_idx'
        status index: 'status_idx'
        message sqlType: "text"
        validationStatus index: 'validation_status_idx'
        validationMessage sqlType: "text"
        url sqlType: "text"
        totalRecords defaultValue: "0"
        totalPrices defaultValue: "0"
        totalValidation defaultValue: "0"
        validationPassed defaultValue: "0"
        validationWarn defaultValue: "0"
        validationFailed defaultValue: "0"

        jobDate formula: "DATE(`date_start`)"
    }

    def getWebCrawler() {
        return WebCrawler.findByCode(code)
    }

    def getExecutionTime() {
        if (!dateEnd || !dateStart) {
            return new TimeDuration(0,0,0,0)
        }

        return TimeCategory.minus(dateEnd, dateStart)
    }

    def getValidation() {
        validationPassed + " / " + totalValidation
    }

    def getLogStatus() {
        if (status == Status.FAILED || status == Status.RUNNING) {
            return status
        }

        return validationStatus
    }

    def getLogMessage() {
        if (status == Status.FAILED) {
            return message
        }

        return validationMessage
    }
}
