package wine.alert

import enums.Status
import groovy.time.TimeCategory
import groovy.time.TimeDuration

class MatchedLog {

    static transients = ['webCrawler', 'executionTime']

    String code
    Boolean processKeyword = true
    Integer totalRecords = 0
    Integer historyMatched = 0
    Integer keywordMatched = 0
    Integer ambiguousMatched = 0
    Integer unknownWines = 0

    Status status
    Date crawlDate
    Date dateStart
    Date dateEnd
    String message

    Date dateCreated
    Date lastUpdated

    static constraints = {
        message nullable: true, blank: true
        dateEnd nullable: true
    }

    static mapping = {
        code index: 'code_idx'
        message sqlType: "text"
        processKeyword defaultValue: "1"
        totalRecords defaultValue: "0"
        historyMatched defaultValue: "0"
        keywordMatched defaultValue: "0"
        ambiguousMatched defaultValue: "0"
        unknownWines defaultValue: "0"
    }

    def getWebCrawler() {
        return WebCrawler.findByCode(code)
    }

    def getExecutionTime() {
        if (!dateEnd || !dateStart) {
            return new TimeDuration(0,0,0,0)
        }

        return TimeCategory.minus(dateEnd, dateStart)
    }
}
