package wine.db

import org.apache.commons.lang.StringUtils

class BottleSize {

    static transients = ['aliases', 'exclusions']

    String id
    String name
    String alias
    String exclusion

    Date dateCreated
    Date lastUpdated

    transient beforeDelete = {
        throw new RuntimeException('delete not allowed')
    }

    static constraints = {
        id length: 4
        name unique: true
        alias nullable: true
        exclusion nullable: true
    }

    static mapping = {
        datasource "wineDatabase"
        id column: 'id', generator: 'assigned'
        alias sqlType: "text"
        exclusion sqlType: "text"
    }

    def getAliases() {
        def aliases = StringUtils.isNotBlank(alias) ? alias.toLowerCase()?.tokenize("|") : new ArrayList<String>()
        return aliases?.unique()
    }

    def getExclusions() {
        def exclusions = StringUtils.isNotBlank(exclusion) ? exclusion.toLowerCase()?.tokenize("|") : new ArrayList<String>()
        return exclusions?.unique()
    }
}
