package wine.db

import wine.db.utils.WineUtils
import wine.db.enums.Source

class WineDb {

    enum Status {
        ACTIVE,
        DELETED
    }

    static hasOne = [wineKeyword: WineKeyword]
    static hasMany = [wines: Wine, wineNoises: WineNoise]

    static transients = ['displayName', 'description']

    String id
    String name
    String color
    String type
    String sweetness
    String variety
    String note

    Producer producer

    String wineCountry
    String wineRegion
    String wineLocation
    String wineLocale
    String wineSite
    String wineNId

    Source source
    Status status

    Date dateCreated
    Date lastUpdated

    static constraints = {
        wineKeyword nullable: true

        id length: 9
        name nullable: true, blank: true
        color nullable: true, blank: true
        type nullable: true, blank: true
        sweetness nullable: true, blank: true
        variety nullable: true, blank: true
        note nullable: true
        wineCountry nullable: true
        wineRegion nullable: true
        wineLocation nullable: true
        wineLocale nullable: true
        wineSite nullable: true
        wineNId nullable: true, length: 100

        source nullable: true
        status nullable: true
    }

    static mapping = {
        datasource "wineDatabase"
        id column: 'id', generator: 'assigned'
        name sqlType: "text"
        note sqlType: "text"
        wineNId column: 'wine_n_id'
    }

    String getDisplayName() {
        return WineUtils.getDisplayName(producer?.nameShow, name)
    }

    String getDescription() {
        return WineUtils.getDescription(producer?.nameShow, name, color)
    }
}
