import enums.*
import grails.util.Environment
import grails.util.Holders
import wine.alert.utils.ServerUtils
import wine.alert.CrawlProperty
import wine.alert.JobLog
import wine.alert.MatchedLog
import wine.alert.RemapKeywordsJob
import wine.alert.WebCrawler

class BootStrap {

    def keywordsService

    def init = { servletContext ->

        if (Environment.current == Environment.TEST) {
            keywordsService.prepare()
            return
        }

        if (Holders.config.getProperty("dataSources.dataSource.dbCreate") == "update") {
            log.info("Server identifier: " + (ServerUtils.getCurrent() ?: ""))

            RemapKeywordsJob.triggerNow()

            // update logs status
            def mLogs = MatchedLog.createCriteria().list {
                eq("status", Status.RUNNING)
            }
            mLogs?.each { MatchedLog m ->
                m.status = Status.FAILED
                m.save()
            }
            def jLogs = JobLog.createCriteria().list {
                eq("status", Status.RUNNING)
            }
            jLogs?.each { JobLog j ->
                j.status = Status.FAILED
                j.save()
            }

            return
        }

        AuthInit.init()

        SeparatedValuesNewInit.init()
        HtmlInit.init()
        CsvInit.init()
        XmlInit.init()
        XlsInit.init()


        WebCrawler www36thavewine = WebCrawler.findByCode("36thavewine")
        if (www36thavewine == null) {
            www36thavewine = new WebCrawler(code: "36thavewine", name: "36th Ave Wine", url: "https://www.36thavewine.com/feeds/36thavewine/winealerts.txt", ignoreSslError: true,
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    priceIndex: 1,  outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: www36thavewine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: www36thavewine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: www36thavewine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: www36thavewine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: www36thavewine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler amantivino = WebCrawler.findByCode("amantivino")
        if (amantivino == null) {
            amantivino = new WebCrawler(code: "amantivino", name: "amantivino", url: "https://www.amantivino.com/feeds/amantivino/winealerts.txt", ignoreSslError: true,
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: amantivino, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: amantivino, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: amantivino, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: amantivino, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: amantivino, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler americas = WebCrawler.findByCode("america's")
        if (americas == null) {
            americas = new WebCrawler(code: "america's", name: "", url: "https://www.americaswineshop.com/feeds/americaswineshop/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: americas, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: americas, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: americas, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: americas, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: americas, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler applejack = WebCrawler.findByCode("applejack")
        if (applejack == null) {
            applejack = new WebCrawler(code: "applejack", name: "applejack", url: "https://www.applejack.com/feeds/applejack/winealerts.txt", ignoreSslError: true,
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: applejack, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: applejack, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: applejack, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: applejack, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: applejack, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler arlingtonwine = WebCrawler.findByCode("arlingtonwine")
        if (arlingtonwine == null) {
            arlingtonwine = new WebCrawler(code: "arlingtonwine", name: "Arlington Wine and Liquor", url: "https://www.arlingtonwine.net/feeds/arlingtonwine/winealerts.txt", ignoreSslError: true,
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: arlingtonwine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: arlingtonwine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: arlingtonwine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: arlingtonwine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: arlingtonwine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler bounty = WebCrawler.findByCode("bounty")
        if (bounty == null) {
            bounty = new WebCrawler(code: "bounty", name: "Bounty Hunter", url: "http://www.bountyhunterwine.com/datafeeds/winealert/wine_alert.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.PIPE, selector: "body", saveAsIs: true,
                    outputTitle: "itemno|merchantcategory|brand|regularprice|case_price|sale_price|case_sale_price|case_discount|isnetitem|category_display|sales|noshow|manufact|varietal|winetype|appellation|subappellation|country|vintage|image_thumbnail|image_detail|offername|offerdescription|webdesc_summary|countreviews|customer_rating|actionurl|payment_accepted|currency").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLc: "Merchant Category", dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Brand", dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.REGULAR_PRICE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Case Price", dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Case Sale Price", dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Case Discount", dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Is Net Item", dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Category Display", dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Sales", dataOutput: true, propertyIndex: 10).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Noshow", dataOutput: true, propertyIndex: 11).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Manufact", dataOutput: true, propertyIndex: 12).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.VARIETAL, dataOutput: true, propertyIndex: 13).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 14).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 15).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.SUB_APPELLATION, dataOutput: true, propertyIndex: 16).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 17).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 18).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Image Thumb", dataOutput: true, propertyIndex: 19).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Image Detail", dataOutput: true, propertyIndex: 20).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Offer Name", dataOutput: true, propertyIndex: 21).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Offer Desc", dataOutput: true, propertyIndex: 22).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Web Desc", dataOutput: true, propertyIndex: 23).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Counter Reviews", dataOutput: true, propertyIndex: 24).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Customer Rating", dataOutput: true, propertyIndex: 25).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 26).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CUSTOM, customLabel: "Payment Accepted", dataOutput: true, propertyIndex: 27).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bounty, wineProperty: WineProperty.CURRENCY, dataOutput: true, propertyIndex: 28).save(flush: true, failOnError: true)
        }

        WebCrawler buyritewines = WebCrawler.findByCode("buyritewines")
        if (buyritewines == null) {
            buyritewines = new WebCrawler(code: "buyritewines", name: "Jersey City Super Buy-Rite", url: "https://www.buyritewines.com/feeds/buyritewines/winealerts.txt", ignoreSslError: true,
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: buyritewines, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: buyritewines, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: buyritewines, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: buyritewines, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: buyritewines, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler bwi = WebCrawler.findByCode("bwi")
        if (bwi == null) {
            bwi = new WebCrawler(code: "bwi", name: "Bottleshop Wines", url: "http://www.grandvinwinemerchants.co.uk/wine/wine-searcher",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.PIPE, selector: "body", saveAsIs: true, harsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: bwi, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bwi, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bwi, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bwi, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: bwi, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler calvert = WebCrawler.findByCode("calvert")
        if (calvert == null) {
            calvert = new WebCrawler(code: "calvert", name: "Calvert Woodley", url: "https://www.calvertwoodley.com/feeds/calvertwoodley/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: calvert, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: calvert, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: calvert, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: calvert, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: calvert, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler centralwinemerchants = WebCrawler.findByCode("centralwinemerchants")
        if (centralwinemerchants == null) {
            centralwinemerchants = new WebCrawler(code: "centralwinemerchants", name: "Central Wine Merchants", url: "https://www.centralwinemerchants.com/feeds/centralwinemerchants/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: centralwinemerchants, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: centralwinemerchants, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: centralwinemerchants, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: centralwinemerchants, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: centralwinemerchants, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler columbuswines = WebCrawler.findByCode("columbuswines")
        if (columbuswines == null) {
            columbuswines = new WebCrawler(code: "columbuswines", name: "Columbus Avenue Wine & Spirits", url: "https://www.columbuswines.com/feeds/columbuswines/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: columbuswines, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: columbuswines, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: columbuswines, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: columbuswines, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: columbuswines, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler dions = WebCrawler.findByCode("dions")
        if (dions == null) {
            dions = new WebCrawler(code: "dions", name: "", url: "http://dionswine.com/winedirectory.asp?source=winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 5, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Vintage|Country|Region|Sub Region|Appellation|Bottle Size|Price|URL|PRODID").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.SUB_REGION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.APPELLATION, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: dions, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
        }

        WebCrawler europeanwineresource = WebCrawler.findByCode("europeanwineresource")
        if (europeanwineresource == null) {
            europeanwineresource = new WebCrawler(code: "europeanwineresource", name: "European Wine Resource", url: "https://www.europeanwineresource.com/feeds/europeanwineresource/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: europeanwineresource, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: europeanwineresource, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: europeanwineresource, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: europeanwineresource, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: europeanwineresource, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler finewineonline = WebCrawler.findByCode("finewineonline")
        if (finewineonline == null) {
            finewineonline = new WebCrawler(code: "finewineonline", name: "Gerard's Wine & Spirits", url: "https://www.finewineonline.com/feeds/finewineonline/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: finewineonline, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: finewineonline, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: finewineonline, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: finewineonline, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: finewineonline, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler fwi = WebCrawler.findByCode("fwi")
        if (fwi == null) {
            fwi = new WebCrawler(code: "fwi", name: "Vinrare", url: "http://www.vinrare.com/datafeed/Dwa0Mc9zZ3",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.COMMA, dataBorder: "\"", selector: "body",
                    saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "PRODUCTID|Name|Price|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: fwi, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: fwi, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: fwi, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: fwi, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
        }

        WebCrawler garyswine = WebCrawler.findByCode("garyswine")
        if (garyswine == null) {
            garyswine = new WebCrawler(code: "garyswine", name: "Gary's Wine & Marketplace", url: "https://www.garyswine.com/feeds/garyswine/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: garyswine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: garyswine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: garyswine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: garyswine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: garyswine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler gillettewine = WebCrawler.findByCode("gillettewine")
        if (gillettewine == null) {
            gillettewine = new WebCrawler(code: "gillettewine", name: "Gillette Ridge Wine & Spirits", url: "https://www.gillettewine.com/feeds/gillettewine/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: gillettewine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: gillettewine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: gillettewine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: gillettewine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: gillettewine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler kdwine = WebCrawler.findByCode("kdwine")
        if (kdwine == null) {
            kdwine = new WebCrawler(code: "kdwine", name: "KD Wines", url: "https://www.kdwine.com/feeds/kdwine/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: kdwine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: kdwine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: kdwine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: kdwine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: kdwine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler linwoodwineco = WebCrawler.findByCode("linwoodwineco")
        if (linwoodwineco == null) {
            linwoodwineco = new WebCrawler(code: "linwoodwineco", name: "Linwood Wine & Liquor Company", url: "https://www.linwoodwineco.com/feeds/linwoodwineco/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: linwoodwineco, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: linwoodwineco, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: linwoodwineco, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: linwoodwineco, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: linwoodwineco, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler ogfinewines = WebCrawler.findByCode("ogfinewines")
        if (ogfinewines == null) {
            ogfinewines = new WebCrawler(code: "ogfinewines", name: "Old Greenwich Fine Wines", url: "https://www.ogfinewines.com/feeds/ogfinewines/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: ogfinewines, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ogfinewines, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ogfinewines, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ogfinewines, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ogfinewines, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler pogoswine = WebCrawler.findByCode("pogoswine")
        if (pogoswine == null) {
            pogoswine = new WebCrawler(code: "pogoswine", name: "Pogo's Wine & Spirits", url: "https://www.pogoswine.com/feeds/pogoswine/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: pogoswine, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: pogoswine, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: pogoswine, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: pogoswine, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: pogoswine, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler premierchampagne = WebCrawler.findByCode("premierchampagne")
        if (premierchampagne == null) {
            premierchampagne = new WebCrawler(code: "premierchampagne", name: "Premier Champagne", url: "http://www.premierchampagne.com/feed-page/",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.PIPE, selector: "div.PageContent p",
                    outputTitle: "Name|Size|Vintage|Price|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: premierchampagne, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: premierchampagne, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: premierchampagne, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: premierchampagne, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: premierchampagne, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler redd = WebCrawler.findByCode("redd")
        if (redd == null) {
            redd = new WebCrawler(code: "redd", name: "", url: "http://reddcollection.com/winedirectory.asp?source=winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 5, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Vintage|Country|Region|Sub Region|Appellation|Bottle Size|Price|URL|PRODID").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.SUB_REGION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.APPELLATION, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: redd, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
        }

        WebCrawler russo = WebCrawler.findByCode("russo")
        if (russo == null) {
            russo = new WebCrawler(code: "russo", name: "Russo Wines", url: "http://www.carlorussowine.com/ic/winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Type|Vintage|Region|Appellation|Price|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: russo, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
        }

        WebCrawler ryebrook = WebCrawler.findByCode("ryebrook")
        if (ryebrook == null) {
            ryebrook = new WebCrawler(code: "ryebrook", name: "Ryebrook Wines", url: "http://www.ryebrookwines.com/winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Type|Vintage|Region|Appellation|Price|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: ryebrook, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
        }

        WebCrawler saratoga = WebCrawler.findByCode("saratoga")
        if (saratoga == null) {
            saratoga = new WebCrawler(code: "saratoga", name: "Saratoga", url: "https://www.saratogawine.com/import/feeds/robertparker.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "PRODID|URL|Price|Name|Description|Region|Sub Region|Country|Vintage|Varietal|Appellation|Sub Appellation|Bottle Size").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.DESCRIPTION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.SUB_REGION, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.VARIETAL, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 10).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.CUSTOM, customLabel: "Sub Appellation", dataOutput: true, propertyIndex: 11).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: saratoga, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 12).save(flush: true, failOnError: true)
        }

        WebCrawler sokolin = WebCrawler.findByCode("sokolin")
        if (sokolin == null) {
            sokolin = new WebCrawler(code: "sokolin", name: "Sokolin", url: "http://www.sokolin.com/media/productsfeed/eRobertParker.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, dataBorder: "\"", selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: sokolin, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: sokolin, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: sokolin, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
        }

        WebCrawler stationplaza = WebCrawler.findByCode("stationplaza")
        if (stationplaza == null) {
            stationplaza = new WebCrawler(code: "stationplaza", name: "Stationplaza Wines", url: "http://www.stationplazawine.com/winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Type|Vintage|Region|Appellation|Price|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stationplaza, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
        }

        WebCrawler stewartswines = WebCrawler.findByCode("stewartswines")
        if (stewartswines == null) {
            stewartswines = new WebCrawler(code: "stewartswines", name: "Stewart's Spirits", url: "https://www.stewartswines.com/feeds/stewartswines/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: stewartswines, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stewartswines, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stewartswines, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stewartswines, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: stewartswines, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler suburban = WebCrawler.findByCode("suburban")
        if (suburban == null) {
            suburban = new WebCrawler(code: "suburban", name: "Suburban Wines", url: "http://www.suburbanwines.com/winealert",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Type|Vintage|Region|Appellation|Price|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: suburban, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
        }

        WebCrawler varmax = WebCrawler.findByCode("varmax")
        if (varmax == null) {
            varmax = new WebCrawler(code: "varmax", name: "varmax", url: "https://www.varmax.com/feeds/varmax/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: varmax, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: varmax, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: varmax, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: varmax, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: varmax, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler wallys = WebCrawler.findByCode("wally's")
        if (wallys == null) {
            wallys = new WebCrawler(code: "wally's", name: "Wallys", url: "http://www.wallywine.com/engine.aspx?XmlPackage=erobertparker&amp;template=Blank",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.COMMA, selector: "div#ctl00_PageContent_pnlContent",
                    outputTitle: "URL|Bottle Size|Vintage|Description|Price").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: wallys, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: wallys, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: wallys, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: wallys, wineProperty: WineProperty.DESCRIPTION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: wallys, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler whwc = WebCrawler.findByCode("whwc")
        if (whwc == null) {
            whwc = new WebCrawler(code: "whwc", name: "Woodland Hills Wine Company", url: "http://www.whwcalerts.com/feeds/erobertparker.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.PIPE, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: whwc, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: whwc, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: whwc, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: whwc, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: whwc, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler winedeals = WebCrawler.findByCode("winedeals")
        if (winedeals == null) {
            winedeals = new WebCrawler(code: "winedeals", name: "WineDeals", url: "http://www.winedeals.com/media/winesearcher.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 0, delimiter: Separator.COMMA, dataBorder: "\"", selector: "body",
                    saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "URL|Name|Vintage|Country|Region|Type|Price|Bottle Size").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.CUSTOM, customLabel: "Image URL", propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.CUSTOM, customLabel: "Null 1", propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.CUSTOM, customLabel: "Null 2", propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 10).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.CUSTOM, customLabel: "Null 3", propertyIndex: 11).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winedeals, wineProperty: WineProperty.CUSTOM, customLabel: "Null 4", propertyIndex: 12).save(flush: true, failOnError: true)
        }

        WebCrawler winemadeeasy = WebCrawler.findByCode("winemadeeasy")
        if (winemadeeasy == null) {
            winemadeeasy = new WebCrawler(code: "winemadeeasy", name: "WineMadeEasy", url: "http://www.winemadeeasy.com/media/winesearcher.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 0, delimiter: Separator.COMMA, dataBorder: "\"", selector: "body",
                    saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "URL|Name|Vintage|Country|Region|Type|Price|Bottle Size").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.CUSTOM, customLabel: "Image URL", propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.CUSTOM, customLabel: "Null 1", propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.CUSTOM, customLabel: "Null 2", propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 10).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.CUSTOM, customLabel: "Null 3", propertyIndex: 11).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winemadeeasy, wineProperty: WineProperty.CUSTOM, customLabel: "Null 4", propertyIndex: 12).save(flush: true, failOnError: true)
        }

        WebCrawler winetransit = WebCrawler.findByCode("winetransit")
        if (winetransit == null) {
            winetransit = new WebCrawler(code: "winetransit", name: "WineTransit", url: "http://www.winetransit.com/media/winesearcher.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 0, delimiter: Separator.COMMA, dataBorder: "\"", selector: "body",
                    saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "URL|Name|Vintage|Country|Region|Type|Price|Bottle Size").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.CUSTOM, customLabel: "Image URL", propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.CUSTOM, customLabel: "Null 2", propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.TYPE, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.CUSTOM, customLabel: "Null 3", propertyIndex: 10).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winetransit, wineProperty: WineProperty.CUSTOM, customLabel: "Null 4", propertyIndex: 11).save(flush: true, failOnError: true)
        }

        WebCrawler winfieldflynn = WebCrawler.findByCode("winfieldflynn")
        if (winfieldflynn == null) {
            winfieldflynn = new WebCrawler(code: "winfieldflynn", name: "winfieldflynn", url: "https://www.winfieldflynn.com/feeds/winfieldflynn/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: winfieldflynn, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winfieldflynn, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winfieldflynn, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winfieldflynn, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: winfieldflynn, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler yorkshirewines = WebCrawler.findByCode("yorkshirewines")
        if (yorkshirewines == null) {
            yorkshirewines = new WebCrawler(code: "yorkshirewines", name: "Yorkshire Wines & Spirits", url: "https://www.yorkshirewines.com/feeds/yorkshirewines/winealerts.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.TAB, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: yorkshirewines, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: yorkshirewines, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: yorkshirewines, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: yorkshirewines, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: yorkshirewines, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler zachys = WebCrawler.findByCode("zachy's")
        if (zachys == null) {
            zachys = new WebCrawler(code: "zachy's", name: "Zachys", url: "http://cms.zachys.com//uploadedfiles/PartnerFiles/winesearcherexport.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 1, delimiter: Separator.PIPE, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Price|Vintage|Bottle Size|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: zachys, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: zachys, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: zachys, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: zachys, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: zachys, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler bevfly = WebCrawler.findByCode("bevfly")
        if (bevfly == null) {
            bevfly = new WebCrawler(code: "bevfly", name: "Bevfly", url: "http://www.bevfly.com/c.3832325/BevFly-LLC/search/index.ssp#/Wine", pageType: PageType.HTML_ASYNC,
                    maxThreads: 3, selector: "div.fs-item-cell-content", usePagination: true, paginationSelector: "#pagination-content a[ng-class='getPageClass(\$index + 1)']",
                    digFurther: false, detailsLinkSelector: ".fs-item-info a", detailsSelectors: "#pdp-general-td-content", detailsSelectorToWait: ".price > div",  charsetEncoding: "ISO-8859-1").save(failOnError: true)

            int count = 0
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.WINE_TITLE, selectors: "h3.fs-item-name", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.BOTTLE_SIZE, selectors: "div.fs-item-description span.item-size", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.TYPE, selectors: "div.fs-item-description span.item-type", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.REGULAR_PRICE, selectors: "div.fs-price-item-list div.regPrice span", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.PRICE, selectors: "div.fs-price-item-list div.specialPrice span", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.URL, selectors: "div.fs-item-info a", dataOutput: true, propertyIndex: count++).save(failOnError: true)
            new CrawlProperty(webCrawler: bevfly, wineProperty: WineProperty.IN_STOCK, selectors: "div.fs-view-more-btn-content div.nostock:not(.ng-hide)", dataOutput: true,
                    ignoreIf: IgnoreIf.EQUALS, ignoreValues: "OUT OF STOCK", propertyIndex: count++).save(failOnError: true)
        }

    }

    def destroy = {
    }
}
