import enums.PageType
import enums.PaginationType
import enums.Separator
import enums.WineProperty
import wine.alert.CrawlProperty
import wine.alert.WebCrawler

/**
 * Created by marjan on 1/22/17.
 */
class DeletedInit {
    public static init() {
        WebCrawler antique = WebCrawler.findByCode("antique")
        if (antique == null) {
            antique = new WebCrawler(code: "antique", name: "Antique Wine", url: "https://awc-wine.com/winesearcher/gbp/ws-datafeed.xml", pageType: PageType.XML,
                    selector: "wine", ignoreSslError: true, outputTitle: "Name|Price|Vintage|Bottle SIze|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: antique, wineProperty: WineProperty.WINE_TITLE, selectors: "wine-name", dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: antique, wineProperty: WineProperty.SELL_PRICE, selectors: "price", dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: antique, wineProperty: WineProperty.VINTAGE, selectors: "vintage", dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: antique, wineProperty: WineProperty.BOTTLE_SIZE, selectors: "bottle-size", dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: antique, wineProperty: WineProperty.URL, selectors: "link", dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
        }

        WebCrawler benchmark = WebCrawler.findByCode("benchmark")
        if (benchmark == null) {
            benchmark = new WebCrawler(code: "benchmark", name: "Benchmark Wines", url: "https://www.benchmarkwine.com/search", pageType: PageType.HTML,
                    selector: "div#search_contents > div.result", detailsLinkSelector: "h2 a[href]", currencyCode: "\$", baseUrl: "https://www.benchmarkwine.com",
                    usePagination: true, paginationType: PaginationType.PAGE_NUMBER, pageNumberName: "page", lastPage: 52, paginationParams: "utf8=%E2%9C%93&per_page=100&sort_by=a_to_z", maxThreads: 10,
                    outputTitle: "Name|Country|Size|Vintage|Price|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.WINE_TITLE, selectors: "div.desc h2 a", dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.COUNTRY, selectors: "div.desc ul.details li:eq(0)", dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.BOTTLE_SIZE, selectors: "div.desc ul.details li:eq(2)", dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.VINTAGE, selectors: "div.desc ul.details li:eq(3)", dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.SELL_PRICE, selectors: "div.desc ul.pricing li.price", textToRemove: "\$", dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: benchmark, wineProperty: WineProperty.URL, selectors: "div.desc h2 a[href]", dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
        }

        WebCrawler petrocks = WebCrawler.findByCode("petrocks")
        if (petrocks == null) {
            petrocks = new WebCrawler(code: "petrocks", name: "", url: "http://petrocksliquors.com/winedirectory.asp?source=winealert&request=w",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 4, delimiter: Separator.TAB, selector: "pre",
                    outputTitle: "Name|Vintage|Country|Region|Sub Region|Appellation|Bottle Size|Price|URL|PRODID").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.COUNTRY, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.REGION, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.SUB_REGION, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.APPELLATION, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: petrocks, wineProperty: WineProperty.PROD_ID, dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
        }

        WebCrawler rl = WebCrawler.findByCode("rl")
        if (rl == null) {
            rl = new WebCrawler(code: "rl", name: "RLLiquid Assets", url: "http://www.rlliquidassets.com/winesearcher.txt",
                    pageType: PageType.SEPARATED_VALUES, rowStart: 0, delimiter: Separator.PIPE, selector: "body", saveAsIs: true, charsetEncoding: "ISO-8859-1",
                    outputTitle: "Name|Description|Vintage|Bottle Size|Price|URL").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.WINE_TITLE, dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.DESCRIPTION, dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.VINTAGE, dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.BOTTLE_SIZE, dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.SELL_PRICE, dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: rl, wineProperty: WineProperty.URL, dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
        }

        WebCrawler vineandtable = WebCrawler.findByCode("vineandtable")
        if (vineandtable == null) {
            vineandtable = new WebCrawler(code: "vineandtable", name: "Vine and Table", url: "http://www.vineandtable.com/robert-parker-products/data-feed.aspx", pageType: PageType.XML,
                    selector: "wine", charsetEncoding: "UTF-16", outputTitle: "Name|Vintage|Type|Varietal|Country|Appellation|URL|Price|Currency|Bottle Size|In Stock").save(flush: true, failOnError: true)

            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.WINE_TITLE, selectors: "name", dataOutput: true, propertyIndex: 0).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.VINTAGE, selectors: "vintage", dataOutput: true, propertyIndex: 1).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.TYPE, selectors: "vintage", dataOutput: true, propertyIndex: 2).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.VARIETAL, selectors: "varietal", dataOutput: true, propertyIndex: 3).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.COUNTRY, selectors: "country", dataOutput: true, propertyIndex: 4).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.APPELLATION, selectors: "appellation", dataOutput: true, propertyIndex: 5).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.URL, selectors: "devise", product_url: true, propertyIndex: 6).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.SELL_PRICE, selectors: "price", dataOutput: true, propertyIndex: 7).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.CURRENCY, selectors: "currency", dataOutput: true, propertyIndex: 8).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.BOTTLE_SIZE, selectors: "bottle_size", dataOutput: true, propertyIndex: 9).save(flush: true, failOnError: true)
            new CrawlProperty(webCrawler: vineandtable, wineProperty: WineProperty.IN_STOCK, selectors: "in_stock", dataOutput: true, propertyIndex: 10).save(flush: true, failOnError: true)
        }
    }
}
