package wine.alert

import groovy.time.Duration
import groovy.time.TimeCategory

/**
 * Cron job to archive unused files.
 */
class ArchiveJob {
    static triggers = {
        //simple repeatInterval: 5000l // execute job once in 5 seconds
        cron name: 'Archive', cronExpression: "0 59 22 ? * SAT" // Fire every saturday at 10:59pm
    }

    def concurrent = false

    def archiveService

    def execute(context) {
        Date date = context.mergedJobDataMap.get('date')
        if (!date) {
            date = new Date()
        }
        date = TimeCategory.minus(date, new Duration(7, 0, 0, 0, 0))
        log.info( "Archive Job started for date: " + date)

        archiveService.archiveCrawlerOutputBackup(date)
        archiveService.archiveKeywordMatchBackup(date)
        archiveService.archiveHistoryMatchBackup(date)
        archiveService.archiveHistoryBackup(date)
        archiveService.archiveRollup(date)
        log.info( "Archive Job completed." )
    }
}
