package wine.alert

import groovy.time.TimeCategory
import wine.alert.utils.ServerUtils

/**
 * Cron Job to clear outdated logs from database.
 */
class ClearLogsJob {
    static triggers = {
        cron name: "ClearLogs", cronExpression: "0 0 1 ? * MON-SUN" // everyday at 1:00am
    }

    def execute() {
        if (!ServerUtils.isDefault()) {
            return
        }

        Date archivedDate = new Date()
        use(TimeCategory) {
            archivedDate = archivedDate - 2.days
        }

        try {
            log.info("Clearing old crawler logs started")
            CrawlLog.withTransaction {
                def affected = CrawlLog.executeUpdate("delete CrawlLog where dateCreated <= :dateCreated", [dateCreated: archivedDate])
                log.info("Clearing old crawler logs affected row(s): " + affected)
            }
        } catch (Exception e) {
            log.error(e.message, e)
        }
        log.info("Clearing old crawler logs finish")
    }
}
