package wine.alert

import enums.MatchedType
import enums.Status
import groovy.time.TimeCategory
import wine.alert.utils.ServerUtils

/**
 * Cron Job for keyword match.
 */
class KeywordMatchJob {
    static triggers = {
        //cron name: "MatchAllKeyword", cronExpression: "0 59 2 ? * *" // everyday at 2:59am
    }

    def concurrent = false

    def matcherService
    def logsService
    def outputService

    def execute() {
        log.info( "Keyword Match Job started." )

        Date today = new Date()

        List<String> crawlers = new ArrayList<>()
        WebCrawler.withTransaction {
            crawlers = WebCrawler.createCriteria().list {
                projections {
                    property("code")
                }
                and {
                    ne("disableKeyword", Boolean.TRUE)
                    ne("status", Status.DELETED)
                    if (ServerUtils.isDefault()) {
                        or {
                            eq("serverIdentifier", ServerUtils.DEFAULT_SERVER)
                            eq("serverIdentifier", "")
                            isNull("serverIdentifier")
                        }
                    } else {
                        eq("serverIdentifier", ServerUtils.getCurrent())
                    }
                }
                order("code", "asc")
            }
        }

        if (crawlers?.size() > 0) {
            matcherService.clearRunningMatches(MatchedType.KEYWORD)
            matcherService.addToRunningMatches(crawlers, MatchedType.KEYWORD)

            crawlers?.each { String code ->
                if (!matcherService.isTerminatedMatch(code, MatchedType.KEYWORD)) {
                    Date crawlDate = outputService.getLatestCrawlDate(code)
                    if (crawlDate) {
                        MatchedLog.withNewTransaction {
                            logsService.saveMatchLog(code, crawlDate, Boolean.TRUE, Status.RUNNING, today)
                        }

                        WebCrawler.withNewTransaction {
                            matcherService.execute(code, Boolean.TRUE, crawlDate)
                        }
                    }
                    matcherService.removeFromRunningMatches(code, MatchedType.KEYWORD)
                } else {
                    log.warn("[" + code + "] was terminated from keyword match.")
                }
            }

            log.info("Keyword Match Job completed. Execution times: " + TimeCategory.minus(new Date(), today) )
        }
    }
}
