package wine.alert

import grails.util.Environment
import org.springframework.beans.factory.annotation.Value
import wine.alert.utils.ServerUtils

/**
 * Cron Job to process RobertParker tasting note files.<br>
 * Moves files from RobertParker Cron Server to WineAlert SFTP Server.
 */
class NotesUploaderJob {
    static triggers = {
      //simple repeatInterval: 5000l // execute job once in 5 seconds
        //cron name: "NotesUploader", cronExpression: "0 59 5 ? * *" // everyday at 5:59am
    }

    def concurrent = false

    static int SFTP_PORT = 22

    @Value('${application.cronServer.sftpHost}')
    private String cronSftpHost

    @Value('${application.cronServer.sftpUsername}')
    private String cronSftpUsername

    @Value('${application.cronServer.sftpPassword}')
    private String cronSftpPassword

    @Value('${application.cronServer.workingDir}')
    private String cronSftpWorkingDir

    @Value('${application.tastingNotes.sftpHost}')
    private String tastingNotesSftpHost

    @Value('${application.tastingNotes.sftpUsername}')
    private String tastingNotesSftpUsername

    @Value('${application.tastingNotes.sftpPassword}')
    private String tastingNotesSftpPassword

    @Value('${application.tastingNotes.workingDir}')
    private String tastingNotesSftpWorkingDir

    def sftpService

    def execute(context) {
        // execute job
        if (!ServerUtils.isDefault()) {
            return
        }

        Boolean test = context.mergedJobDataMap.get('test')
        if (test || Environment.getCurrent() == Environment.DEVELOPMENT) {
            uploadNote("realwine", tastingNotesSftpUsername, tastingNotesSftpPassword)
        } else {
            def crawlers = WebCrawler.createCriteria().list {
                projections {
                    property("code")
                    property("tastingNoteUsername")
                    property("tastingNotePassword")
                }
                eq("uploadTastingNote", true)
            }
            crawlers?.each {
                try {
                    uploadNote((String) it[0], (String) it[1], (String) it[2])
                } catch(Exception e) {
                    log.error(e?.message, e)
                }
            }
        }
    }

    /**
     * Moves files from RobertParker Cron Server to WineAlert SFTP Server.
     * @param retailerCode the retailer code
     * @param ftpUsername the retailer's ftp username
     * @param ftpPassword the retailer's ftp password
     * @return
     */
    def uploadNote(String retailerCode, String ftpUsername, String ftpPassword) {
        String fileName = "retailerTastingNotes-${retailerCode}.txt"
        String cronRemote = cronSftpWorkingDir + fileName
        InputStream inputStream = sftpService.download(cronRemote, cronSftpHost, cronSftpUsername, SFTP_PORT, cronSftpPassword)

        String remote = tastingNotesSftpWorkingDir + fileName
        sftpService.upload(inputStream, remote, tastingNotesSftpHost, ftpUsername, SFTP_PORT, ftpPassword)
    }
}
