/**
 * Service for crawler. To replace the old {@link WebCrawlerService}
 */

package wine.alert

import enums.AuthType
import enums.RequestType
import grails.transaction.Transactional
import wine.alert.crawler.DynamicCrawler
import wine.alert.crawler.request.CrawlEndpoint
import wine.alert.crawler.request.CrawlRequest
import wine.alert.crawler.response.CrawlResponse
import wine.alert.crawler.rest.ReqMethod
import wine.alert.utils.CrawlerUtils

@Transactional(readOnly = true)
class CrawlerService {

    /**
     * Runs a crawler with retry
     * @param webCrawler the instance of {@link WebCrawler}
     * @return the total of records found
     */
    Integer runWithRetry(WebCrawler webCrawler) {
        try {
            return run(webCrawler)
        } catch (Exception ignored) {
            return run(webCrawler)
        }
    }

    /**
     * Runs a crawler
     * @param webCrawler the instance of {@link WebCrawler}
     * @return the total of records found
     */
    Integer run(WebCrawler webCrawler) {
        if (!webCrawler) return 0
        def auth = getAuth(webCrawler)
        CrawlEndpoint endpoint = new CrawlEndpoint(
                url: webCrawler.url,
                auth: auth,
                requestMethod: webCrawler.requestType == RequestType.POST ? ReqMethod.POST : ReqMethod.GET,
                charset: webCrawler.charsetEncoding
        )
        CrawlRequest request = new CrawlRequest(endpoint: endpoint, charset: webCrawler.charsetEncoding)
        DynamicCrawler crawler = new DynamicCrawler(request)
        CrawlResponse response = crawler.crawl() as CrawlResponse
        CrawlerUtils.saveResult(response, webCrawler.code)
        response.total
    }

    /**
     * Gets authentication object
     * @param webCrawler the instance of {@link WebCrawler}
     * @return the instance of {@link PasswordAuthentication}
     */
    private static PasswordAuthentication getAuth(WebCrawler webCrawler) {
        if (webCrawler.authType == AuthType.NONE) return null
        return new PasswordAuthentication(webCrawler.authUsername, webCrawler.authPassword?.chars)
    }
}
