package wine.alert

import grails.transaction.Transactional
import wine.alert.utils.CrawlPostProcessing

/**
 * Service to execute custom processing to the crawler results
 */
@Transactional
class PostProcessingService {

    /**
     * Applies post processing to the crawler results.
     * @param webCrawler the web_crawler/retailer
     * @param data the data to be processed (crawler results)
     */
    void process(WebCrawler webCrawler, List<Map<String, Object>> data) {

        def crawlProperties = CrawlProperty.createCriteria().list(sort: "propertyIndex", order: "asc") {
            and {
                eq("webCrawler", webCrawler)
                eq("dataOutput", true)
            }
        }

        def postProcessingList = PostProcessing.createCriteria().list(sort: "processOrder", order: "asc") {
            'in'("crawlProperty", crawlProperties)
        }

        if (postProcessingList && postProcessingList?.size() > 0) {

            def propIndexMap = [:]
            crawlProperties?.each { CrawlProperty prop ->
                propIndexMap.put(prop.id, prop.indexName)
            }

            postProcessingList?.each { PostProcessing postProcessing ->
                String indexName = propIndexMap.get(postProcessing.crawlPropertyId)
                String[] args = postProcessing?.processArgs?.split("\\r?\\n")

                data.each {
                    def objData = it.get(indexName)
                    String rawText = objData instanceof String ? objData : objData?.toString()

                    rawText = postProcess(postProcessing.processName, it, rawText, args)
                    it[indexName] = rawText ?: ""
                }
            }
        }
    }

    /**
     * Executes given function to the given input. Functions are in {@link wine.alert.utils.CrawlPostProcessing}.
     * @param functionName the function/method name
     * @param dataMap the map of data in current roe
     * @param inputText the input to be processed, taken from the first args.
     * @param args the other arguments
     * @return the processed string
     */
    String postProcess(String functionName, def dataMap, String inputText, String[] args) {
        try {
            CrawlPostProcessing pp = new CrawlPostProcessing()
            if (args && args.length > 0) {
                return pp."${functionName}"(inputText, dataMap, *args)
            } else {
                return pp."${functionName}"(inputText, dataMap)
            }
        } catch (Exception e) {
            log.error(e?.message, e)
        }
        return inputText
    }
}
