package wine.alert

import com.jcraft.jsch.ChannelSftp
import com.jcraft.jsch.JSch
import com.jcraft.jsch.Session
import grails.transaction.Transactional

/**
 * Provides method to download file from sftp server.<br>
 * Provides method to upload file into sftp server.
 */
@Transactional
class SftpService {

    /**
     * Uploads file into sftp server.
     * @param inputStream the file input stream
     * @param remoteFile the location to put the file
     * @param host the sftp host
     * @param username the sftp username
     * @param port the sftp port
     * @param password the sftp password
     */
    void upload(InputStream inputStream, String remoteFile, String host, String username, int port, String password) {
        def fs = remoteFile?.tokenize("/")
        String remoteDir = fs?.size() > 1 ? "/" + fs?.take(fs?.size() - 1)?.join("/") : "/"
        String filename = fs?.getAt(fs?.size() - 1)

        Session session = getSession(host, username, port, password)
        log.debug("SFTP connected to " + host)

        ChannelSftp channel = (ChannelSftp) session.openChannel("sftp")
        channel.connect()
        if (remoteDir != "/") channel.cd(remoteDir)

        log.info("SFTP uploading file " + filename)
        channel.put(inputStream, filename, ChannelSftp.OVERWRITE)
        log.info("SFTP file uploaded " + filename)
    }

    /**
     * Downloads file from sftp server.
     * @param remoteFile the location of the file in remote host.
     * @param host the sftp host
     * @param username the sftp username
     * @param port the sftp port
     * @param password the sftp password
     * @return the file as InputStream {@link InputStream}
     */
    InputStream download(String remoteFile, String host, String username, int port, String password) {
        def fs = remoteFile?.tokenize("/")
        String remoteDir = fs?.size() > 1 ? "/" + fs?.take(fs?.size() - 1)?.join("/") : "/"
        String filename = fs?.getAt(fs?.size() - 1)

        Session session = getSession(host, username, port, password)
        log.debug("SFTP connected to " + host)

        ChannelSftp channel = (ChannelSftp) session.openChannel("sftp")
        channel.connect()
        if (remoteDir != "/") channel.cd(remoteDir)

        log.debug("SFTP downloading file " + filename)
        InputStream inputStream = channel.get(filename)
        return inputStream
    }

    /**
     * Creates sftp connection session.
     * @param host the sftp host
     * @param username the sftp username
     * @param port the sftp port
     * @param password the sftp password
     * @return the sftp login session {@link Session}
     */
    Session getSession(String host, String username, int port, String password) {
        JSch jsch = new JSch()
        Session session = jsch.getSession(username, host, port)
        session.setConfig("StrictHostKeyChecking", "no")
        session.setPassword(password)
        session.connect()
        return session
    }
}
