package wine.db

import grails.async.Promise
import grails.async.Promises
import grails.plugins.rest.client.RestBuilder
import grails.transaction.Transactional
import org.springframework.beans.factory.annotation.Value
import org.springframework.util.LinkedMultiValueMap
import org.springframework.util.MultiValueMap

/**
 * Service to interact with wine API endpoints.
 */
@Transactional
class WineApiService {

    @Value('${application.api.wine-data.token}')
    private String wineDataToken

    @Value('${application.api.wine-data.url.index-wines}')
    private String indexWinesUrl

    private restBuilder = new RestBuilder(connectTimeout: 6000, readTimeout: 20000)

    /**
     * Calls wine-data indexing API endpoint.
     * @param wineDbIds the list of {@link WineDb#id} to be indexed
     */
    void indexingWineDb(List<String> wineDbIds) {
        if (!wineDbIds || wineDbIds.isEmpty()) return
        Promise p = Promises.task {
            Thread.sleep(5000)
            MultiValueMap<String, Object> reqParams = new LinkedMultiValueMap<>()
            reqParams.add("token", wineDataToken)
            reqParams.put("ids", wineDbIds)
            def resp = restBuilder.post(indexWinesUrl) {
                contentType "application/x-www-form-urlencoded"
                body(reqParams)
            }

            if (resp.status == 200) {
                log.info("Wine-data indexing result: " + resp?.json?.message + "\n\t" + resp?.json?.result)
            } else {
                log.warn("Wine-data indexing result: " + resp?.status)
            }
        }
        p.onError { Throwable err ->
            log.error "An error occured ${err.message}"
        }
        p.onComplete { result ->
            log.info "Promise returned $result"
        }
    }
}
