package utils

import org.apache.commons.lang.StringUtils
import org.mozilla.universalchardet.UniversalDetector

/**
 * Utility to detect file encoding.
 */
class CharsetUtils {

    /**
     * Detects encoding of the given file location.
     * @param fName the file location
     * @return the file encoding
     */
    static detect = { String fName ->
        if (StringUtils.isBlank(fName)) {
            return null
        }

        return check(fName)
    }

    /**
     * Detects encoding of the given input stream {@link InputStream}
     * @param fis the file input stream {@link InputStream}
     * @return the file encoding
     */
    static detectInputStream = { InputStream fis ->
        byte[] buf = new byte[1024]
        String encoding = null
        try {
            UniversalDetector detector = new UniversalDetector(null)

            int nread
            while ((nread = fis.read(buf)) > 0 && !detector.isDone()) {
                detector.handleData(buf, 0, nread)
            }
            detector.dataEnd()

            encoding = detector.getDetectedCharset()
            detector.reset()
        } catch (Exception e) {
            e.printStackTrace()
        }
        return encoding
    }

    /**
     * Detects encoding of the given file location.
     * @param fName the file location
     * @return the file encoding
     */
    private static String check(String fName) {
        byte[] buf = new byte[1024]

        String encoding = null
        try {
            FileInputStream fis = new FileInputStream(fName)

            UniversalDetector detector = new UniversalDetector(null)

            int nread
            while ((nread = fis.read(buf)) > 0 && !detector.isDone()) {
                detector.handleData(buf, 0, nread)
            }
            detector.dataEnd()

            encoding = detector.getDetectedCharset()
            detector.reset()
        } catch (Exception e) {
            e.printStackTrace()
        }

        return encoding
    }
}
