package wine.alert.crawler

import com.univocity.parsers.csv.CsvParser
import com.univocity.parsers.csv.CsvParserSettings
import wine.alert.crawler.request.CrawlElement
import wine.alert.crawler.request.CrawlRequest
import wine.alert.crawler.response.CrawlResponse
import wine.alert.crawler.response.IResponse
import wine.alert.crawler.response.Property

/**
 * Crawler for csv file, including tsv and other separated values.
 */
class CsvCrawler extends AbstractCrawler {

    CsvCrawler(CrawlRequest request) {
        this.request = request
    }

    CrawlResponse getFixedProps(IResponse response) {
        if (!response || !response.inputStream) null
        CrawlResponse crawlResponse = new CrawlResponse()
        def rows = getRows(response)
        for (String[] record in rows) {
            List<String> list = record.toList()
            List<Property> props = new ArrayList<>()
            for (CrawlElement element : request.elements) {
                if (list.size() > element.index) continue
                Property property = prepare(list[element.index], element)
                props.push(property)
            }
            if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
        }
        return crawlResponse
    }

    CrawlResponse getAllProps(IResponse response) {
        if (!response || !response.inputStream) null
        CrawlResponse crawlResponse = new CrawlResponse()
        def rows = getRows(response)
        for (String[] record in rows) {
            List<Property> props = new ArrayList<>()
            record.each {
                props.push(new Property(
                        value: it
                ))
            }
            if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
        }
        return crawlResponse
    }

    /**
     * Gets rows from csv file
     * @param response the instance of {@link IResponse}
     * @return the list of array of string
     */
    private List<String[]> getRows(IResponse response) {
        CsvParserSettings settings = new CsvParserSettings()
        if (request.separator) {
            settings.setDelimiterDetectionEnabled(true, request.separator.getChars())
        } else {
            settings.detectFormatAutomatically()
        }
        if (request.start) settings.setNumberOfRowsToSkip(request.start)
        CsvParser parser = new CsvParser(settings)
        List<String[]> rows = parser.parseAll(response.inputStream)
        return rows
    }
}
