/**
 * Crawler for json file.
 */

package wine.alert.crawler

import com.github.opendevl.JFlat
import grails.converters.JSON
import org.grails.web.json.JSONElement
import wine.alert.crawler.request.CrawlElement
import wine.alert.crawler.request.CrawlRequest
import wine.alert.crawler.response.CrawlResponse
import wine.alert.crawler.response.IResponse
import wine.alert.crawler.response.Property
import wine.alert.crawler.utils.JsonUtils

class JsonCrawler extends AbstractCrawler {

    JsonCrawler(CrawlRequest request) {
        this.request = request
    }

    CrawlResponse getFixedProps(IResponse response) {
        if (!response || !response.inputStream) return null
        CrawlResponse crawlResponse = new CrawlResponse()
        JSONElement jsonRoot = JSON.parse(response.getContent())
        JSONElement jsonElement = (JSONElement) Eval.me('ROOT', jsonRoot, 'ROOT.' + request.selector)
        if (jsonElement instanceof List) {
            for (JSONElement item : jsonElement) {
                List<Property> props = new ArrayList<>()
                for (CrawlElement element : request.elements) {
                    Property property = getProperty(item, element)
                    props.push(property)
                }
                if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
            }
        }
        return crawlResponse
    }

    CrawlResponse getAllProps(IResponse response) {
        if (!response || !response.inputStream) return null
        CrawlResponse crawlResponse = new CrawlResponse()
        JFlat flatMe = new JFlat(response.getContent()).json2Sheet()
        for (Object[] o : flatMe.getJsonAsSheet()) {
            List<Property> props = new ArrayList<>()
            for (Object t : o) {
                props.push(new Property(value: JsonUtils.readValue(t) ?: ""))
            }
            if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
        }
        return crawlResponse
    }

    /**
     * Gets property
     * @param json the json element
     * @param element the instance of {@link CrawlElement}
     * @return the instance of {@Link Property}
     */
    private static Property getProperty(JSONElement json, CrawlElement element) {
        Object val = Eval.me('ROOT', json, 'ROOT.' + element.selector)
        return prepare(val, element)
    }
}
