/**
 * Crawler for xml file.
 */

package wine.alert.crawler

import com.github.opendevl.JFlat
import org.apache.commons.lang3.RegExUtils
import org.apache.commons.lang3.StringUtils
import org.json.JSONObject
import org.json.XML
import org.jsoup.Jsoup
import org.jsoup.nodes.Document
import org.jsoup.nodes.Element
import org.jsoup.select.Elements
import wine.alert.crawler.request.CrawlElement
import wine.alert.crawler.request.CrawlRequest
import wine.alert.crawler.response.CrawlResponse
import wine.alert.crawler.response.IResponse
import wine.alert.crawler.response.Property
import wine.alert.crawler.utils.JsonUtils

import java.util.regex.Pattern

class XmlCrawler extends AbstractCrawler {

    private static final Pattern PATTERN_SPACES = Pattern.compile("[\t\\xA0\\u1680\\u180e\\u2000-\\u200a\\u202f\\u205f\\u3000]")
    private static final Pattern PATTERN_MULTIPLE_SPACE = Pattern.compile("\\s+")
    private static final String STR_SPACE = " "

    XmlCrawler(CrawlRequest request) {
        this.request = request
    }

    CrawlResponse getFixedProps(IResponse response) {
        if (!response || !response.inputStream) null
        CrawlResponse crawlResponse = new CrawlResponse()
        Document doc = Jsoup.parse(response.inputStream, request.charset, request.getBaseUrl())
        Elements items = doc.select(request.selector)
        for (Element item : items) {
            List<Property> props = new ArrayList<>()
            for (CrawlElement element : request.elements) {
                Property property = getProperty(item, element)
                props.push(property)
            }
            if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
        }
        return crawlResponse
    }

    CrawlResponse getAllProps(IResponse response) {
        if (!response || !response.inputStream) null
        CrawlResponse crawlResponse = new CrawlResponse()
        JSONObject xmlJSONObj = XML.toJSONObject(response.getContent())
        JFlat flatMe = new JFlat(xmlJSONObj.toString()).json2Sheet()
        for (Object[] o : flatMe.getJsonAsSheet()) {
            List<Property> props = new ArrayList<>()
            for (Object t : o) {
                props.push(new Property(value: JsonUtils.readValue(t) ?: ""))
            }
            if (!shouldIgnoreRecord(props)) crawlResponse.addRecord(props)
        }
        return crawlResponse
    }

    /**
     * Gets property
     * @param node the node, instance of {@link Element}
     * @param element the instance of {@link CrawlElement}
     * @return the instance of {@Link Property}
     */
    private static Property getProperty(Element node, CrawlElement element) {
        String val = null
        Element nodeElement = node.getElementsByTag(element.selector)?.first()
        if (nodeElement) {
            if (element.isAttr) {
                val = nodeElement.attr(element.attrName)
            } else if (element.isUrl) {
                val = nodeElement.absUrl("href")
            } else {
                val = nodeElement.text()
            }
        }
        val = cleanup(val)
        return prepare(val, element)
    }

    /**
     * Replace all non-standard spaces (include \t) with standard space
     * @param text text to be processed
     * @return the processed text
     */
    private static String cleanup(String text) {
        if (StringUtils.isBlank(text)) return text
        text = RegExUtils.replaceAll(text, PATTERN_SPACES, STR_SPACE)
        text = RegExUtils.replaceAll(text, PATTERN_MULTIPLE_SPACE, STR_SPACE)
        text = text.replace("\u000b", "")
        text = text.trim()
        return text
    }
}
