/**
 * Endpoint object of crawl request
 */

package wine.alert.crawler.request

import wine.alert.crawler.rest.ReqMethod

class CrawlEndpoint {
    String id
    String url
    Integer port
    String userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36"
    Integer connectionTimeout = 10000
    Integer readTimeout = 60000
    PasswordAuthentication auth
    ReqMethod requestMethod = ReqMethod.GET
    String charset = "UTF-8"
    String region //Eg: S3 region

    /**
     * Gets base url
     * @return the base url
     */
    String getBaseUrl() {
        if (isWeb()) return new URL(url).host
        null
    }

    /**
     * Checks if the endpoint is locale file
     * @return Boolean
     */
    Boolean isFile() {
        try {
            return new File(url).exists()
        } catch (Exception ignored) {
        }
        return false
    }

    /**
     * Checks if the endpoint is web
     * @return Boolean
     */
    boolean isWeb() {
        return ['http', 'https'].contains(getScheme())
    }

    /**
     * Checks if the endpoint is ftp
     * @return Boolean
     */
    boolean isFtp() {
        return ['ftp'].contains(getScheme())
    }

    /**
     * Checks if the endpoint is sftp
     * @return Boolean
     */
    boolean isSftp() {
        return ['sftp'].contains(getScheme())
    }

    /**
     * Checks if the endpoint is s3
     * @return Boolean
     */
    boolean isS3() {
        return ['s3'].contains(getScheme())
    }

    /**
     * Gets protocol
     * @return the protocol
     */
    String getScheme() {
        return new URI(url).getScheme()
    }
}
