package wine.alert.crawler.response

import org.apache.commons.collections.CollectionUtils
import org.apache.http.MethodNotSupportedException

/**
 * Rest response for crawler
 */
class CrawlResponse implements IResponse {

    CrawlResponse() {
        records = new ArrayList<>()
    }

    List<List<Property>> records

    String getCharset() {
        throw new MethodNotSupportedException("Method getCharset is not supported by CrawlResponse")
    }

    InputStream getInputStream() {
        throw new MethodNotSupportedException("Method getInputStream is not supported by CrawlResponse")
    }

    String getContent() {
        throw new MethodNotSupportedException("Method getContent is not supported by CrawlResponse")
    }

    /**
     * Adds record
     * @param record the list of {@link Property}
     */
    void addRecord(List<Property> record) {
        records.push(record)
    }

    /**
     * Merges another response to this instance
     * @param other the other {@link CrawlResponse}
     */
    void merge(CrawlResponse other) {
        if (other && CollectionUtils.isNotEmpty(other.records)) {
            records.addAll(other.records)
        }
    }

    /**
     * Gets total records
     * @return the total records
     */
    Integer getTotal() {
        if (CollectionUtils.isEmpty(records)) return 0
        records.size()
    }
}
