/**
 * Rest service to get an object from web service
 */

package wine.alert.crawler.rest

import org.jsoup.Connection
import org.jsoup.Jsoup
import wine.alert.crawler.request.CrawlEndpoint
import wine.alert.crawler.response.CommonResponse

class WebRest extends AbstractRest {

    WebRest(CrawlEndpoint endpoint) {
        this.endpoint = endpoint
    }

    CommonResponse execute() {
        Connection conn = Jsoup.connect(endpoint.url)
                .userAgent(endpoint.userAgent)
                .timeout(endpoint.readTimeout)
                .ignoreContentType(true)
                .maxBodySize(0)
        if (endpoint.auth) {
            conn.auth { auth ->
                return auth.credentials(endpoint.auth.userName, endpoint.auth.password?.toString())
            }
        }
        if (endpoint.requestMethod == ReqMethod.POST) {
            conn = conn.method(Connection.Method.POST)

            // Work around for form data
            String params = endpoint.url.tokenize('?').getAt(1)
            params?.split("&")?.each {
                String[] kv = it.split("=")
                conn.data(kv[0], kv[1])
            }
        }
        new CommonResponse(inputStream: conn.execute().bodyStream(), charset: endpoint.charset)
    }

    void done() {
        // no-op
    }
}
