package wine.alert.crawler.utils

import groovy.transform.CompileStatic
import org.apache.commons.lang3.StringUtils
import org.apache.commons.net.ftp.FTPClient
import org.apache.commons.net.ftp.FTPReply

/**
 * A utility for ftp service
 */
@CompileStatic
class FtpUtils {

    /**
     * Downloads file from ftp server
     * @param client the instance of {@link FTPClient}
     * @param url the url
     * @return the file as InputStream {@link InputStream}
     */
    static InputStream download(FTPClient client, String username, String password, String url) {
        if (StringUtils.isBlank(url)) return null
        List<String> fs = url.tokenize("/")
        String serverAddress = fs.get(0)
        client.connect(serverAddress)

        int reply = client.getReplyCode()
        if (!FTPReply.isPositiveCompletion(reply)) {
            client.disconnect()
            throw new Exception("FTP server refused connection: " + serverAddress)
        }

        client.enterLocalPassiveMode()

        if (StringUtils.isNoneBlank(username, password)) {
            if (!client.login(username, password)) {
                throw new Exception("Could not login to FTP Server: " + username)
            }
        }

        if (StringUtils.isAnyBlank(username, password)) {
            if (!client.login("anonymous", "")) {
                throw new Exception("Could not login to FTP Server with anonymous user: " + serverAddress)
            }
        }

        String filename = "/" + fs?.drop(1)?.join("/")
        InputStream inputStream = client.retrieveFileStream(filename)
        return inputStream
    }
}
