package wine.alert.crawler.utils

import com.jcraft.jsch.ChannelSftp
import com.jcraft.jsch.JSch
import com.jcraft.jsch.Session
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import org.apache.commons.lang3.StringUtils

/**
 * A utility for sftp service
 */
@Slf4j
@CompileStatic
class SftpUtils {

    /**
     * Downloads file from sftp server
     * @param session the session
     * @param remoteFile the location of the file in remote host
     * @return the file as InputStream {@link InputStream}
     */
    static InputStream download(Session session, String remoteFile) {
        if (StringUtils.isBlank(remoteFile)) return null
        List<String> fs = remoteFile.tokenize("/")
        String remoteDir = fs.size() > 1 ? "/" + fs.take(fs.size() - 1).join("/") : "/"
        String filename = fs.get(fs.size() - 1)

        ChannelSftp channel = (ChannelSftp) session.openChannel("sftp")
        channel.connect()
        if (remoteDir != "/") channel.cd(remoteDir)

        log.debug("SFTP downloading file " + filename)
        InputStream inputStream = channel.get(filename)
        return inputStream
    }

    /**
     * Creates sftp connection session
     * @param username the sftp username
     * @param host the sftp host
     * @param port the sftp port
     * @param password the sftp password
     * @return the sftp login session {@link Session}
     */
    static Session createSession(String username, String host, int port, String password) {
        JSch jsch = new JSch()
        Session session = jsch.getSession(username, host, port)
        session.setConfig("StrictHostKeyChecking", "no")
        session.setPassword(password)
        session.connect()
        return session
    }
}
