package wine.alert.model

import groovy.transform.CompileStatic

/**
 * Model for wine match output.
 */
@CompileStatic
class MatchOutput {
    private String wineDbId
    private String wineId
    private String wineName
    private String match
    private String errors
    private String bottleSize
    private String vintage
    private BigDecimal price
    private String taxStatus
    private String url
    private String description
    private String keywordString
    private String originalKeywordString
    private String historyString
    private String originalHistoryString
    private String sku

    private String delimiter
    private String inputText
    private String inputDelimiter

    MatchOutput() {
        this.wineId = "?"
        this.wineName = "unknown"
        this.match = "unknown"
        this.errors = ""
        this.bottleSize = ""
        this.vintage = ""
        this.price = null
        this.taxStatus = ""
        this.url = ""
        this.description = ""
        this.keywordString = ""
        this.originalKeywordString = ""
        this.historyString = ""
        this.originalHistoryString = ""
        this.sku = ""

        this.delimiter = "|"
        this.inputText = ""
        this.inputDelimiter = "\\|"
    }

    MatchOutput(MatchInput matchInput, String output, String inputDelimiter) {
        this.wineId = "?"
        this.wineName = "unknown"
        this.match = "unknown"
        this.errors = ""
        this.bottleSize = matchInput.size
        this.vintage = matchInput.vintage
        this.price = matchInput.price
        this.taxStatus = matchInput.taxStatus
        this.url = matchInput.url
        this.description = matchInput.description
        this.keywordString = matchInput.keywordText
        this.originalKeywordString = matchInput.originalKeywordText
        this.historyString = matchInput.historyText
        this.originalHistoryString = matchInput.originalHistoryText
        this.sku = matchInput.sku

        this.delimiter = "|"
        this.inputText = output
        this.inputDelimiter = inputDelimiter
    }

    String getWineDbId() {
        return wineDbId
    }

    void setWineDbId(String wineDbId) {
        this.wineDbId = wineDbId
    }

    String getWineId() {
        return wineId
    }

    void setWineId(String wineId) {
        this.wineId = wineId
    }

    String getWineName() {
        return wineName
    }

    void setWineName(String wineName) {
        this.wineName = wineName
    }

    String getMatch() {
        return match
    }

    void setMatch(String match) {
        this.match = match
    }

    String getErrors() {
        return errors
    }

    void setErrors(String errors) {
        this.errors = errors
    }

    String getBottleSize() {
        return bottleSize
    }

    void setBottleSize(String bottleSize) {
        this.bottleSize = bottleSize
    }

    String getVintage() {
        return vintage
    }

    void setVintage(String vintage) {
        this.vintage = vintage
    }

    BigDecimal getPrice() {
        return price
    }

    void setPrice(BigDecimal price) {
        this.price = price
    }

    String getTaxStatus() {
        return taxStatus
    }

    void setTaxStatus(String taxStatus) {
        this.taxStatus = taxStatus
    }

    String getUrl() {
        return url
    }

    void setUrl(String url) {
        this.url = url
    }

    String getDescription() {
        return description
    }

    void setDescription(String description) {
        this.description = description
    }

    String getKeywordString() {
        return keywordString
    }

    void setKeywordString(String keywordString) {
        this.keywordString = keywordString
    }

    String getOriginalKeywordString() {
        return originalKeywordString
    }

    void setOriginalKeywordString(String originalKeywordString) {
        this.originalKeywordString = originalKeywordString
    }

    String getHistoryString() {
        return historyString
    }

    void setHistoryString(String historyString) {
        this.historyString = historyString
    }

    String getOriginalHistoryString() {
        return originalHistoryString
    }

    void setOriginalHistoryString(String originalHistoryString) {
        this.originalHistoryString = originalHistoryString
    }

    String getSku() {
        return sku
    }

    void setSku(String sku) {
        this.sku = sku
    }

    String getDelimiter() {
        return delimiter
    }

    void setDelimiter(String delimiter) {
        this.delimiter = delimiter
    }

    String getInputText() {
        return inputText
    }

    void setInputText(String inputText) {
        this.inputText = inputText
    }

    String getInputDelimiter() {
        return inputDelimiter
    }

    void setInputDelimiter(String inputDelimiter) {
        this.inputDelimiter = inputDelimiter
    }

    public String getOutput() {
        // Output should match these following fields
        // Wine Id|Wine Name|Match|Errors|Bottle Size|Vintage|Price|Tax Status|URL|Retailer Description|Keyword String|History String|SKU

        StringBuilder stringBuilder = new StringBuilder()
        stringBuilder.append(wineId ?: "").append(delimiter)
        stringBuilder.append(wineName ?: "").append(delimiter)
        stringBuilder.append(match ?: "").append(delimiter)
        stringBuilder.append(errors ?: "").append(delimiter)
        stringBuilder.append(bottleSize ?: "").append(delimiter)
        stringBuilder.append(vintage ?: "").append(delimiter)
        stringBuilder.append(price ?: "").append(delimiter)
        stringBuilder.append(taxStatus ?: "").append(delimiter)
        stringBuilder.append(url ?: "").append(delimiter)
        stringBuilder.append(description ?: "").append(delimiter)
        stringBuilder.append(originalKeywordString ?: "").append(delimiter)
        stringBuilder.append(originalHistoryString ?: "").append(delimiter)
        stringBuilder.append(sku ?: "").append(delimiter)
        String originalInput = inputText?.split(inputDelimiter)?.join(delimiter)
        stringBuilder.append(originalInput ?: "")

        return stringBuilder.toString()
    }
}
