package wine.alert.parser

import com.opencsv.CSVParser
import com.opencsv.CSVReader
import com.opencsv.CSVReaderBuilder

/**
 * CSV parser, utilize com.opencsv package.
 */
class CsvReader {

    /**
     * Converts csv text to list of string.
     * @param text the text to be parsed
     * @param parser the CSVParser instance {@link CSVParser}
     * @return the list of string
     * @throws Exception
     */
    List<String[]> parseText(String text, CSVParser parser) throws Exception {
        CSVReader csvReader = new CSVReaderBuilder(new StringReader(text)).withCSVParser(parser).build()
        List<String[]> list = csvReader.readAll()
        csvReader.close()
        return list
    }

    /**
     * Reads content from Reader {@link Reader} using CSVReader {@link CSVReader}
     * @param reader the reader
     * @return the list of string
     * @throws Exception
     */
    List<String[]> readAll(Reader reader) throws Exception {
        CSVReader csvReader = new CSVReader(reader)
        List<String[]> list = csvReader.readAll()
        reader.close()
        csvReader.close()
        return list
    }
}
