package wine.alert.utils

import enums.Status

/**
 * Used to validate crawler results.
 */
class CrawlDataValidator {

    /**
     * Validates the total rows.
     * @param data the data to be validated
     * @param minExpectedRecords the minimum expected records
     * @return the validation status {@link Status}
     */
    public Status minTotalRecords(def data, String minExpectedRecords) {
        if (data?.size() < minExpectedRecords?.toInteger()) {
            return Status.FAILED
        }
        return Status.SUCCESS
    }

    /**
     * Validates the price summary.
     * @param data the data to be validated
     * @param totalPrices the total price
     * @param averageTotalPrices the average of the total price
     * @param allowedMargin the minimum allowed margin
     * @return the validation status {@link Status}
     */
    public Status isValidTotalPrices(def data, String totalPrices, String averageTotalPrices, String allowedMargin) {
        BigDecimal t = totalPrices ?  new BigDecimal(totalPrices) : BigDecimal.ZERO
        BigDecimal av = averageTotalPrices ? new BigDecimal(averageTotalPrices) : BigDecimal.ZERO
        BigDecimal margin = allowedMargin ? new BigDecimal(allowedMargin) : BigDecimal.ZERO
        if (av != null && av > 0) {
            if ((t - av).abs() / av * new BigDecimal(100) > margin) {
                return Status.FAILED
            }
        }
        return Status.SUCCESS
    }
}
