package wine.alert.utils

import groovy.util.logging.Slf4j
import org.apache.commons.lang.StringUtils
import utils.CharsetUtils

@Slf4j
class IOUtils {

    private final static String DEFAULT_CHARSET = "UTF-8"
    private final static String TEXT_BORDER = "\""

    static List<List<String>> getContents(String fileName, String delimiter, boolean hasHeader) {
        if (StringUtils.isBlank(fileName)) {
            return Collections.emptyList()
        }
        File file = new File(fileName)
        if (!file) {
            log.error("File not found.")
            return Collections.emptyList()
        }
        List<List<String>> cts = new ArrayList<>()
        String fileEncoding = CharsetUtils.detect(fileName) ?: DEFAULT_CHARSET
        file.withReader(fileEncoding) { reader ->
            int c = 0
            String line
            while ((line = reader.readLine())!=null) {
                c += 1
                if (StringUtils.isBlank(line) || (c == 1 && hasHeader)) continue
                List<String> contents = new ArrayList<>()
                String[] fields = line.split(delimiter)
                for (String field: fields) {
                    if (StringUtils.startsWith(field, TEXT_BORDER) && StringUtils.endsWith(field, TEXT_BORDER)) { // Only if start and end with quote. Handler for excel quote escape
                        field = StringUtils.substring(field, 1, field.length() - 1)
                        field = field.trim()
                    }
                    contents.push(StringUtils.isBlank(field) ? null : field)
                }
                cts.add(contents)
            }
        }
        return cts
    }
}
