package wine.alert.utils

import org.apache.commons.lang.StringUtils

/**
 * Utility to translate old application field index to the new field index.<br>
 * Field index of old application starts from 1<br>
 * The new field index starts from 0
 */
class IndexesUtils {

    /**
     * Increase indexes by 1
     * @param indexes the text that contains indexes
     * @return the increased indexes
     */
    static increase = { String indexes ->
        if (StringUtils.isBlank(indexes)) {
            return null
        }
        return plusOne(splitToInteger(indexes)).join(",")
    }

    /**
     * Decrease indexes by 1
     * @param indexes the text that contains indexes
     * @return the decreased indexes
     */
    static decrease = { String indexes ->
        if (StringUtils.isBlank(indexes)) {
            return null
        }
        return minusOne(splitToInteger(indexes)).join(",")
    }

    /**
     * Split list of indexes (comma separated).
     * @param indexes the text that contains indexes
     * @return the list of indexes
     */
    private static List<Integer> splitToInteger(String indexes) {
        List<Integer> iList = new ArrayList<>()
        indexes.tokenize(",")?.each {
            iList.add(Integer.valueOf(it))
        }
        return iList
    }

    /**
     * Increase all indexes by 1
     * @param indexes the list of indexes
     * @return the increased indexes
     */
    private static List<Integer> plusOne(List<Integer> indexes) {
        List<Integer> iList = new ArrayList<>()
        indexes.each {
            iList.add(it+1)
        }
        return iList
    }

    /**
     * Decrease all indexes by 1
     * @param indexes the list of indexes
     * @return the decreased indexes
     */
    private static List<Integer> minusOne(List<Integer> indexes) {
        List<Integer> iList = new ArrayList<>()
        indexes.each {
            if (it >= 1) {
                iList.add(it-1)
            }
        }
        return iList
    }
}
