package wine.alert.utils

/**
 * Utility to check the server identifier.<br>
 * Default server identifier are "DEFAULT", BLANK or NULL
 */
class ServerUtils {

    public static final DEFAULT_SERVER = "DEFAULT"

    /**
     * Gets current server identifier.
     * @return the server identifier
     */
    static getCurrent = {
        return getIdentifier()
    }

    /**
     * Check if the current server is default server.
     * @return true if the current server is default server
     */
    static isDefault = {
        String id = getIdentifier()
        return id == null || id.length() == 0 || id.toUpperCase() == DEFAULT_SERVER
    }

    /**
     * Reads serverIdentifier from System property.
     * @return the server identifier
     */
    private static String getIdentifier() {
        return System.getProperty("serverIdentifier")
    }
}
