package wine.alert;

import java.util.ArrayList;
import java.util.List;

/**
 * Model for keyword match result.<br>
 * Support concurrent process.
 */
public class AtomicKwMatch {

    private static final int MAX_AMBIGUOUS = 5;
    private volatile int score;
    private volatile List<String> matches;

    public AtomicKwMatch() {
        this.matches = new ArrayList<>();
        this.score = 0;
    }

    public AtomicKwMatch(List<String> matches, int score) {
        this.matches = matches;
        this.score = score;
    }

    public int getScore() {
        return this.score;
    }

    public List<String> getMatches() {
        return this.matches;
    }

    public int getMatchesSize() {
        return this.matches.size();
    }

    public String getFirstMatch() {
        return this.matches.size() > 0 ? this.matches.get(0) : null;
    }

    public synchronized void set(String match, int score) {
        if (score > 0) {
            if (score > this.score) {
                this.matches.clear();
                this.matches.add(match);
                this.score = score;
            } else if (score == this.score && getMatchesSize() < MAX_AMBIGUOUS) {
                this.matches.add(match);
            }
        }
    }
}
