package wine.alert

import enums.MatchedType
import grails.test.mixin.TestFor
import groovy.time.Duration
import groovy.time.TimeCategory
import spock.lang.Shared
import spock.lang.Specification

import java.text.SimpleDateFormat

/**
 * See the API for {@link grails.test.mixin.services.ServiceUnitTestMixin} for usage instructions
 */
@TestFor(ArchiveService)
class ArchiveServiceSpec extends Specification {

    @Shared Date date
    @Shared SimpleDateFormat sdf
    @Shared String retailer
    @Shared String backupMatchedLocation
    @Shared String backupCrawlLocation
    @Shared String archiveLocation
    @Shared String crawlFileName
    @Shared String historyFileName
    @Shared String keywordFileName

    def setup() {
        retailer = "retailer"
        sdf = new SimpleDateFormat("yyyy-MM-dd")
        date = TimeCategory.minus(new Date(), new Duration(8, 0, 0, 0, 0))
        backupCrawlLocation = getConfig().getProperty("application.crawlOut.backupLocation")
        backupMatchedLocation = getConfig().getProperty("application.matched.backupLocation")
        archiveLocation = getConfig().getProperty("application.archive.location")

        // Create history match file
        crawlFileName = backupCrawlLocation + File.separator + retailer + "-" + sdf.format(date) + ".txt"
        File crawlFile = new File(crawlFileName)
        crawlFile.getParentFile().mkdirs()
        crawlFile.createNewFile()
        crawlFile.write("Test", "UTF-8")

        // Create history match file
        historyFileName = backupMatchedLocation + File.separator + MatchedType.HISTORY.value() + File.separator + retailer + "-" + sdf.format(date) + "-matched.txt"
        File historyFile = new File(historyFileName)
        historyFile.getParentFile().mkdirs()
        historyFile.createNewFile()
        historyFile.write("Test", "UTF-8")

        // Create keyword match file
        keywordFileName = backupMatchedLocation + File.separator + MatchedType.KEYWORD.value() + File.separator + retailer + "-" + sdf.format(date) + "-matched.txt"
        File keywordFile = new File(keywordFileName)
        keywordFile.getParentFile().mkdirs()
        keywordFile.createNewFile()
        keywordFile.write("Test", "UTF-8")

        // Create archive test dir
        new File(archiveLocation).mkdirs()
    }

    def cleanup() {
        new File(archiveLocation)?.deleteDir()
        new File(crawlFileName)?.delete()
        new File(historyFileName)?.delete()
        new File(keywordFileName)?.delete()
    }

    void "Test archive 1: "() {
        service.archive(date)
        def archives = []
        new File(archiveLocation)?.eachFile() { file ->
            if (file.name.startsWith("input_${sdf.format(date)}") ||
                    file.name.startsWith("matched-keyword_${sdf.format(date)}") ||
                    file.name.startsWith("matched-history_${sdf.format(date)}")) {
                archives.add(file.getAbsolutePath())
            }
        }
        expect:
        archives.size() == 3
    }
}
