package wine.alert

import spock.lang.Specification

class CalculatorSpec extends Specification {

    String wineName = "Adam Gewurztraminer Cuvee Jean Baptiste Kaefferkopf"
    String pattern = "adam&jean&baptiste&gewur_&kaef_&cuvee&kaef_&cuvee&kaef_&cuvee&kaef_&cuvee&!red&!rouge&!rose&!reserve&!sgn&!vt&!vv"

    def setup() {
    }

    def cleanup() {
    }

    void "Test for cleanup: "() {
        expect:
        Calculator.cleanup("1 bt. \"Maya\", Dalla Valle Vineyards, Napa Valley 1993 A (hf/in).") == "1 bt maya dalla valle vineyards napa valley 1993 a hf in"
    }

    /**
     * Test for string eval that return Boolean "TRUE or FALSE", eg: "1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&!0"
     */
    void "Test for score match 1: "() {
        expect:
            Calculator.getScore(Calculator.cleanup(wineName), pattern,
                    Calculator.normalizeAndSortPattern(pattern),
                    Calculator.normalizePattern(pattern), 0, Calculator.requiredWords(pattern, true)) == 12
    }

    /**
     * Test for string eval that return Integer "1 or 0", eg: "1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&1&&1"
     */
    void "Test for score match 2: "() {
        String wineName = "les haut de la gaffeliere les haut de la gaffeliere 2016 red st emilion"
        String wineKeyword = "gaffeliere&haut&gaffeliere&haut&gaffeliere&haut&gaffeliere&haut&gaffeliere&haut&gaffeliere&haut&gaffeliere"
        expect:
            Calculator.getScore(wineName, wineKeyword,
                    Calculator.normalizeAndSortPattern(wineKeyword),
                    Calculator.normalizePattern(wineKeyword), 0,
                    Calculator.requiredWords(wineKeyword, true)) == 13
    }

    void "Test for positive words 1: "() {
        expect:
            Calculator.truePositiveWords(pattern) == ["adam", "jean", "baptiste", "gewur_", "kaef_", "cuvee"]
    }

    void "Test for positive words 2: "() {
        String p = "camel&valley&pinot&!gris&!grigio&!auxer&!blanc&!bianco&brut&(rose|rosado|rosato)&!red&!rouge&!white&!blanc&!blanco&!reserve"
        expect:
            Calculator.truePositiveWords(p) == ["camel", "valley", "pinot", "brut", "rose", "rosado", "rosato"]
    }

    void "Test for positive words 3: "() {
        String p = "(zind humbrecht|zh)&gewur_&windsbuhl&windsbuhl&windsbuhl&windsbuhl&!red&!rouge&!rose&!reserve&!sgn&!vt&!vv"
        expect:
            Calculator.truePositiveWords(p, true) == ["zind humbrecht", "zh", "gewur_", "windsbuhl", "windsbuhl", "windsbuhl", "windsbuhl"]
    }

    void "Test for required words 1: "() {
        expect:
            Calculator.requiredWords("camel&valley&pinot&!gris&!grigio&!auxer&!blanc&!bianco&brut&(rose|rosado|rosato)&!red&!rouge&!white&!blanc&!blanco&!reserve", true) == ["camel","valley","pinot","brut"]
    }

    void "Test for required words 2: "() {
        expect:
            Calculator.requiredWords("(zind humbrecht|zh)&gewur_&windsbuhl&windsbuhl&windsbuhl&windsbuhl&!red&!rouge&!rose&!reserve&!sgn&!vt&!vv", true) == ["gewur","windsbuhl"]
    }

    void "Test for required words 3: "() {
        expect:
            Calculator.requiredWords("troplong&mondot&!2nd&!de&!white&!rose&!bianco&!rosado&!bl&!rosato&!blanc&!blanco&!branco&!weiss&!reserve&!vv", true) == ["troplong","mondot"]
    }

    void "Test for required words 4: "() {
        expect:
            Calculator.requiredWords("chasse&spleen&!l'h_&!her_&!white&!rose&!bianco&!rosado&!bl&!rosato&!blanc&!blanco&!branco&!weiss&!reserve&!vv", true) == ["chasse","spleen"]
    }

    void "Test for required words 5: "() {
        expect:
            Calculator.requiredWords("fargueirol&chateauneuf&100%&mourvedre&100%&mourvedre&100%&mourvedre&100%&mourvedre&100%&mourvedre&100%&mourvedre&!white&!rose&!bianco&!rosado&!bl&!rosato&!blanc&!blanco&!branco&!weiss&!reserve", true) == ["fargueirol","chateauneuf","100%","mourvedre"]
    }

    void "Test for vintage parser 1: "() {
        expect:
            Calculator.parseVintage("Vega Sicilia Unico Reserva Especial (1994)") == "1994"
    }

    void "Test for vintage parser 2: "() {
        expect:
            Calculator.parseVintage("2017 Vega Sicilia Unico Reserva Especial") == "2017"
    }

    void "Test for vintage parser 3: "() {
        expect:
            Calculator.parseVintage("Vega Sicilia Unico Reserva Especial - 2017") == "2017"
    }

    void "Test for vintage parser 4: "() {
        expect:
            Calculator.parseVintage("Vega Sicilia Unico Reserva Especial NV") == "NV"
    }

    void "Test for vintage parser 5: "() {
        expect:
            Calculator.parseVintage("Vega Sicilia Unico Reserva Especial N.V. Nova") == "NV"
    }

    void "Test for vintage parser 6: "() {
        expect:
        Calculator.parseVintage("Vega Sicilia Unico Reserva Especial (2017-2008-2009)") == "2017"
    }

    void "Test for vintage parser 7: "() {
        expect:
        Calculator.parseVintage("Vega Sicilia Unico Reserva Especial 2017 hf/in") == "2017"
    }

    void  "Test for vintages cleanup 1: "() {
        expect:
            Calculator.removeVintages("2017 Penfolds Cabernet Shiraz Bin 389 2014st 2005 2005 1997, 1998-2000, (1999) ,2010 N.V. NV") == "Penfolds Cabernet Shiraz Bin 389 2014st"
    }

    void  "Test for vintages cleanup 2: "() {
        expect:
        Calculator.cleanup(Calculator.removeVintages(
                    "Penfolds Cabernet Shiraz Bin 2017 2014st 2005 2005 1997, 1998-2000, (1999) ,2010 N.V. NV",
                    "penfolds&cabernet&shiraz&bin"
            )) == Calculator.cleanup("Penfolds Cabernet Shiraz Bin 2014st")
    }

    void  "Test for vintages cleanup 3: "() {
        expect:
        Calculator.cleanup(Calculator.removeVintages(
                    "Penfolds Cabernet Shiraz Bin 2017 2014st 2005 2005 1997, 1998-2000, (1999) ,2010 NV",
                    "penfolds&cabernet&shiraz&bin&2017"
            )) == Calculator.cleanup("Penfolds Cabernet Shiraz Bin 2017 2014st")
    }

    void  "Test for ascii 1: "() {
        expect:
            Calculator.isAscii("Penfolds Cabernet Shiraz Bin 389 1997, 1998, (1999) ,2010".toLowerCase())
    }

    void  "Test for ascii 2: "() {
        expect:
            !Calculator.isAscii("Vosne-Romanée 1er cru".toLowerCase())
    }

    void  "Test for money parser 1: "() {
        expect:
        Calculator.parseMoney("£1,071.00") == new BigDecimal("1071")
    }

    void  "Test for money parser 2: "() {
        expect:
        Calculator.parseMoney("\"£1,071.00\"") == new BigDecimal("1071")
    }
}
