package wine.alert

import enums.MatchedType
import grails.test.mixin.TestFor
import spock.lang.Shared
import spock.lang.Specification

import java.text.SimpleDateFormat

/**
 * See the API for {@link grails.test.mixin.services.ServiceUnitTestMixin} for usage instructions
 */
@TestFor(OutputService)
class OutputServiceSpec extends Specification {

    @Shared WebCrawler webCrawler
    @Shared String kwMatchLocation
    @Shared String kwMatchData
    @Shared String kwMatchDataAddition
    @Shared Date date

    def setupSpec() {
        mockDomain WebCrawler
    }

    def setup() {
        webCrawler = new WebCrawler(
                code: "retailer",
                name: "Retailer",
                outputEncoding: "UTF-8"
        )
        kwMatchLocation = getConfig().getProperty("application.matched.location")
        kwMatchData = """Wine Id|Wine Name|Match|Errors|Bottle Size|Vintage|Price|Tax Status|URL|Retailer Description|Keyword String|History String|SKU|
?|unknown|unknown|unknown|750 ml - Standard Bottle|2011||||Agrapart Avizoise 2011|Agrapart Avizoise 2011|Agrapart Avizoise 2011||Agrapart Avizoise 2011
ch110110621175|Agrapart Extra Brut Blanc de Blancs Grand Cru Mineral  White|keyword||750 ml - Standard Bottle|2011||||Agrapart Minéral 2011|Agrapart Minéral 2011|Agrapart Minéral 2011||Agrapart Minéral 2011
ch110610610075|Ayala Brut Majeur  White|keyword||750 ml - Standard Bottle|NV||||Ayala Brut Majeur NV|Ayala Brut Majeur NV|Ayala Brut Majeur NV||Ayala Brut Majeur NV
ch111310420775|Billecart-Salmon Brut Elisabeth Salmon Rose  Rose|keyword||750 ml - Standard Bottle|2007||||Billecart-Salmon Rosé Cuvee Elisabeth Salmon 2007|Billecart-Salmon Rosé Cuvee Elisabeth Salmon 2007|Billecart-Salmon Rosé Cuvee Elisabeth Salmon 2007||Billecart-Salmon Rosé Cuvee Elisabeth Salmon 2007
ch111610320275|Bollinger R.D.  White|keyword||750 ml - Standard Bottle|2002||||Bollinger R.D. 2002|Bollinger R.D. 2002|Bollinger R.D. 2002||Bollinger R.D. 2002
"""
        kwMatchDataAddition = """bo138510198675|Chateau Leoville-Las Cases  Red|keyword||750 ml - Standard Bottle|1986||||Leoville Las Cases 1986|Leoville Las Cases 1986|Leoville Las Cases 1986||Leoville Las Cases 1986
bo156510099575|Troplong-Mondot  Red|keyword||750 ml - Standard Bottle|1995||||Troplong Mondot 1995|Troplong Mondot 1995|Troplong Mondot 1995||Troplong Mondot 1995
"""
        date = new Date()
    }

    def cleanup() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd")
        new File(kwMatchLocation + File.separator + MatchedType.KEYWORD.value() + File.separator + "retailer" + "-" + sdf.format(date) + "-matched.txt").delete()
    }

    void "Test keyword match file loader 1: "() {
        service.saveMatchedToFile(webCrawler.code, webCrawler.outputEncoding, kwMatchData, date, true)
        sleep(5)
        expect:
        kwMatchData.split("\n").toList().drop(1) == service.loadMatchedResults(webCrawler, MatchedType.KEYWORD)
    }

    void "Test keyword match file loader 2: "() {
        (0..10).each {
            service.saveMatchedToFile(webCrawler.code, webCrawler.outputEncoding, kwMatchData, date, true)
            sleep(5)
        }
        expect:
        kwMatchData.split("\n").toList().drop(1) == service.loadMatchedResults(webCrawler, MatchedType.KEYWORD)
    }

    void "Test keyword match file loader 3: "() {
        String updateData = kwMatchData + kwMatchDataAddition
        service.saveMatchedToFile(webCrawler.code, webCrawler.outputEncoding, updateData, date, true)
        sleep(5)
        expect:
        updateData.split("\n").toList().drop(1) == service.loadMatchedResults(webCrawler, MatchedType.KEYWORD)
    }
}
