package wine.alert

import grails.test.mixin.TestFor
import spock.lang.Specification
import wine.alert.model.RollupOutput

/**
 * See the API for {@link grails.test.mixin.services.ServiceUnitTestMixin} for usage instructions
 */
@TestFor(RollupService)
class RollupServiceSpec extends Specification {

    void setup() {

    }

    void cleanup() {

    }

    def "test standardizePrice with standard bottle"() {
        expect: "does not change standard bottle price"
            service.standardizePrice("750 ml - Standard Bottle", 100.0) == 100.0
            service.standardizePrice("750 ml - Standard Bottle w/screwcap", 100.0) == 100.0
    }

    def "test standardizePrice with multiple standard bottles"() {
        expect: "divides price by number of bottles"
            Math.round(service.standardizePrice("8 Standard Bottles", 100.0)) == Math.round(100.0 / 8)
            Math.round(service.standardizePrice("9 Standard Bottles", 100.0)) == Math.round(100.0 / 9)
    }

    def "test standardizePrice with non-standard bottles"() {
        expect: "divides price by number of bottles and multiply by the ratio to standard bottle"
            Math.round(service.standardizePrice("1.5 L - Magnum", 100.0)) == Math.round(100.0 * 750 / 1500)
            Math.round(service.standardizePrice("12 Bottle Case -  187 ml - Splits", 100.0)) == Math.round(100.0 / 12 * 750 / 187)
            Math.round(service.standardizePrice("12 Bottle Case - 620 ml Clavelins (Jura)", 100.0)) == Math.round(100.0 / 12  * 750 / 620)
            Math.round(service.standardizePrice("3 L - Jeroboam (Champagne/Burgundy)", 100.0)) == Math.round(100.0 * 750 / 3000)
            Math.round(service.standardizePrice("12 Bottle Case - Double Magnums", 100.0)) == Math.round(100.0 / 12 * 750 / 3000)
    }

    def "test filterByMedianPrice case 1"() {
        List<RollupOutput> list = [
                new RollupOutput(price: 1),
                new RollupOutput(price: 2),
                new RollupOutput(price: 3),
                new RollupOutput(price: 3),
                new RollupOutput(price: 4),
                new RollupOutput(price: 5),
                new RollupOutput(price: 7),
        ]
        List<RollupOutput> filtered = service.filterByMedianPrice(list)
        expect: "filter price case 1 by median correctly"
            // median = 3
            // min = 1.5
            // max = 6
            // expected to be filtered list[0], list[6]
            filtered == [ list[1], list[2], list[3], list[4], list[5] ]

    }

    def "test filterByMedianPrice case 2"() {
        List<RollupOutput> list = [
                new RollupOutput(price: 5),
                new RollupOutput(price: 2.5),
                new RollupOutput(price: 8.1),
                new RollupOutput(price: 3),
                new RollupOutput(price: 6),
                new RollupOutput(price: 1.7),
        ]
        List<RollupOutput> filtered = service.filterByMedianPrice(list)
        expect: "filter price case 1 by median correctly"
        // median = 4
        // min = 2
        // max = 8
        // expected to be filtered list[2], list[5]
        filtered == [ list[0], list[1], list[3], list[4] ]

    }

    def "test filterByMedianPricePerWineId case 1"() {
        List<RollupOutput> list = [
            new RollupOutput(wineId: 'A', price: 1),
            new RollupOutput(wineId: 'A', price: 2),
            new RollupOutput(wineId: 'A', price: 3),
            new RollupOutput(wineId: 'A', price: 3),
            new RollupOutput(wineId: 'A', price: 4),
            new RollupOutput(wineId: 'A', price: 5),
            new RollupOutput(wineId: 'A', price: 7),
            new RollupOutput(wineId: 'B', price: 5),
            new RollupOutput(wineId: 'B', price: 2.5),
            new RollupOutput(wineId: 'B', price: 8.1),
            new RollupOutput(wineId: 'B', price: 3),
            new RollupOutput(wineId: 'B', price: 6),
            new RollupOutput(wineId: 'B', price: 1.7),
        ]
        List<RollupOutput> filtered = service.filterByMedianPricePerWineId(list)
        expect: "filter price case 1 by median per wineId correctly"
            // wineId A
            // median = 3
            // min = 1.5
            // max = 6
            // expected to be filtered list[0], list[6]

            // wineId B
            // median = 4
            // min = 2
            // max = 8
            // expected to be filtered list[9], list[12]
            filtered == [ list[1], list[2], list[3], list[4], list[5], list[7], list[8], list[10], list[11] ]
    }

}
