package wine.db

import grails.test.mixin.*
import spock.lang.*

@TestFor(WineDbController)
@Mock(WineDb)
class WineDbControllerSpec extends Specification {

    def populateValidParams(params) {
        assert params != null

        // TODO: Populate valid properties like...
        //params["name"] = 'someValidName'
        assert false, "TODO: Provide a populateValidParams() implementation for this generated test suite"
    }

    void "Test the index action returns the correct model"() {

        when:"The index action is executed"
            controller.index()

        then:"The model is correct"
            !model.wineDbList
            model.wineDbCount == 0
    }

    void "Test the create action returns the correct model"() {
        when:"The create action is executed"
            controller.create()

        then:"The model is correctly created"
            model.wineDb!= null
    }

    void "Test the save action correctly persists an instance"() {

        when:"The save action is executed with an invalid instance"
            request.contentType = FORM_CONTENT_TYPE
            request.method = 'POST'
            def wineDb = new WineDb()
            wineDb.validate()
            controller.save(wineDb)

        then:"The create view is rendered again with the correct model"
            model.wineDb!= null
            view == 'create'

        when:"The save action is executed with a valid instance"
            response.reset()
            populateValidParams(params)
            wineDb = new WineDb(params)

            controller.save(wineDb)

        then:"A redirect is issued to the show action"
            response.redirectedUrl == '/wineDb/show/1'
            controller.flash.message != null
            WineDb.count() == 1
    }

    void "Test that the show action returns the correct model"() {
        when:"The show action is executed with a null domain"
            controller.show(null)

        then:"A 404 error is returned"
            response.status == 404

        when:"A domain instance is passed to the show action"
            populateValidParams(params)
            def wineDb = new WineDb(params)
            controller.show(wineDb)

        then:"A model is populated containing the domain instance"
            model.wineDb == wineDb
    }

    void "Test that the edit action returns the correct model"() {
        when:"The edit action is executed with a null domain"
            controller.edit(null)

        then:"A 404 error is returned"
            response.status == 404

        when:"A domain instance is passed to the edit action"
            populateValidParams(params)
            def wineDb = new WineDb(params)
            controller.edit(wineDb)

        then:"A model is populated containing the domain instance"
            model.wineDb == wineDb
    }

    void "Test the update action performs an update on a valid domain instance"() {
        when:"Update is called for a domain instance that doesn't exist"
            request.contentType = FORM_CONTENT_TYPE
            request.method = 'PUT'
            controller.update(null)

        then:"A 404 error is returned"
            response.redirectedUrl == '/wineDb/index'
            flash.message != null

        when:"An invalid domain instance is passed to the update action"
            response.reset()
            def wineDb = new WineDb()
            wineDb.validate()
            controller.update(wineDb)

        then:"The edit view is rendered again with the invalid instance"
            view == 'edit'
            model.wineDb == wineDb

        when:"A valid domain instance is passed to the update action"
            response.reset()
            populateValidParams(params)
            wineDb = new WineDb(params).save(flush: true)
            controller.update(wineDb)

        then:"A redirect is issued to the show action"
            wineDb != null
            response.redirectedUrl == "/wineDb/show/$wineDb.id"
            flash.message != null
    }

    void "Test that the delete action deletes an instance if it exists"() {
        when:"The delete action is called for a null instance"
            request.contentType = FORM_CONTENT_TYPE
            request.method = 'DELETE'
            controller.delete(null)

        then:"A 404 is returned"
            response.redirectedUrl == '/wineDb/index'
            flash.message != null

        when:"A domain instance is created"
            response.reset()
            populateValidParams(params)
            def wineDb = new WineDb(params).save(flush: true)

        then:"It exists"
            WineDb.count() == 1

        when:"The domain instance is passed to the delete action"
            controller.delete(wineDb)

        then:"The instance is deleted"
            WineDb.count() == 0
            response.redirectedUrl == '/wineDb/index'
            flash.message != null
    }
}
